@@ -11,17 +11,20 @@
 prefix=`dirname $(readlink -f $0 || echo $0)`
 
 # Check whether or not to use the bundled JDK
-if [ -d "${prefix}/jdk-11" ]; then
+if [ -d "${prefix}/jdk-17" ]; then
     echo echo "Using bundled JDK."
-    JAVA_HOME="${prefix}/jdk-11"
+    JAVA_HOME="${prefix}/jdk-17"
     PATH=$JAVA_HOME/bin:$PATH
 else
-    echo "Using system JDK."
+    echo "Using system JDK. IGV requires Java 17."
 fi
 
+# Report on Java version
+java -version
+
 # Check if there is a user-specified Java arguments file
 if [ -e "$HOME/.igv/java_arguments" ]; then
-    java -showversion --module-path="${prefix}/lib" -Xmx8g \
+    java --module-path="${prefix}/lib" -Xmx8g \
         @"${prefix}/igv.args" \
         -Dapple.laf.useScreenMenuBar=true \
         -Djava.net.preferIPv4Stack=true \
@@ -29,10 +32,10 @@ if [ -e "$HOME/.igv/java_arguments" ]; then
         @"$HOME/.igv/java_arguments" \
         --module=org.igv/org.broad.igv.ui.Main "$@"
 else
-    java -showversion --module-path="${prefix}/lib" -Xmx8g \
+    java --module-path="${prefix}/lib" -Xmx8g \
         @"${prefix}/igv.args" \
         -Dapple.laf.useScreenMenuBar=true \
         -Djava.net.preferIPv4Stack=true \
         -Djava.net.useSystemProxies=true \
         --module=org.igv/org.broad.igv.ui.Main "$@"
-fi
+fi
\ No newline at end of file
