@@ -35,6 +35,7 @@ import org.broad.igv.feature.Locus;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.feature.Strand;
+import org.broad.igv.feature.genome.GenomeListItem;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.logging.LogManager;
 import org.broad.igv.logging.Logger;
@@ -50,6 +51,7 @@ import org.broad.igv.track.*;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.action.OverlayTracksMenuAction;
 import org.broad.igv.ui.color.ColorUtilities;
+import org.broad.igv.ui.commandbar.GenomeListManager;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.ui.util.SnapshotUtilities;
@@ -121,6 +123,9 @@ public class CommandExecutor {
             } else if (cmd.equalsIgnoreCase("scrolltotrack") || cmd.equalsIgnoreCase("gototrack")) {
                 boolean res = this.igv.scrollToTrack(StringUtils.stripQuotes(param1));
                 result = res ? "OK" : String.format("Error: Track %s not found", param1);
+            } else if (cmd.equalsIgnoreCase("scrolltotop") ) {
+                this.igv.scrollToTop();
+                result = "OK";
             } else if (cmd.equalsIgnoreCase("snapshotdirectory")) {
                 result = setSnapshotDirectory(param1);
             } else if (cmd.equalsIgnoreCase("snapshot")) {
@@ -132,6 +137,20 @@ public class CommandExecutor {
                 result = load(param1, args.size() > 2 ? args.subList(2, args.size()) : null);
             } else if (cmd.equalsIgnoreCase("genome") && args.size() > 1) {
                 result = genome(param1);
+            } else if (cmd.equalsIgnoreCase("currentGenomePath")) {
+                result = "";
+                if (GenomeManager.getInstance().getCurrentGenome() == null) {
+                    return result;
+                }
+                String id = GenomeManager.getInstance().getCurrentGenome().getId();
+                if (id != null)
+                {
+                    GenomeListItem item = GenomeListManager.getInstance().getGenomeListItem(id);
+                    if (item != null) {
+                        result = item.getPath();
+                    }
+                }
+                return result;
             } else if (cmd.equalsIgnoreCase("new") || cmd.equalsIgnoreCase("reset") || cmd.equalsIgnoreCase("clear")) {
                 igv.newSession();
             } else if (cmd.equalsIgnoreCase("region")) {
@@ -624,7 +643,7 @@ public class CommandExecutor {
 
         String genomePath = resolveFileReference(genomeID);
         try {
-            GenomeManager.getInstance().loadGenome(genomePath, null);
+            GenomeManager.getInstance().loadGenome(genomePath);
         } catch (IOException e) {
             result = "ERROR: Could not load genome: " + genomeID;
             MessageUtils.showMessage(result);
@@ -1082,7 +1101,7 @@ public class CommandExecutor {
 
             boolean invertSort = "reverse".equalsIgnoreCase(reverseString);
 
-            igv.sortAlignmentTracks(getAlignmentSortOption(sortArg), location, tag, invertSort);
+            igv.sortAlignmentTracks(getAlignmentSortOption(sortArg), location, tag, invertSort, null);
             return "OK";
         }
     }
@@ -1091,7 +1110,7 @@ public class CommandExecutor {
     private String group(String groupArg, String tagArg) {
         final AlignmentTrack.GroupOption groupOption = getAlignmentGroupOption(groupArg);
         Range r = null;
-        if (groupOption == AlignmentTrack.GroupOption.BASE_AT_POS) {
+        if (groupOption == AlignmentTrack.GroupOption.BASE_AT_POS || groupOption == AlignmentTrack.GroupOption.INSERTION_AT_POS) {
             if (tagArg == null) {
                 return "Error: position is required";
             } else {
@@ -1242,7 +1261,11 @@ public class CommandExecutor {
             return AlignmentTrack.GroupOption.LIBRARY;
         } else if (str.equalsIgnoreCase("readGroup") || str.equalsIgnoreCase("read_group")) {
             return AlignmentTrack.GroupOption.READ_GROUP;
-        } else {
+        } else if (str.equalsIgnoreCase("base")) {
+            return AlignmentTrack.GroupOption.BASE_AT_POS;
+        }else if (str.equalsIgnoreCase("insertion")) {
+            return AlignmentTrack.GroupOption.INSERTION_AT_POS;
+        }else {
             try {
                 return AlignmentTrack.GroupOption.valueOf(str.toUpperCase());
             } catch (IllegalArgumentException e) {
