@@ -37,7 +37,6 @@ import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.ui.util.UIUtilities;
-import org.broad.igv.util.HttpUtils;
 import org.broad.igv.util.StringUtils;
 
 import java.awt.*;
@@ -78,7 +77,7 @@ public class CommandListener implements Runnable {
     public static Set<String> indexParams;
 
     static {
-        String[] fps = new String[]{"file", "bigDataURL", "sessionURL", "dataURL"};
+        String[] fps = new String[]{"file", "bigDataURL", "sessionURL", "dataURL", "hubURL"};
         fileParams = new LinkedHashSet<String>(Arrays.asList(fps));
         fileParams = Collections.unmodifiableSet(fileParams);
 
@@ -182,7 +181,11 @@ public class CommandListener implements Runnable {
 
                 String cmd = inputLine;
                 if (!cmd.contains("/oauthCallback")) {
-                    log.info(cmd);
+                    if (cmd.startsWith("SetAccessToken")) {
+                        log.info(cmd.substring(0, 14) + " *****");
+                    } else {
+                        log.info(cmd);
+                    }
                 }
 
                 boolean isHTTP = cmd.startsWith("OPTIONS") || cmd.startsWith("HEAD") || cmd.startsWith("GET");
@@ -220,15 +223,13 @@ public class CommandListener implements Runnable {
                         } else {
 
                             if (command != null) {
-
                                 // Detect  oauth callback
                                 if (command.equals("/oauthCallback")) {
-
                                     OAuthProvider provider = OAuthUtils.getInstance().getProviderForState(params.get("state"));
                                     if (params.containsKey("error")) {
                                         sendTextResponse(out, "Error authorizing IGV: " + params.get("error"));
                                     } else if (params.containsKey("code")) {
-                                        provider.setAuthorizationCode(params.get("code"));
+                                        provider.fetchAccessToken(params.get("code"));
                                         sendTextResponse(out, "Authorization successful.  You may close this tab.");
                                     } else if (params.containsKey("token")) {
                                         // Very doubtful this is ever called -- its not a normal OAuth flow
@@ -238,12 +239,11 @@ public class CommandListener implements Runnable {
                                     } else {
                                         sendTextResponse(out, "Unsuccessful authorization response: " + inputLine);
                                     }
-
-
                                     if (PreferencesManager.getPreferences().getAsBoolean(Constants.PORT_ENABLED) == false) {
                                         // Turn off port
                                         halt();
                                     }
+
                                 } else {
                                     // Process the request.
                                     result = processGet(command, params, cmdExe); // Send no response if result is "OK".
@@ -312,6 +312,7 @@ public class CommandListener implements Runnable {
     private static final String CONNECTION_CLOSE = "Connection: close";
     private static final String NO_CACHE = "Cache-Control: no-cache, no-store";
     private static final String ACCESS_CONTROL_ALLOW_ORIGIN = "Access-Control-Allow-Origin: *";
+    private static final String ACCESS_CONTROL_ALLOW_HEADERS = "Access-Control-Allow-Headers: access-control-allow-origin";
 
     private void sendTextResponse(PrintWriter out, String result) {
         sendHTTPResponse(out, result, "text/html", "GET");
@@ -338,6 +339,8 @@ public class CommandListener implements Runnable {
                 out.print(result);
                 out.print(CRLF);
             }
+        } else {
+            out.print(CRLF);
         }
         out.close();
     }
@@ -348,6 +351,8 @@ public class CommandListener implements Runnable {
         out.print(CRLF);
         out.print(ACCESS_CONTROL_ALLOW_ORIGIN);
         out.print(CRLF);
+        out.print(ACCESS_CONTROL_ALLOW_HEADERS);
+        out.print(CRLF);
         out.println("Access-Control-Allow-Methods: HEAD, GET, OPTIONS");
         out.print(CRLF);
 
@@ -369,8 +374,11 @@ public class CommandListener implements Runnable {
         mainFrame.setAlwaysOnTop(true);
         mainFrame.setAlwaysOnTop(false);
 
-
-        if (command.equals("/load")) {
+        if (command.equals("/")) {
+            if (params.containsKey("hubURL")) {
+                GenomeManager.getInstance().loadGenome(URLDecoder.decode(params.get("hubURL")));
+            }
+        } else if (command.equals("/load")) {
             String file = null;
             for (String fp : fileParams) {
                 file = params.get(fp);
@@ -425,7 +433,11 @@ public class CommandListener implements Runnable {
             IGV.getInstance().goToLocus(locus);
         } else if (command.equals("/execute")) {
             String param = StringUtils.decodeURL(params.get("command"));
-            return cmdExe.execute(param);
+            result = cmdExe.execute(param);
+        } else if (command.equals("/ping")) {
+            result = "OK";
+        } else if (command.equals("/version")) {
+            result = "3.0";   // TODO, use actual version
         } else {
             return ("ERROR Unknown command: " + command);
         }
