@@ -1,13 +1,14 @@
 package org.broad.igv.feature.genome.load;
 
 import htsjdk.tribble.Feature;
-import org.broad.igv.logging.*;
 import org.broad.igv.feature.*;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.Sequence;
 import org.broad.igv.feature.genome.fasta.FastaBlockCompressedSequence;
 import org.broad.igv.feature.genome.fasta.FastaIndexedSequence;
 import org.broad.igv.feature.gff.GFFFeatureSource;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.track.FeatureCollectionSource;
 import org.broad.igv.track.FeatureTrack;
 import org.broad.igv.track.TrackProperties;
@@ -17,9 +18,12 @@ import org.broad.igv.util.ResourceLocator;
 
 import java.awt.*;
 import java.io.*;
-import java.util.Collection;
 import java.util.List;
 
+
+/**
+ * Load for the legacy ".genome" format.   This format id deprecated.
+ */
 public class DotGenomeLoader extends GenomeLoader {
 
     private static Logger log = LogManager.getLogger(DotGenomeLoader.class);
@@ -37,7 +41,7 @@ public class DotGenomeLoader extends GenomeLoader {
      * @param geneTrackName
      */
     private static FeatureTrack createGeneTrack(Genome genome, BufferedReader reader, String geneFileName, String geneTrackName,
-                                               String annotationURL) {
+                                                String annotationURL) {
 
         FeatureDB.clearFeatures();
         FeatureTrack geneFeatureTrack = null;
@@ -83,7 +87,7 @@ public class DotGenomeLoader extends GenomeLoader {
     @Override
     public Genome loadGenome() throws IOException {
 
-        Genome newGenome;
+        GenomeConfig config = new GenomeConfig();
 
         GenomeDescriptor genomeDescriptor = GenomeDescriptor.parseGenomeArchiveFile(archiveFile);
         final String id = genomeDescriptor.getId();
@@ -93,7 +97,6 @@ public class DotGenomeLoader extends GenomeLoader {
                 genomeDescriptor.getSequencePath();
 
         Sequence sequence;
-        boolean chromosOrdered = false;
         if (sequencePath == null) {
             sequence = null;
         } else {
@@ -102,17 +105,19 @@ public class DotGenomeLoader extends GenomeLoader {
             } else {
                 sequence = new FastaIndexedSequence(sequencePath);
             }
-            chromosOrdered = true;
         }
 
-        newGenome = new Genome(id, displayName, sequence, chromosOrdered);
+        config.id = id;
+        config.name = displayName;
+        config.sequence = sequence;
 
         if (genomeDescriptor.hasCytobands()) {
             InputStream cytobandStream = null;
             try {
                 cytobandStream = genomeDescriptor.getCytoBandStream();
                 BufferedReader reader = new BufferedReader(new InputStreamReader(cytobandStream));
-                newGenome.setCytobands(CytoBandFileParser.loadData(reader));
+                config.cytobands = CytoBandFileParser.loadData(reader);
+
             } catch (IOException ex) {
                 log.error("Error loading cytoband file", ex);
                 throw new RuntimeException("Error loading cytoband file" + genomeDescriptor.cytoBandFileName);
@@ -121,16 +126,12 @@ public class DotGenomeLoader extends GenomeLoader {
             }
         }
 
-
         InputStream aliasStream = null;
         try {
             aliasStream = genomeDescriptor.getChrAliasStream();
             if (aliasStream != null) {
                 BufferedReader reader = new BufferedReader(new InputStreamReader(aliasStream));
-                Collection<Collection<String>> aliases = loadChrAliases(reader);
-                if (aliases != null) {
-                    newGenome.addChrAliases(aliases);
-                }
+                config.chromAliases =  ChromAliasParser.loadChrAliases  (reader);
             }
         } catch (IOException e) {
             // We don't want to bomb if the alias load fails.  Just log it and proceed.
@@ -139,6 +140,8 @@ public class DotGenomeLoader extends GenomeLoader {
             closeSilently(aliasStream);
         }
 
+        Genome   newGenome = new Genome(config);
+
 
         String geneFileName = genomeDescriptor.getGeneFileName();
         InputStream geneStream = null;
@@ -156,7 +159,6 @@ public class DotGenomeLoader extends GenomeLoader {
                     FeatureTrack geneFeatureTrack = createGeneTrack(newGenome, reader,
                             geneFileName, genomeDescriptor.getGeneTrackName(),
                             genomeDescriptor.getUrl());
-
                     newGenome.setGeneTrack(geneFeatureTrack);
                 }
             } finally {
