@@ -25,6 +25,10 @@
 
 package org.broad.igv.sam;
 
+import org.broad.igv.event.DataLoadedEvent;
+import org.broad.igv.feature.genome.ChromAlias;
+import org.broad.igv.feature.genome.ChromAliasManager;
+import org.broad.igv.event.IGVEvent;
 import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.event.IGVEventBus;
@@ -33,8 +37,11 @@ import org.broad.igv.event.RefreshEvent;
 import org.broad.igv.feature.Chromosome;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
+import org.broad.igv.sam.mods.BaseModificationKey;
+import org.broad.igv.sam.mods.BaseModificationSet;
 import org.broad.igv.sam.reader.AlignmentReader;
 import org.broad.igv.sam.reader.AlignmentReaderFactory;
 import org.broad.igv.track.Track;
@@ -44,6 +51,7 @@ import org.broad.igv.util.ResourceLocator;
 
 import java.io.IOException;
 import java.util.*;
+import java.util.stream.Collectors;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -55,20 +63,26 @@ import static org.broad.igv.prefs.Constants.*;
 public class AlignmentDataManager implements IGVEventObserver {
 
     private static Logger log = LogManager.getLogger(AlignmentDataManager.class);
+
+    private final Genome genome;
+    private ChromAliasManager chromAliasManager;
     private final AlignmentReader reader;
-
-
     private AlignmentTrack alignmentTrack;
     private CoverageTrack coverageTrack;
     private Set<Track> subscribedTracks;
-
     private List<AlignmentInterval> intervalCache;
     private ResourceLocator locator;
-    private HashMap<String, String> chrMappings = new HashMap();
+
+    private Set<String> sequenceNames;
     private AlignmentTileLoader loader;
     private Map<String, PEStats> peStats;
-    private SpliceJunctionHelper.LoadOptions loadOptions;
+    private Set<BaseModificationKey> allBaseModificationKeys = new HashSet<>();
+    private Set<String> simplexBaseModfications = new HashSet<>();
+
     private Range currentlyLoading;
+    private AlignmentTrack.ExperimentType inferredType;
+
+    Map<String, String> chrAliasCache = new HashMap<>();
 
     public AlignmentDataManager(ResourceLocator locator, Genome genome) throws IOException {
         this.locator = locator;
@@ -77,17 +91,19 @@ public class AlignmentDataManager implements IGVEventObserver {
         // mechanism to refresh expired presigned URLs.
         reader = AlignmentReaderFactory.getReader(locator);
         loader = new AlignmentTileLoader(reader);
+        this.inferType();
         peStats = new HashMap();
-        initLoadOptions();
-        initChrMap(genome);
+        this.genome = genome;
         intervalCache = Collections.synchronizedList(new ArrayList<>());
         subscribedTracks = Collections.synchronizedSet(new HashSet<>());
+        sequenceNames = new HashSet(getLoader().getSequenceNames());
+        chromAliasManager = new ChromAliasManager(sequenceNames, genome);
 
         IGVEventBus.getInstance().subscribe(FrameManager.ChangeEvent.class, this);
         IGVEventBus.getInstance().subscribe(RefreshEvent.class, this);
     }
 
-    public void receiveEvent(Object event) {
+    public void receiveEvent(IGVEvent event) {
         if (event instanceof FrameManager.ChangeEvent) {
             trimCache();
         } else if (event instanceof RefreshEvent) {
@@ -110,76 +126,6 @@ public class AlignmentDataManager implements IGVEventObserver {
         }
     }
 
-    void initLoadOptions() {
-        this.loadOptions = new SpliceJunctionHelper.LoadOptions();
-    }
-
-    /**
-     * Create an alias -> chromosome lookup map.  Enables loading BAM files that use alternative names for chromosomes
-     * (e.g. 1 -> chr1,  etc).
-     */
-    private void initChrMap(Genome genome) throws IOException {
-
-        if (genome != null) {
-
-            // Build a chr size -> name lookup table.   We will assume sizes are unique.  This will be used if no alias
-            // is defined for a sequence.
-            Map<Long, String> inverseDict = null;
-            Map<String, Long> sequenceDictionary = getLoader().getSequenceDictionary();
-
-            if (sequenceDictionary != null) {
-
-                Set<Long> nonUnique = new HashSet<>();
-                Set<Long> seen = new HashSet<>();
-                // First find sequences whose size are not unique,  we'll filter these
-                for (Long size : sequenceDictionary.values()) {
-                    if (seen.contains(size)) {
-                        nonUnique.add(size);
-                    } else {
-                        seen.add(size);
-                    }
-                }
-
-                inverseDict = new HashMap<>();
-
-                for (Chromosome chromosome : genome.getChromosomes()) {
-
-                    Long size = (long) chromosome.getLength();
-                    if (!nonUnique.contains(size)) {
-                        if (inverseDict.containsKey(size)) {
-                            inverseDict.remove(size);
-                            nonUnique.add(size);
-                        } else {
-                            inverseDict.put(size, chromosome.getName());
-                        }
-                    }
-                }
-            }
-
-
-            List<String> seqNames = getLoader().getSequenceNames();
-            if (seqNames != null) {
-                for (String seq : seqNames) {
-
-                    if (genome.isKnownChr(seq)) {
-                        String chr = genome.getCanonicalChrName(seq);
-                        chrMappings.put(chr, seq);
-                    } else if (sequenceDictionary != null) {
-                        Long size = sequenceDictionary.get(seq);
-                        String chr = inverseDict.get(size);
-                        if (chr != null) {
-                            chrMappings.put(chr, seq);
-                        }
-                    }
-                }
-            }
-        }
-    }
-
-    public boolean hasMatchingSequences() {
-        return chrMappings.size() > 0;
-    }
-
     public AlignmentTileLoader getLoader() {
         return loader;
     }
@@ -192,6 +138,13 @@ public class AlignmentDataManager implements IGVEventObserver {
         return peStats;
     }
 
+    /**
+     * Return all base modfications seen in loaded alignments
+     */
+    public Set<BaseModificationKey> getAllBaseModificationKeys() {
+        return allBaseModificationKeys;
+    }
+
     public boolean isPairedEnd() {
         return getLoader().isPairedEnd();
     }
@@ -216,14 +169,11 @@ public class AlignmentDataManager implements IGVEventObserver {
         return coverageTrack;
     }
 
-    public double getMinVisibleScale() {
-        return getVisibilityWindow() / 700;
-    }
-
     public double getVisibilityWindow() {
         return getPreferences().getAsFloat(SAM_MAX_VISIBLE_RANGE) * 1000;
     }
 
+
     private IGVPreferences getPreferences() {
         String category = NULL_CATEGORY;
         AlignmentTrack.ExperimentType experimentType = getExperimentType();
@@ -246,21 +196,18 @@ public class AlignmentDataManager implements IGVEventObserver {
         return getLoader().getSequenceNames();
     }
 
-    public Map<String, Long> getSequenceDictionary() {
-        return getLoader().getSequenceDictionary();
-    }
-
     public AlignmentInterval getLoadedInterval(ReferenceFrame frame) {
         return getLoadedInterval(frame, false);
     }
 
     public AlignmentInterval getLoadedInterval(ReferenceFrame frame, boolean includeOverlaps) {
+        // Search for interval completely containing reference frame region
         for (AlignmentInterval interval : intervalCache) {
             if (interval.contains(frame.getCurrentRange())) {
                 return interval;
             }
         }
-        // No contains, look for overlap
+        // No interval contains entire region of frame, look for intervael with at least some overlap
         if (includeOverlaps) {
             for (AlignmentInterval interval : intervalCache) {
                 if (interval.overlaps(frame.getCurrentRange())) {
@@ -301,7 +248,7 @@ public class AlignmentDataManager implements IGVEventObserver {
                      AlignmentTrack.RenderOptions renderOptions,
                      boolean expandEnds) {
 
-        if (frame.getChrName().equals(Globals.CHR_ALL) || frame.getScale() > getMinVisibleScale())
+        if (frame.getChrName().equals(Globals.CHR_ALL) || (frame.getEnd() - frame.getOrigin()) > getVisibilityWindow())
             return; // should not happen
 
         if (isLoaded(frame)) {
@@ -330,7 +277,7 @@ public class AlignmentDataManager implements IGVEventObserver {
             if (expandEnds) {
                 adjustedStart = Math.max(0, Math.min(start, center - expand));
                 adjustedEnd = Math.max(end, center + expand);
-                if(adjustedEnd < 0) {
+                if (adjustedEnd < 0) {
                     adjustedEnd = Integer.MAX_VALUE;  // Overflow
                 }
             }
@@ -341,13 +288,15 @@ public class AlignmentDataManager implements IGVEventObserver {
 
             intervalCache.add(loadedInterval);
 
-            packAlignments(renderOptions);
+            loadedInterval.packAlignments(renderOptions);
 
         } finally {
             currentlyLoading = null;
         }
 
-        //  IGVEventBus.getInstance().post(new DataLoadedEvent(frame));
+
+        IGVEventBus.getInstance().post(new DataLoadedEvent(frame));
+
 
     }
 
@@ -379,40 +328,80 @@ public class AlignmentDataManager implements IGVEventObserver {
 
     AlignmentInterval loadInterval(String chr, int start, int end, AlignmentTrack.RenderOptions renderOptions) {
 
-        String sequence = chrMappings.containsKey(chr) ? chrMappings.get(chr) : chr;
+        final String seqName;
+        if (sequenceNames.contains(chr)) {
+            seqName = chr;
+        } else if (chrAliasCache.containsKey(chr)) {
+            seqName = chrAliasCache.get(chr);
+        } else {
+            seqName = chromAliasManager.getAliasName(chr);
+            chrAliasCache.put(chr, seqName);   // This might be null, but record to prevent further attempts
+        }
+
+        if (seqName == null) {
+            // No alignments with this chr name -- return empty interval
+            return new AlignmentInterval(chr, start, end);
+        }
 
         DownsampleOptions downsampleOptions = new DownsampleOptions();
-
         final AlignmentTrack.BisulfiteContext bisulfiteContext =
                 renderOptions != null ? renderOptions.bisulfiteContext : null;
+        SpliceJunctionHelper spliceJunctionHelper = new SpliceJunctionHelper();
 
-        SpliceJunctionHelper spliceJunctionHelper = new SpliceJunctionHelper(this.loadOptions);
-
-        AlignmentTileLoader.AlignmentTile t = getLoader().loadTile(sequence, start, end, spliceJunctionHelper,
+        AlignmentTileLoader.AlignmentTile t = getLoader().loadTile(seqName, start, end, spliceJunctionHelper,
                 downsampleOptions, peStats, bisulfiteContext, renderOptions);
-      List<Alignment> alignments = t.getAlignments();
+        List<Alignment> alignments = t.getAlignments();
         List<DownsampledInterval> downsampledIntervals = t.getDownsampledIntervals();
+        this.updateBaseModfications(alignments);
         return new AlignmentInterval(chr, start, end, alignments, t.getCounts(), spliceJunctionHelper, downsampledIntervals);
     }
 
+    private void updateBaseModfications(List<Alignment> alignments) {
+
+        for (Alignment a : alignments) {
+            List<BaseModificationSet> bmSets = a.getBaseModificationSets();
+            if (bmSets != null) {
+                for (BaseModificationSet bms : bmSets) {
+                    allBaseModificationKeys.add(BaseModificationKey.getKey(bms.getBase(), bms.getStrand(), bms.getModification()));
+                }
+            }
+        }
+
+        // Search for simplex modifications (single strand read, e.g. C+m with no G-m)
+        Set<String> minusStranMods = allBaseModificationKeys.stream()
+                .filter(key -> key.getStrand() == '-')
+                .map(key -> key.getModification())
+                .collect(Collectors.toSet());
+        for (BaseModificationKey key : allBaseModificationKeys) {
+            if (key.getStrand() == '+' && !minusStranMods.contains(key.getModification()))
+                simplexBaseModfications.add(key.getModification());
+            simplexBaseModfications.add("NONE_" + key.getCanonicalBase());  // Mix of simplex & duplex keys for same base not supported.
+        }
+    }
+
     public AlignmentTrack.ExperimentType inferType() {
+        if (this.inferredType != null) {
+            return this.inferredType;
+        }
+
         ReadStats readStats = new ReadStats();
         List<Alignment> sample = AlignmentUtils.firstAlignments(reader, 100);
-        for(Alignment a : sample) {
+        for (Alignment a : sample) {
             readStats.addAlignment(a);
         }
-        AlignmentTrack.ExperimentType type = readStats.inferType();
+        inferredType = readStats.inferType();
 
-        if(type == AlignmentTrack.ExperimentType.THIRD_GEN) {
-            return type;
+        if (inferredType == AlignmentTrack.ExperimentType.THIRD_GEN) {
+            return inferredType;
         } else {
             // Get a larger sample to distinguish RNA-Seq
             readStats = new ReadStats();
             sample = AlignmentUtils.firstAlignments(reader, 2000);
-            for(Alignment a : sample) {
+            for (Alignment a : sample) {
                 readStats.addAlignment(a);
             }
-            return readStats.inferType();
+            inferredType = readStats.inferType();
+            return inferredType;
         }
     }
 
@@ -420,10 +409,6 @@ public class AlignmentDataManager implements IGVEventObserver {
         return alignmentTrack == null ? null : alignmentTrack.getExperimentType();
     }
 
-    private void setExperimentType(AlignmentTrack.ExperimentType type) {
-        if (alignmentTrack != null) alignmentTrack.setExperimentType(type);
-    }
-
 
     public PackedAlignments getGroups(AlignmentInterval interval, AlignmentTrack.RenderOptions renderOptions) {
         //AlignmentInterval interval = getLoadedInterval(context.getReferenceFrame());
@@ -520,37 +505,22 @@ public class AlignmentDataManager implements IGVEventObserver {
         }
     }
 
-    public SpliceJunctionHelper.LoadOptions getSpliceJunctionLoadOptions() {
-        return loadOptions;
-    }
-
-    public void setMinJunctionCoverage(int minJunctionCoverage) {
-        this.loadOptions = new SpliceJunctionHelper.LoadOptions(minJunctionCoverage, this.loadOptions.minReadFlankingWidth);
-        for (AlignmentInterval interval : intervalCache) {
-            interval.getSpliceJunctionHelper().setLoadOptions(this.loadOptions);
-        }
-    }
-
     public void alleleThresholdChanged() {
         coverageTrack.setSnpThreshold(PreferencesManager.getPreferences().getAsFloat(SAM_ALLELE_THRESHOLD));
     }
 
-    public boolean isTenX() {
-        return getLoader().isTenX();
-    }
-
     public boolean isPhased() {
         return getLoader().isPhased();
     }
 
-    public boolean isMoleculo() {
-        return getLoader().isMoleculo();
-    }
-
     public Collection<AlignmentInterval> getLoadedIntervals() {
         return intervalCache;
     }
 
+    public Set<String> getSimplexBaseModifications() {
+        return simplexBaseModfications;
+    }
+
     public static class DownsampleOptions {
         private boolean downsample;
         private int sampleWindowSize;
@@ -587,71 +557,5 @@ public class AlignmentDataManager implements IGVEventObserver {
 
     }
 
-    static class IntervalCache {
-
-        private int maxSize;
-        ArrayList<AlignmentInterval> intervals;
-
-        public IntervalCache() {
-            this(1);
-        }
-
-        public IntervalCache(int ms) {
-            this.maxSize = Math.max(1, ms);
-            intervals = new ArrayList<>(maxSize);
-        }
-
-        void setMaxSize(int ms, List<ReferenceFrame> frames) {
-            this.maxSize = Math.max(1, ms);
-            if (intervals.size() > maxSize) {
-                // Reduce size.  Try to keep intervals that cover frame ranges.  This involves a linear search
-                // of potentially (intervals.size X frames.size) elements.  Don't attempt if this number is too large
-                if (frames.size() * intervals.size() < 25) {
-                    ArrayList<AlignmentInterval> tmp = new ArrayList<>(maxSize);
-                    for (AlignmentInterval interval : intervals) {
-                        if (tmp.size() == maxSize) break;
-                        for (ReferenceFrame frame : frames) {
-                            Range range = frame.getCurrentRange();
-                            if (interval.contains(range.getChr(), range.getStart(), range.getEnd())) {
-                                tmp.add(interval);
-                                break;
-                            }
-                        }
-                    }
-                    intervals = tmp;
-                } else {
-                    intervals = new ArrayList(intervals.subList(0, maxSize));
-                    intervals.trimToSize();
-                }
-            }
-        }
-
-        public void add(AlignmentInterval interval) {
-            if (intervals.size() >= maxSize) {
-                intervals.remove(0);
-            }
-            intervals.add(interval);
-        }
-
-        public AlignmentInterval getIntervalForRange(Range range) {
-
-            for (AlignmentInterval interval : intervals) {
-                if (interval.contains(range.getChr(), range.getStart(), range.getEnd())) {
-                    return interval;
-                }
-            }
-
-            return null;
-
-        }
-
-        public Collection<AlignmentInterval> values() {
-            return intervals;
-        }
-
-        public void clear() {
-            intervals.clear();
-        }
-    }
 }
 
