@@ -26,7 +26,9 @@
 package org.broad.igv.sam;
 
 import htsjdk.samtools.SAMFileHeader;
+import htsjdk.samtools.SAMTag;
 import htsjdk.samtools.util.CloseableIterator;
+import org.broad.igv.event.IGVEvent;
 import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.prefs.IGVPreferences;
@@ -60,7 +62,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
 
     static DecimalFormat df = new DecimalFormat("###,###,###");
 
-    private static Set<WeakReference<AlignmentTileLoader>> activeLoaders = Collections.synchronizedSet(new HashSet());
+    private static Set<WeakReference<AlignmentTileLoader>> activeLoaders = Collections.synchronizedSet(new HashSet<>());
 
     /**
      * Flag to mark a corrupt index.  Without this attempted reads will continue in an infinite loop
@@ -297,7 +299,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
             t.finish();
 
             // TODO -- make this optional (on a preference)
-            InsertionManager.getInstance().processAlignments(chr, t.alignments, renderOptions);
+            InsertionManager.getInstance().processAlignments(chr, t.alignments);
 
 
         } catch (java.nio.BufferUnderflowException e) {
@@ -379,7 +381,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
     }
 
     @Override
-    public void receiveEvent(Object event) {
+    public void receiveEvent(IGVEvent event) {
         if (event instanceof StopEvent) {
             cancel = true;
             reader.cancelQuery();
