@@ -28,6 +28,8 @@ package org.broad.igv.sam;
 
 import org.broad.igv.Globals;
 import org.broad.igv.event.AlignmentTrackEvent;
+import org.broad.igv.event.DataLoadedEvent;
+import org.broad.igv.event.IGVEvent;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.feature.FeatureUtils;
@@ -40,6 +42,7 @@ import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.renderer.GraphicUtils;
+import org.broad.igv.sam.mods.BaseModficationFilter;
 import org.broad.igv.session.Persistable;
 import org.broad.igv.track.*;
 import org.broad.igv.ui.FontManager;
@@ -51,6 +54,7 @@ import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.IGVPopupMenu;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.util.MessageUtils;
+import org.broad.igv.ui.util.UIUtilities;
 import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.util.StringUtils;
 import org.broad.igv.util.blat.BlatClient;
@@ -65,6 +69,7 @@ import java.awt.event.MouseEvent;
 import java.awt.geom.Rectangle2D;
 import java.util.List;
 import java.util.*;
+import java.util.function.Consumer;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -74,11 +79,19 @@ import static org.broad.igv.prefs.Constants.*;
 
 public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
 
-    private static Logger log = LogManager.getLogger(AlignmentTrack.class);
+    private static final Logger log = LogManager.getLogger(AlignmentTrack.class);
 
     // Alignment colors
     static final Color DEFAULT_ALIGNMENT_COLOR = new Color(185, 185, 185); //200, 200, 200);
 
+    public static void sortSelectedReadsToTheTop(final Set<String> selectedReadNames) {
+        //copy this in case it changes out from under us
+        Set<String> selectedReadNameCopy = new HashSet<>(selectedReadNames);
+        //Run this on the event thread to make sure it happens after loading begins
+        UIUtilities.invokeOnEventThread(() ->
+                IGV.getInstance().sortAlignmentTracks(SortOption.NONE, null, null, false, selectedReadNameCopy));
+    }
+
     public enum ColorOption {
         INSERT_SIZE,
         READ_STRAND,
@@ -99,8 +112,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         LINK_STRAND,
         YC_TAG,
         BASE_MODIFICATION,
-        BASE_MODIFICATION_5MC,
-        BASE_MODIFICATION_C,
+        BASE_MODIFICATION_2COLOR,
         SMRT_SUBREAD_IPD,
         SMRT_SUBREAD_PW,
         SMRT_CCS_FWD_IPD,
@@ -109,7 +121,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         SMRT_CCS_REV_PW;
 
         public boolean isBaseMod() {
-            return this == BASE_MODIFICATION || this == BASE_MODIFICATION_5MC || this == BASE_MODIFICATION_C;
+            return this == BASE_MODIFICATION || this == BASE_MODIFICATION_2COLOR;
         }
 
         public boolean isSMRTKinetics() {
@@ -128,7 +140,6 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     }
 
 
-
     public enum ShadeAlignmentsOption {
         NONE("none"),
         MAPPING_QUALITY_HIGH("mapping quality high"),
@@ -151,8 +162,10 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         PAIR_ORIENTATION("pair orientation"),
         MATE_CHROMOSOME("chromosome of mate"),
         NONE("none"),
+        CHIMERIC("chimeric"),
         SUPPLEMENTARY("supplementary flag"),
         BASE_AT_POS("base at position"),
+        INSERTION_AT_POS("insertion at position", true),
         MOVIE("movie"),
         ZMW("ZMW"),
         HAPLOTYPE("haplotype"),
@@ -163,9 +176,16 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         MAPPING_QUALITY("mapping quality");
 
         public final String label;
+        public final boolean reverse;
 
         GroupOption(String label) {
             this.label = label;
+            this.reverse = false;
+        }
+
+        GroupOption(String label, boolean reverse) {
+            this.label = label;
+            this.reverse = reverse;
         }
 
     }
@@ -178,9 +198,8 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     private static final int GROUP_MARGIN = 5;
     private static final int TOP_MARGIN = 20;
     private static final int DS_MARGIN_0 = 2;
-    private static final int DOWNAMPLED_ROW_HEIGHT = 3;
+    private static final int DOWNSAMPLED_ROW_HEIGHT = 3;
     private static final int INSERTION_ROW_HEIGHT = 9;
-    private static final int DS_MARGIN_2 = 5;
 
     public enum BisulfiteContext {
         CG("CG", new byte[]{}, new byte[]{'G'}),
@@ -248,9 +267,11 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             return (matchesContext) ? this : null;
         }
 
-        public String getLabel() { return label; }
+        public String getLabel() {
+            return label;
+        }
 
-        BisulfiteContext(String label, byte[] preContext, byte[] postContext){
+        BisulfiteContext(String label, byte[] preContext, byte[] postContext) {
             this.label = label;
             this.preContext = preContext;
             this.postContext = postContext;
@@ -269,11 +290,10 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     private final Genome genome;
     private ExperimentType experimentType;
     private final AlignmentRenderer renderer;
-    private RenderOptions renderOptions;
+    RenderOptions renderOptions;
 
     private boolean removed = false;
     private boolean showGroupLine;
-    private Map<ReferenceFrame, List<InsertionInterval>> insertionIntervalsMap;
     private int expandedHeight = 14;
     private int collapsedHeight = 9;
     private final int maxSquishedHeight = 5;
@@ -286,6 +306,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     private ColorTable readNamePalette;
     private final HashMap<String, Color> selectedReadNames = new HashMap<>();
 
+    private final HashMap<ReferenceFrame, Consumer<ReferenceFrame>> actionToPerformOnFrameLoad = new HashMap<>();
 
     /**
      * Create a new alignment track
@@ -335,16 +356,16 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         spliceJunctionTrack.setHeight(60);
         this.spliceJunctionTrack = spliceJunctionTrack;
 
-        if (renderOptions.colorOption == ColorOption.BISULFITE) {
+        if (renderOptions.getColorOption() == ColorOption.BISULFITE) {
             setExperimentType(ExperimentType.BISULFITE);
         }
         readNamePalette = new PaletteColorTable(ColorUtilities.getDefaultPalette());
-        insertionIntervalsMap = Collections.synchronizedMap(new HashMap<>());
 
         dataManager.setViewAsPairs(prefs.getAsBoolean(SAM_DISPLAY_PAIRED), renderOptions);
 
         IGVEventBus.getInstance().subscribe(FrameManager.ChangeEvent.class, this);
         IGVEventBus.getInstance().subscribe(AlignmentTrackEvent.class, this);
+        IGVEventBus.getInstance().subscribe(DataLoadedEvent.class, this);
     }
 
     public void init() {
@@ -356,33 +377,27 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
 
 
     @Override
-    public void receiveEvent(Object event) {
+    public void receiveEvent(IGVEvent event) {
 
         if (event instanceof FrameManager.ChangeEvent) {
             // Trim insertionInterval map to current frames
-            Map<ReferenceFrame, List<InsertionInterval>> newMap = Collections.synchronizedMap(new HashMap<>());
-            for (ReferenceFrame frame : ((FrameManager.ChangeEvent) event).getFrames()) {
-                if (insertionIntervalsMap.containsKey(frame)) {
-                    newMap.put(frame, insertionIntervalsMap.get(frame));
-                }
-            }
-            insertionIntervalsMap = newMap;
 
-        } else if (event instanceof AlignmentTrackEvent) {
-            AlignmentTrackEvent e = (AlignmentTrackEvent) event;
-            AlignmentTrackEvent.Type eventType = e.getType();
-            switch (eventType) {
-                case ALLELE_THRESHOLD:
-                    dataManager.alleleThresholdChanged();
-                    break;
-                case RELOAD:
+        } else if (event instanceof AlignmentTrackEvent e) {
+            switch (e.type()) {
+                case ALLELE_THRESHOLD -> dataManager.alleleThresholdChanged();
+                case RELOAD -> {
                     clearCaches();
                     repaint();
-                case REFRESH:
-                    repaint();
-                    break;
+                }
+                case REFRESH -> repaint();
+            }
+        } else if (event instanceof DataLoadedEvent dataLoaded) {
+            if (dataManager.isLoaded(dataLoaded.referenceFrame())) {
+                actionToPerformOnFrameLoad.computeIfPresent(dataLoaded.referenceFrame(), (k, v) -> {
+                    v.accept(k);
+                    return null; //remove this action from the map
+                });
             }
-
         }
     }
 
@@ -420,7 +435,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         }
     }
 
-    ExperimentType getExperimentType() {
+    public ExperimentType getExperimentType() {
         return experimentType;
     }
 
@@ -464,10 +479,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
 
         int nGroups = dataManager.getMaxGroupCount();
         int h = Math.max(minHeight, getNLevels() * getRowHeight() + nGroups * GROUP_MARGIN + TOP_MARGIN
-                + DS_MARGIN_0 + DOWNAMPLED_ROW_HEIGHT + DS_MARGIN_2);
-        //if (insertionRect != null) {   // TODO - replace with expand insertions preference
-        h += INSERTION_ROW_HEIGHT + DS_MARGIN_0;
-        //}
+                + DS_MARGIN_0 + DOWNSAMPLED_ROW_HEIGHT);
         return Math.max(minimumHeight, h);
     }
 
@@ -488,11 +500,10 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     @Override
     public boolean isReadyToPaint(ReferenceFrame frame) {
 
-        if (frame.getChrName().equals(Globals.CHR_ALL) || frame.getScale() > dataManager.getMinVisibleScale()) {
+        double extent = frame.getEnd() - frame.getOrigin();
+        if (frame.getChrName().equals(Globals.CHR_ALL) || extent > getVisibilityWindow()) {
             return true;   // Nothing to paint
         } else {
-            List<InsertionInterval> insertionIntervals = getInsertionIntervals(frame);
-            insertionIntervals.clear();
             return dataManager.isLoaded(frame);
         }
     }
@@ -535,17 +546,9 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         rect.y += DS_MARGIN_0;
 
         downsampleRect = new Rectangle(rect);
-        downsampleRect.height = DOWNAMPLED_ROW_HEIGHT;
+        downsampleRect.height = DOWNSAMPLED_ROW_HEIGHT;
         renderDownsampledIntervals(context, downsampleRect);
 
-        if (renderOptions.isShowInsertionMarkers()) {
-            insertionRect = new Rectangle(rect);
-            insertionRect.y += DOWNAMPLED_ROW_HEIGHT + DS_MARGIN_0;
-            insertionRect.height = INSERTION_ROW_HEIGHT;
-            renderInsertionIntervals(context, insertionRect);
-            rect.y = insertionRect.y + insertionRect.height;
-        }
-
         alignmentsRect = new Rectangle(rect);
         alignmentsRect.y += 2;
         alignmentsRect.height -= (alignmentsRect.y - rect.y);
@@ -598,8 +601,8 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         }
 
         // Check for YC tag
-        if (renderOptions.colorOption == null && dataManager.hasYCTags()) {
-            renderOptions.colorOption = ColorOption.YC_TAG;
+        if (renderOptions.getColorOption() == null && dataManager.hasYCTags()) {
+            renderOptions.setColorOption(ColorOption.YC_TAG);
         }
 
         Map<String, PEStats> peStats = dataManager.getPEStats();
@@ -667,7 +670,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
 
                 // Label the group, if there is room
                 double groupHeight = rows.size() * h;
-                if (groupHeight > GROUP_LABEL_HEIGHT + 2) {
+                if (groupHeight > GROUP_LABEL_HEIGHT + 2 && !context.multiframe) {
                     String groupName = entry.getKey();
                     Graphics2D g = context.getGraphics2D("LABEL");
                     FontMetrics fm = g.getFontMetrics();
@@ -684,77 +687,23 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         groupBorderGraphics.drawLine(inputRect.x, bottom, inputRect.width, bottom);
     }
 
-    private List<InsertionInterval> getInsertionIntervals(ReferenceFrame frame) {
-        List<InsertionInterval> insertionIntervals = insertionIntervalsMap.computeIfAbsent(frame, k -> new ArrayList<>());
-        return insertionIntervals;
-    }
-
-    private void renderInsertionIntervals(RenderContext context, Rectangle rect) {
-
-        // Might be offscreen
-        if (!context.getVisibleRect().intersects(rect)) return;
-
-        List<InsertionMarker> intervals = context.getInsertionMarkers();
-        if (intervals == null) return;
-
-        InsertionMarker selected = InsertionManager.getInstance().getSelectedInsertion(context.getChr());
-
-        int w = (int) ((1.41 * rect.height) / 2);
-
-
-        boolean hideSmallIndels = renderOptions.isHideSmallIndels();
-        int smallIndelThreshold = renderOptions.getSmallIndelThreshold();
-
-        List<InsertionInterval> insertionIntervals = getInsertionIntervals(context.getReferenceFrame());
-        insertionIntervals.clear();
-        for (InsertionMarker insertionMarker : intervals) {
-            if (hideSmallIndels && insertionMarker.size < smallIndelThreshold) continue;
-
-            final double scale = context.getScale();
-            final double origin = context.getOrigin();
-            int midpoint = (int) ((insertionMarker.position - origin) / scale);
-            int x0 = midpoint - w;
-            int x1 = midpoint + w;
-
-            Rectangle iRect = new Rectangle(x0 + context.translateX, rect.y, 2 * w, rect.height);
-
-            insertionIntervals.add(new InsertionInterval(iRect, insertionMarker));
-
-            Color c = (selected != null && selected.position == insertionMarker.position) ? new Color(200, 0, 0, 80) : AlignmentRenderer.purple;
-            Graphics2D g = context.getGraphic2DForColor(c);
-
-
-            g.fillPolygon(new Polygon(new int[]{x0, x1, midpoint},
-                    new int[]{rect.y, rect.y, rect.y + rect.height}, 3));
-        }
-    }
-
+    /**
+     * Render insertions at position of marker in expanded form, showing sequence.  Much of this method is just
+     * a repeat of the loop for alignment rendering to compute the Y position of affected alignments.
+     *
+     * @param insertionMarker
+     * @param context
+     * @param inputRect
+     */
     public void renderExpandedInsertion(InsertionMarker insertionMarker, RenderContext context, Rectangle inputRect) {
 
-
         boolean leaveMargin = getDisplayMode() != DisplayMode.SQUISHED;
+        inputRect.y += DS_MARGIN_0 + DOWNSAMPLED_ROW_HEIGHT + DS_MARGIN_0;
 
-        // Insertion interval
-        if(this.renderOptions.isShowInsertionMarkers()) {
-            Graphics2D g = context.getGraphic2DForColor(Color.red);
-            Rectangle iRect = new Rectangle(inputRect.x, insertionRect.y, inputRect.width, insertionRect.height);
-            g.fill(iRect);
-
-            List<InsertionInterval> insertionIntervals = getInsertionIntervals(context.getReferenceFrame());
-
-            iRect.x += context.translateX;
-            insertionIntervals.add(new InsertionInterval(iRect, insertionMarker));
-        }
-
-
-        inputRect.y += DS_MARGIN_0 + DOWNAMPLED_ROW_HEIGHT + DS_MARGIN_0 + INSERTION_ROW_HEIGHT + DS_MARGIN_2;
-
-        //log.debug("Render features");
         final AlignmentInterval loadedInterval = dataManager.getLoadedInterval(context.getReferenceFrame(), true);
         PackedAlignments groups = dataManager.getGroups(loadedInterval, renderOptions);
         if (groups == null) {
             //Assume we are still loading.
-            //This might not always be true
             return;
         }
 
@@ -778,10 +727,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             h = squishedHeight;
         }
 
-
         for (Map.Entry<String, List<Row>> entry : groups.entrySet()) {
-
-
             // Loop through the alignment rows for this group
             List<Row> rows = entry.getValue();
             for (Row row : rows) {
@@ -792,7 +738,8 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
                 assert visibleRect != null;
                 if (y + h > visibleRect.getY()) {
                     Rectangle rowRectangle = new Rectangle(inputRect.x, (int) y, inputRect.width, (int) h);
-                    renderer.renderExpandedInsertion(insertionMarker, row.alignments, context, rowRectangle, leaveMargin);
+                    if (row.alignments != null)  // TODO -- not sure this is needed
+                        BaseRenderer.drawExpandedInsertions(insertionMarker, row.alignments, context, rowRectangle, leaveMargin, renderOptions);
                     row.y = y;
                     row.h = h;
                 }
@@ -806,24 +753,21 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
 
     }
 
-    private InsertionInterval getInsertionInterval(ReferenceFrame frame, int x, int y) {
-        List<InsertionInterval> insertionIntervals = getInsertionIntervals(frame);
-        for (InsertionInterval i : insertionIntervals) {
-            if (i.rect.contains(x, y)) return i;
-        }
 
-        return null;
-    }
-
-    public void sortRows(final SortOption option, final Double location, final String tag, final boolean invertSort) {
+    public void sortRows(final SortOption option, final Double location, final String tag, final boolean invertSort, final Set<String> priorityRecords) {
         final List<ReferenceFrame> frames = FrameManager.getFrames();
         for (ReferenceFrame frame : frames) {
-            final double actloc = location != null ? location : frame.getCenter();
-            final AlignmentInterval interval = getDataManager().getLoadedInterval(frame);
-            if(interval != null) {
-                interval.sortRows(option, actloc, tag, invertSort);
+            Consumer<ReferenceFrame> sort = (ReferenceFrame f) -> {
+                final AlignmentInterval interval = getDataManager().getLoadedInterval(f);
+                final double actloc = location != null ? location : f.getCenter();
+                interval.sortRows(option, actloc, tag, invertSort, priorityRecords);
+            };
+            //If the data is loaded sort now, otherwise delay until we get a message that it is loaded.
+            if (getDataManager().isLoaded(frame)) {
+                sort.accept(frame);
             } else {
-                log.warn("Attempt to sort alignments prior to loading");
+                log.debug("Attempt to sort alignments prior to loading");
+                actionToPerformOnFrameLoad.put(frame, sort);
             }
         }
     }
@@ -848,13 +792,12 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         if (option == GroupOption.TAG && tag != null) {
             renderOptions.setGroupByTag(tag);
         }
-        if (option == GroupOption.BASE_AT_POS && pos != null) {
+        if ((option == GroupOption.BASE_AT_POS || option == GroupOption.INSERTION_AT_POS) && pos != null) {
             renderOptions.setGroupByPos(pos);
         }
         renderOptions.setGroupByOption(option);
         dataManager.packAlignments(renderOptions);
         repaint();
-
     }
 
     public void setBisulfiteContext(BisulfiteContext option) {
@@ -904,23 +847,15 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
                 return null;
             }
         } else {
-
-            InsertionInterval insertionInterval = getInsertionInterval(frame, mouseX, mouseY);
-            if (insertionInterval != null) {
-                return "Insertions (" + insertionInterval.insertionMarker.size + " bases)";
-            } else {
-                Alignment feature = getAlignmentAt(position, mouseY, frame);
-                if (feature != null) {
-                    return feature.getAlignmentValueString(position, mouseX, renderOptions);
-                }
+            Alignment feature = getAlignmentAt(position, mouseY, frame);
+            if (feature != null) {
+                return feature.getAlignmentValueString(position, mouseX, renderOptions);
             }
-
         }
         return null;
     }
 
 
-
     Alignment getAlignmentAt(final TrackClickEvent te) {
         MouseEvent e = te.getMouseEvent();
         final ReferenceFrame frame = te.getFrame();
@@ -957,6 +892,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     public boolean handleDataClick(TrackClickEvent te) {
 
         MouseEvent e = te.getMouseEvent();
+
         if (Globals.IS_MAC && e.isMetaDown() || (!Globals.IS_MAC && e.isControlDown())) {
             // Selection
             Alignment alignment = this.getAlignmentAt(te);
@@ -971,23 +907,6 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             return true;
         }
 
-        InsertionInterval insertionInterval = getInsertionInterval(te.getFrame(), te.getMouseEvent().getX(), te.getMouseEvent().getY());
-        if (insertionInterval != null) {
-
-            final String chrName = te.getFrame().getChrName();
-            InsertionMarker currentSelection = InsertionManager.getInstance().getSelectedInsertion(chrName);
-            if (currentSelection != null && currentSelection.position == insertionInterval.insertionMarker.position) {
-                InsertionManager.getInstance().clearSelected();
-            } else {
-                InsertionManager.getInstance().setSelected(chrName, insertionInterval.insertionMarker.position);
-            }
-
-            IGVEventBus.getInstance().post(new InsertionSelectionEvent(insertionInterval.insertionMarker));
-
-            return true;
-        }
-
-
         if (IGV.getInstance().isShowDetailsOnClick()) {
             openTooltipWindow(te);
             return true;
@@ -996,7 +915,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         return false;
     }
 
-    void setSelectedAlignment(Alignment alignment) {
+    public void setSelectedAlignment(Alignment alignment) {
         Color c = readNamePalette.get(alignment.getReadName());
         selectedReadNames.put(alignment.getReadName(), c);
     }
@@ -1009,7 +928,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
 
     public void setViewAsPairs(boolean vAP) {
         // TODO -- generalize this test to all incompatible pairings
-        if (vAP && renderOptions.groupByOption == GroupOption.STRAND) {
+        if (vAP && renderOptions.getGroupByOption() == GroupOption.STRAND) {
             boolean ungroup = MessageUtils.confirm("\"View as pairs\" is incompatible with \"Group by strand\". Ungroup?");
             if (ungroup) {
                 renderOptions.setGroupByOption(null);
@@ -1026,39 +945,6 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     public enum ExperimentType {OTHER, RNA, BISULFITE, THIRD_GEN, UNKOWN}
 
 
-    class RenderRollback {
-        final ColorOption colorOption;
-        final GroupOption groupByOption;
-        final String groupByTag;
-        final String colorByTag;
-        final String linkByTag;
-        final DisplayMode displayMode;
-        final int expandedHeight;
-        final boolean showGroupLine;
-
-        RenderRollback(RenderOptions renderOptions, DisplayMode displayMode) {
-            this.colorOption = renderOptions.colorOption;
-            this.groupByOption = renderOptions.groupByOption;
-            this.colorByTag = renderOptions.colorByTag;
-            this.groupByTag = renderOptions.groupByTag;
-            this.displayMode = displayMode;
-            this.expandedHeight = AlignmentTrack.this.expandedHeight;
-            this.showGroupLine = AlignmentTrack.this.showGroupLine;
-            this.linkByTag = renderOptions.linkByTag;
-        }
-
-        void restore(RenderOptions renderOptions) {
-            renderOptions.colorOption = this.colorOption;
-            renderOptions.groupByOption = this.groupByOption;
-            renderOptions.colorByTag = this.colorByTag;
-            renderOptions.groupByTag = this.groupByTag;
-            renderOptions.linkByTag = this.linkByTag;
-            AlignmentTrack.this.expandedHeight = this.expandedHeight;
-            AlignmentTrack.this.showGroupLine = this.showGroupLine;
-            AlignmentTrack.this.setDisplayMode(this.displayMode);
-        }
-    }
-
     public boolean isRemoved() {
         return removed;
     }
@@ -1213,17 +1099,6 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         }
     }
 
-    private static class InsertionInterval {
-
-        final Rectangle rect;
-        final InsertionMarker insertionMarker;
-
-        InsertionInterval(Rectangle rect, InsertionMarker insertionMarker) {
-            this.rect = rect;
-            this.insertionMarker = insertionMarker;
-        }
-    }
-
 
     AlignmentBlock getInsertion(Alignment alignment, int pixelX) {
         if (alignment != null && alignment.getInsertions() != null) {
@@ -1313,6 +1188,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
     public static class RenderOptions implements Cloneable, Persistable {
 
         public static final String NAME = "RenderOptions";
+        private static final Logger log = LogManager.getLogger(RenderOptions.class);
 
         private AlignmentTrack track;
         private Boolean shadeBasesOption;
@@ -1335,6 +1211,7 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         private Boolean linkedReads;
         private Boolean quickConsensusMode;
         private Boolean showMismatches;
+        private Boolean insertQualColoring;
         Boolean computeIsizes;
         private Double minInsertSizePercentile;
         private Double maxInsertSizePercentile;
@@ -1343,26 +1220,48 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
         private Range groupByPos;
         private Boolean invertSorting;
         private boolean invertGroupSorting;
-        private Boolean showInsertionMarkers;
         private Boolean hideSmallIndels;
         private Integer smallIndelThreshold;
+        private BaseModficationFilter basemodFilter;
+        private Float basemodThreshold;
+        private int baseQualityMin;
+        private int baseQualityMax;
+
+        private Integer minJunctionCoverage;
+
 
         BisulfiteContext bisulfiteContext = BisulfiteContext.CG;
         Map<String, PEStats> peStats;
 
-        public RenderOptions() {
-        }
-
         RenderOptions(AlignmentTrack track) {
-            //updateColorScale();
             this.track = track;
             peStats = new HashMap<>();
+
+            // Set some constants -- for efficiency
+            this.baseQualityMin = track == null ? 5 : track.getPreferences().getAsInt(SAM_BASE_QUALITY_MIN);
+            this.baseQualityMax = track == null ? 20 : track.getPreferences().getAsInt(SAM_BASE_QUALITY_MAX);
         }
 
         IGVPreferences getPreferences() {
             return this.track != null ? this.track.getPreferences() : AlignmentTrack.getPreferences(ExperimentType.OTHER);
         }
 
+        public int getMinJunctionCoverage() {
+            return minJunctionCoverage != null ? minJunctionCoverage : PreferencesManager.getPreferences(Constants.RNA).getAsInt(SAM_JUNCTION_MIN_COVERAGE);
+        }
+
+        public void setMinJunctionCoverage(int minJunctionCoverage) {
+            this.minJunctionCoverage = minJunctionCoverage;
+        }
+
+        public int getBaseQualityMin() {
+            return baseQualityMin;
+        }
+
+        public int getBaseQualityMax() {
+            return baseQualityMax;
+        }
+
         void setShowAllBases(boolean showAllBases) {
             this.showAllBases = showAllBases;
         }
@@ -1380,6 +1279,10 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             this.viewPairs = viewPairs;
         }
 
+        void setInsertQualColoring(boolean insertQualColoring) {
+            this.insertQualColoring = insertQualColoring;
+        }
+
         void setComputeIsizes(boolean computeIsizes) {
             this.computeIsizes = computeIsizes;
         }
@@ -1453,10 +1356,6 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             this.linkedReads = linkedReads;
         }
 
-        public void setShowInsertionMarkers(boolean drawInsertionIntervals) {
-            this.showInsertionMarkers = drawInsertionIntervals;
-        }
-
         public void setHideSmallIndels(boolean hideSmallIndels) {
             this.hideSmallIndels = hideSmallIndels;
         }
@@ -1494,10 +1393,6 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             return shadeCenters == null ? getPreferences().getAsBoolean(SAM_SHADE_CENTER) : shadeCenters;
         }
 
-        boolean isShowInsertionMarkers() {
-            return showInsertionMarkers == null ? getPreferences().getAsBoolean(SAM_SHOW_INSERTION_MARKERS) : showInsertionMarkers;
-        }
-
         public boolean isFlagZeroQualityAlignments() {
             return flagZeroQualityAlignments == null ? getPreferences().getAsBoolean(SAM_FLAG_ZERO_QUALITY) : flagZeroQualityAlignments;
         }
@@ -1506,6 +1401,9 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             return viewPairs;
         }
 
+        public boolean isInsertQualColoring() {
+            return insertQualColoring == null ? getPreferences().getAsBoolean(SAM_INSERT_QUAL_COLORING) : insertQualColoring;
+        }
         public boolean isComputeIsizes() {
             return computeIsizes == null ? getPreferences().getAsBoolean(SAM_COMPUTE_ISIZES) : computeIsizes;
         }
@@ -1617,6 +1515,22 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             return smallIndelThreshold == null ? getPreferences().getAsInt(SAM_SMALL_INDEL_BP_THRESHOLD) : smallIndelThreshold;
         }
 
+        public BaseModficationFilter getBasemodFilter() {
+            return basemodFilter;
+        }
+
+        public float getBasemodThreshold() {
+            return basemodThreshold == null ? getPreferences().getAsFloat(BASEMOD_THRESHOLD) : basemodThreshold.floatValue();
+        }
+
+        public void setBasemodThreshold(float basemodThreshold) {
+            this.basemodThreshold = basemodThreshold;
+        }
+
+        public void setBasemodFilter(BaseModficationFilter basemodFilter) {
+
+            this.basemodFilter = basemodFilter;
+        }
 
         @Override
         public void marshalXML(Document document, Element element) {
@@ -1711,9 +1625,16 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
             if (smallIndelThreshold != null) {
                 element.setAttribute("smallIndelThreshold", smallIndelThreshold.toString());
             }
-            if (showInsertionMarkers != null) {
-                element.setAttribute("showInsertionMarkers", showInsertionMarkers.toString());
+            if (basemodFilter != null) {
+                element.setAttribute("basemodFilter", basemodFilter.toString());
             }
+            if (basemodThreshold != null) {
+                element.setAttribute("basemodThredhold", String.valueOf(basemodThreshold));
+            }
+            if (minJunctionCoverage != null) {
+                element.setAttribute("minJunctionCoverage", String.valueOf(minJunctionCoverage));
+            }
+
         }
 
 
@@ -1742,7 +1663,20 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
                 maxInsertSize = Integer.parseInt(element.getAttribute("maxInsertSize"));
             }
             if (element.hasAttribute("colorOption")) {
-                colorOption = ColorOption.valueOf(element.getAttribute("colorOption"));
+                // Convert deprecated options
+                final String attributeValue = element.getAttribute("colorOption");
+                if ("BASE_MODIFICATION_6MA".equals(attributeValue)) {
+                    colorOption = ColorOption.BASE_MODIFICATION;
+                    basemodFilter = new BaseModficationFilter("a");
+                } else if ("BASE_MODIFICATION_5MC".equals(attributeValue)) {
+                    colorOption = ColorOption.BASE_MODIFICATION_2COLOR;
+                    // basemodFilter = new BaseModficationFilter(null, 'C');
+                } else if ("BASE_MODIFICATION_C".equals(attributeValue)) {
+                    colorOption = ColorOption.BASE_MODIFICATION;
+                    // basemodFilter = new BaseModficationFilter(null, 'C');
+                } else {
+                    colorOption = ColorOption.valueOf(attributeValue);
+                }
             }
             if (element.hasAttribute("sortOption")) {
                 sortOption = SortOption.valueOf((element.getAttribute("sortOption")));
@@ -1814,7 +1748,17 @@ public class AlignmentTrack extends AbstractTrack implements IGVEventObserver {
                 smallIndelThreshold = Integer.parseInt(element.getAttribute("smallIndelThreshold"));
             }
             if (element.hasAttribute("showInsertionMarkers")) {
-                showInsertionMarkers = Boolean.parseBoolean(element.getAttribute("showInsertionMarkers"));
+                // TODO -- something with this
+                // showInsertionMarkers = Boolean.parseBoolean(element.getAttribute("showInsertionMarkers"));
+            }
+            if (element.hasAttribute("basemodFilter")) {
+                basemodFilter = BaseModficationFilter.fromString(element.getAttribute("basemodFilter"));
+            }
+            if (element.hasAttribute("basemodThreshold")) {
+                basemodFilter = BaseModficationFilter.fromString(element.getAttribute("basemodThreshold"));
+            }
+            if (element.hasAttribute("minJunctionCoverage")) {
+                minJunctionCoverage = Integer.parseInt(element.getAttribute("minJunctionCoverage"));
             }
         }
     }
