@@ -1,23 +1,24 @@
 package org.broad.igv.sam;
 
+import htsjdk.samtools.SAMTag;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.Locus;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.Strand;
-import org.broad.igv.feature.genome.ChromosomeNameComparator;
 import org.broad.igv.jbrowse.CircularViewUtilities;
-import org.broad.igv.lists.GeneList;
 import org.broad.igv.logging.LogManager;
 import org.broad.igv.logging.Logger;
-import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
+import org.broad.igv.sam.mods.BaseModficationFilter;
+import org.broad.igv.sam.mods.BaseModificationKey;
+import org.broad.igv.sam.mods.BaseModificationUtils;
 import org.broad.igv.sashimi.SashimiPlot;
-import org.broad.igv.session.Session;
 import org.broad.igv.tools.PFMExporter;
 import org.broad.igv.track.SequenceTrack;
 import org.broad.igv.track.Track;
 import org.broad.igv.track.TrackClickEvent;
 import org.broad.igv.track.TrackMenuUtils;
+import org.broad.igv.ui.supdiagram.SupplementaryAlignmentDiagramDialog;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.InsertSizeSettingsDialog;
 import org.broad.igv.ui.panel.FrameManager;
@@ -39,6 +40,7 @@ import java.awt.event.MouseEvent;
 import java.util.*;
 import java.util.List;
 import java.util.function.BiConsumer;
+import java.util.stream.Collectors;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -48,7 +50,7 @@ import static org.broad.igv.prefs.Constants.*;
  */
 class AlignmentTrackMenu extends IGVPopupMenu {
 
-    private static Logger log = LogManager.getLogger(AlignmentTrackMenu.class);
+    private static final Logger log = LogManager.getLogger(AlignmentTrackMenu.class);
 
     private final AlignmentTrack alignmentTrack;
     private final AlignmentDataManager dataManager;
@@ -85,9 +87,7 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         Collection<Track> tracks = List.of(alignmentTrack);
         addSeparator();
         add(TrackMenuUtils.getTrackRenameItem(tracks));
-        addCopyToClipboardItem(e, clickedAlignment);
 
-        addSeparator();
         JMenuItem item = new JMenuItem("Change Track Color...");
         item.addActionListener(evt -> TrackMenuUtils.changeTrackColor(tracks));
         add(item);
@@ -95,12 +95,10 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         // Experiment type  (RNA, THIRD GEN, OTHER)
         addSeparator();
         addExperimentTypeMenuItem();
-        if (alignmentTrack.getExperimentType() == AlignmentTrack.ExperimentType.THIRD_GEN) {
-            addHaplotype(e);
-        }
+//        if (alignmentTrack.getExperimentType() == AlignmentTrack.ExperimentType.THIRD_GEN) {
+//            addHaplotype(e);
+//        }
 
-        // Linked read items
-        addLinkedReadItems();
 
         // Group, sort, color, shade, and pack
         addSeparator();
@@ -119,18 +117,38 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         misMatchesItem.addActionListener(new Deselector(misMatchesItem, showAllItem));
         showAllItem.addActionListener(new Deselector(showAllItem, misMatchesItem));
 
+
+        // Hide small indels
+        JMenuItem smallIndelsItem = new JCheckBoxMenuItem("Hide small indels");
+        smallIndelsItem.setSelected(renderOptions.isHideSmallIndels());
+        smallIndelsItem.addActionListener(aEvt -> UIUtilities.invokeOnEventThread(() -> {
+            if (smallIndelsItem.isSelected()) {
+                String sith = MessageUtils.showInputDialog("Small indel threshold: ", String.valueOf(renderOptions.getSmallIndelThreshold()));
+                try {
+                    renderOptions.setSmallIndelThreshold(Integer.parseInt(sith));
+                } catch (NumberFormatException exc) {
+                    log.error("Error setting small indel threshold - not an integer", exc);
+                }
+            }
+            renderOptions.setHideSmallIndels(smallIndelsItem.isSelected());
+            alignmentTrack.repaint();
+        }));
+        add(smallIndelsItem);
+
         // Paired end items
-        addSeparator();
-        addViewAsPairsMenuItem();
-        if (clickedAlignment != null) {
-            addGoToMate(e, clickedAlignment);
-            showMateRegion(e, clickedAlignment);
+        if (dataManager.isPairedEnd()) {
+            addSeparator();
+            addViewAsPairsMenuItem();
+            if (clickedAlignment != null) {
+                addGoToMate(e, clickedAlignment);
+                showMateRegion(e, clickedAlignment);
+            }
+            addInsertSizeMenuItem();
         }
-        addInsertSizeMenuItem();
 
         // Third gen (primarily) items
         addSeparator();
-        addThirdGenItems();
+        addThirdGenItems(clickedAlignment, e);
 
         // Display mode items
         addSeparator();
@@ -138,11 +156,12 @@ class AlignmentTrackMenu extends IGVPopupMenu {
 
         // Select alignment items
         addSeparator();
-        addSelectByNameItem();
+        addSelectByNameItem(alignmentTrack, e);
         addClearSelectionsMenuItem();
 
         // Copy items
         addSeparator();
+        addCopyToClipboardItem(e, clickedAlignment);
         addCopySequenceItems(e);
         addConsensusSequence(e);
 
@@ -158,17 +177,59 @@ class AlignmentTrackMenu extends IGVPopupMenu {
             addInsertionItems(insertion);
         }
 
-        // Sashimi plot, probably should be depdenent on experimentType (RNA)
-        addSeparator();
-        JMenuItem sashimi = new JMenuItem("Sashimi Plot");
-        sashimi.addActionListener(e1 -> SashimiPlot.openSashimiPlot());
-        add(sashimi);
+        // Sashimi plot
+        if (alignmentTrack.getExperimentType() == AlignmentTrack.ExperimentType.RNA) {
+            addSeparator();
+            JMenuItem sashimi = new JMenuItem("Sashimi Plot");
+            sashimi.addActionListener(e1 -> SashimiPlot.openSashimiPlot());
+            sashimi.setEnabled(alignmentTrack.getExperimentType() == AlignmentTrack.ExperimentType.RNA);
+            add(sashimi);
+        }
 
         // Show alignments, coverage, splice junctions
         addSeparator();
         addShowItems();
+    }
 
+    private void addShowChimericRegions(final AlignmentTrack alignmentTrack, final TrackClickEvent e, final Alignment clickedAlignment) {
 
+        JMenuItem item = new JMenuItem("View chimeric alignments in split screen");
+        if (clickedAlignment != null && clickedAlignment.getAttribute(SAMTag.SA.name()) != null) {
+            item.setEnabled(true);
+            item.addActionListener(aEvt -> {
+                final String saTag = clickedAlignment.getAttribute(SAMTag.SA.name()).toString();
+                try {
+                    List<SupplementaryAlignment> supplementaryAlignments = SupplementaryAlignment.parseFromSATag(saTag);
+                    alignmentTrack.setSelectedAlignment(clickedAlignment);
+                    FrameManager.addNewLociToFrames(e.getFrame(), supplementaryAlignments, alignmentTrack.getSelectedReadNames().keySet());
+                } catch (final Exception ex) {
+                    MessageUtils.showMessage("Failed to handle SA tag: " + saTag + " due to " + ex.getMessage());
+                    item.setEnabled(false);
+                }
+            });
+        } else {
+            item.setEnabled(false);
+        }
+        add(item);
+    }
+
+    private void addShowDiagram(final TrackClickEvent e, final Alignment clickedAlignment) {
+        JMenuItem item = new JMenuItem("Supplementary Reads Diagram");
+        if (clickedAlignment != null && clickedAlignment.getAttribute(SAMTag.SA.name()) != null) {
+            item.setEnabled(true);
+            item.addActionListener(aEvt -> {
+                try {
+                    final SupplementaryAlignmentDiagramDialog frame = new SupplementaryAlignmentDiagramDialog(IGV.getInstance().getMainFrame(), clickedAlignment, new Dimension(500, 250));
+                    frame.setVisible(true);
+                } catch (final Exception ex) {
+                    MessageUtils.showMessage("Failed to handle SA tag: " + clickedAlignment.getAttribute(SAMTag.SA.name()) + " due to " + ex.getMessage());
+                    item.setEnabled(false);
+                }
+            });
+        } else {
+            item.setEnabled(false);
+        }
+        add(item);
     }
 
 
@@ -302,11 +363,12 @@ class AlignmentTrackMenu extends IGVPopupMenu {
 
     }
 
-    void addSelectByNameItem() {
-        // Change track height by attribute
+    void addSelectByNameItem(AlignmentTrack track, TrackClickEvent e) {
         JMenuItem item = new JMenuItem("Select by name...");
+        final Alignment alignment = track.getAlignmentAt(e);
+        String alignmentName = alignment == null ? "" : alignment.getReadName();
         item.addActionListener(aEvt -> {
-            String val = MessageUtils.showInputDialog("Enter read name: ");
+            String val = MessageUtils.showInputDialog("Enter read name: ", alignmentName);
             if (val != null && val.trim().length() > 0) {
                 alignmentTrack.getSelectedReadNames().put(val, alignmentTrack.getReadNamePalette().get(val));
                 alignmentTrack.repaint();
@@ -353,10 +415,14 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         ButtonGroup group = new ButtonGroup();
 
         AlignmentTrack.GroupOption[] groupOptions = {
-                AlignmentTrack.GroupOption.NONE, AlignmentTrack.GroupOption.STRAND, AlignmentTrack.GroupOption.FIRST_OF_PAIR_STRAND, AlignmentTrack.GroupOption.SAMPLE,
-                AlignmentTrack.GroupOption.LIBRARY, AlignmentTrack.GroupOption.READ_GROUP, AlignmentTrack.GroupOption.MATE_CHROMOSOME,
-                AlignmentTrack.GroupOption.PAIR_ORIENTATION, AlignmentTrack.GroupOption.SUPPLEMENTARY, AlignmentTrack.GroupOption.REFERENCE_CONCORDANCE,
-                AlignmentTrack.GroupOption.MOVIE, AlignmentTrack.GroupOption.ZMW, AlignmentTrack.GroupOption.READ_ORDER, AlignmentTrack.GroupOption.LINKED, AlignmentTrack.GroupOption.PHASE,
+                AlignmentTrack.GroupOption.NONE, AlignmentTrack.GroupOption.STRAND,
+                AlignmentTrack.GroupOption.FIRST_OF_PAIR_STRAND, AlignmentTrack.GroupOption.SAMPLE,
+                AlignmentTrack.GroupOption.LIBRARY, AlignmentTrack.GroupOption.READ_GROUP,
+                AlignmentTrack.GroupOption.MATE_CHROMOSOME,
+                AlignmentTrack.GroupOption.PAIR_ORIENTATION, AlignmentTrack.GroupOption.CHIMERIC,
+                AlignmentTrack.GroupOption.SUPPLEMENTARY, AlignmentTrack.GroupOption.REFERENCE_CONCORDANCE,
+                AlignmentTrack.GroupOption.MOVIE, AlignmentTrack.GroupOption.ZMW, AlignmentTrack.GroupOption.READ_ORDER,
+                AlignmentTrack.GroupOption.LINKED, AlignmentTrack.GroupOption.PHASE,
                 AlignmentTrack.GroupOption.MAPPING_QUALITY
         };
 
@@ -386,25 +452,35 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         groupMenu.add(tagOption);
         group.add(tagOption);
 
-        Range oldGroupByPos = renderOptions.getGroupByPos();
-        if (oldGroupByPos != null && renderOptions.getGroupByOption() == AlignmentTrack.GroupOption.BASE_AT_POS) { // already sorted by the base at a position
-            JCheckBoxMenuItem oldGroupByPosOption = new JCheckBoxMenuItem("base at " + oldGroupByPos.getChr() +
-                    ":" + Globals.DECIMAL_FORMAT.format(1 + oldGroupByPos.getStart()));
-            groupMenu.add(oldGroupByPosOption);
-            oldGroupByPosOption.setSelected(true);
-        }
+//        Range oldGroupByPos = renderOptions.getGroupByPos();
+//        if (oldGroupByPos != null && renderOptions.getGroupByOption() == AlignmentTrack.GroupOption.BASE_AT_POS) { // already sorted by the base at a position
+//            JCheckBoxMenuItem oldGroupByPosOption = new JCheckBoxMenuItem("base at " + oldGroupByPos.getChr() +
+//                    ":" + Globals.DECIMAL_FORMAT.format(1 + oldGroupByPos.getStart()));
+//            groupMenu.add(oldGroupByPosOption);
+//            oldGroupByPosOption.setSelected(true);
+//        }
+//
+//        if (renderOptions.getGroupByOption() != AlignmentTrack.GroupOption.BASE_AT_POS || oldGroupByPos == null ||
+//                !oldGroupByPos.getChr().equals(chrom) || oldGroupByPos.getStart() != chromStart) { // not already sorted by this position
+        JCheckBoxMenuItem newGroupByPosOption = new JCheckBoxMenuItem("base at " + chrom +
+                ":" + Globals.DECIMAL_FORMAT.format(1 + chromStart));
+        newGroupByPosOption.addActionListener(aEvt -> {
+            Range groupByPos = new Range(chrom, chromStart, chromStart + 1);
+            groupAlignments(AlignmentTrack.GroupOption.BASE_AT_POS, null, groupByPos);
+        });
+        groupMenu.add(newGroupByPosOption);
+        group.add(newGroupByPosOption);
+        // }
+
+        JCheckBoxMenuItem newGroupByInsOption = new JCheckBoxMenuItem("insertion at " + chrom +
+                ":" + Globals.DECIMAL_FORMAT.format(1 + chromStart));
+        newGroupByInsOption.addActionListener(aEvt -> {
+            Range groupByPos = new Range(chrom, chromStart, chromStart + 1);
+            groupAlignments(AlignmentTrack.GroupOption.INSERTION_AT_POS, null, groupByPos);
+        });
+        groupMenu.add(newGroupByInsOption);
+        group.add(newGroupByInsOption);
 
-        if (renderOptions.getGroupByOption() != AlignmentTrack.GroupOption.BASE_AT_POS || oldGroupByPos == null ||
-                !oldGroupByPos.getChr().equals(chrom) || oldGroupByPos.getStart() != chromStart) { // not already sorted by this position
-            JCheckBoxMenuItem newGroupByPosOption = new JCheckBoxMenuItem("base at " + chrom +
-                    ":" + Globals.DECIMAL_FORMAT.format(1 + chromStart));
-            newGroupByPosOption.addActionListener(aEvt -> {
-                Range groupByPos = new Range(chrom, chromStart, chromStart + 1);
-                groupAlignments(AlignmentTrack.GroupOption.BASE_AT_POS, null, groupByPos);
-            });
-            groupMenu.add(newGroupByPosOption);
-            group.add(newGroupByPosOption);
-        }
 
         groupMenu.add(new Separator());
         JCheckBoxMenuItem invertGroupNameSortingOption = new JCheckBoxMenuItem("Reverse group order");
@@ -428,8 +504,8 @@ class AlignmentTrackMenu extends IGVPopupMenu {
 
     private void groupAlignments(AlignmentTrack.GroupOption option, String tag, Range pos) {
 
-        if(alignmentTrack.getPreferences().getAsBoolean(SAM_GROUP_ALL)) {
-            for(AlignmentTrack t : IGV.getInstance().getAlignmentTracks()) {
+        if (alignmentTrack.getPreferences().getAsBoolean(SAM_GROUP_ALL)) {
+            for (AlignmentTrack t : IGV.getInstance().getAlignmentTracks()) {
                 t.groupAlignments(option, tag, pos);
             }
         } else {
@@ -457,7 +533,8 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         mappings.put("read order", SortOption.READ_ORDER);
         mappings.put("read name", SortOption.READ_NAME);
         mappings.put("aligned read length", SortOption.ALIGNED_READ_LENGTH);
-// mappings.put("supplementary flag", SortOption.SUPPLEMENTARY);
+        mappings.put("left clip", SortOption.LEFT_CLIP);
+        mappings.put("right clip", SortOption.RIGHT_CLIP);
 
         if (dataManager.isPairedEnd()) {
             mappings.put("insert size", SortOption.INSERT_SIZE);
@@ -524,6 +601,21 @@ class AlignmentTrackMenu extends IGVPopupMenu {
             alignmentTrack.setColorOption(option);
             alignmentTrack.repaint();
         });
+        return mi;
+    }
+
+    private JRadioButtonMenuItem getBasemodColorMenuItem(String label, final AlignmentTrack.ColorOption option, boolean groupByStrand, String filter) {
+        JRadioButtonMenuItem mi = new JRadioButtonMenuItem(label);
+        mi.setSelected(renderOptions.getColorOption() == option);
+        mi.addActionListener(aEvt -> {
+            alignmentTrack.setColorOption(option);
+            renderOptions.setBasemodFilter(filter == null ? null : new BaseModficationFilter(filter));
+            if (groupByStrand) {
+                alignmentTrack.groupAlignments(AlignmentTrack.GroupOption.FIRST_OF_PAIR_STRAND, null, null);
+            } else {
+                alignmentTrack.repaint();
+            }
+        });
 
         return mi;
     }
@@ -587,17 +679,49 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         colorMenu.add(getBisulfiteContextMenuItem(group));
 
         // Base modifications
-        mappings.clear();
-        mappings.put("base modification", AlignmentTrack.ColorOption.BASE_MODIFICATION);
-        mappings.put("base modification (5mC)", AlignmentTrack.ColorOption.BASE_MODIFICATION_5MC);
-        mappings.put("base modification (all C)", AlignmentTrack.ColorOption.BASE_MODIFICATION_C);
-        colorMenu.addSeparator();
-        for (Map.Entry<String, AlignmentTrack.ColorOption> el : mappings.entrySet()) {
-            JRadioButtonMenuItem mi = getColorMenuItem(el.getKey(), el.getValue());
-            colorMenu.add(mi);
-            group.add(mi);
+        JRadioButtonMenuItem bmMenuItem;
+
+        Set<String> allModifications = dataManager.getAllBaseModificationKeys().stream().map(BaseModificationKey::getModification).collect(Collectors.toSet());
+        final int modificationCount = allModifications.size();
+        if (modificationCount > 0) {
+            BaseModficationFilter filter = renderOptions.getBasemodFilter();
+            boolean groupByStrand = alignmentTrack.getPreferences().getAsBoolean(BASEMOD_GROUP_BY_STRAND);
+            colorMenu.addSeparator();
+            if (modificationCount > 1) {
+                bmMenuItem = getBasemodColorMenuItem("base modification (all)", AlignmentTrack.ColorOption.BASE_MODIFICATION, groupByStrand, null);
+                bmMenuItem.setSelected(renderOptions.getColorOption() == AlignmentTrack.ColorOption.BASE_MODIFICATION && filter == null);
+                colorMenu.add(bmMenuItem);
+                group.add(bmMenuItem);
+            }
+            for (String m : allModifications) {
+                String name = BaseModificationUtils.modificationName(m);
+                bmMenuItem = getBasemodColorMenuItem("base modification (" + name + ")", AlignmentTrack.ColorOption.BASE_MODIFICATION, groupByStrand, m);
+                bmMenuItem.setSelected(renderOptions.getColorOption() == AlignmentTrack.ColorOption.BASE_MODIFICATION && (filter != null && filter.pass(m)));
+                colorMenu.add(bmMenuItem);
+                group.add(bmMenuItem);
+            }
+
+
+            colorMenu.addSeparator();
+            if (modificationCount > 1) {
+                bmMenuItem = getBasemodColorMenuItem("base modification 2-color (all)", AlignmentTrack.ColorOption.BASE_MODIFICATION_2COLOR, groupByStrand, null);
+                bmMenuItem.setSelected(renderOptions.getColorOption() == AlignmentTrack.ColorOption.BASE_MODIFICATION_2COLOR && filter == null);
+                colorMenu.add(bmMenuItem);
+                group.add(bmMenuItem);
+            }
+            for (String m : allModifications) {
+                String name = BaseModificationUtils.modificationName(m);
+                bmMenuItem = getBasemodColorMenuItem("base modification 2-color (" + name + ")", AlignmentTrack.ColorOption.BASE_MODIFICATION_2COLOR, groupByStrand, m);
+                bmMenuItem.setSelected(renderOptions.getColorOption() ==
+                        AlignmentTrack.ColorOption.BASE_MODIFICATION_2COLOR &&
+                        (filter != null && filter.pass(m)));
+                colorMenu.add(bmMenuItem);
+                group.add(bmMenuItem);
+            }
         }
 
+
+        // SMRT kinetics
         if (alignmentTrack.getPreferences().getAsBoolean(SMRT_KINETICS_SHOW_OPTIONS)) {
             // Show additional options to help visualize SMRT kinetics data
             mappings.clear();
@@ -606,7 +730,7 @@ class AlignmentTrackMenu extends IGVPopupMenu {
             mappings.put("SMRT CCS fwd-strand aligned IPD", AlignmentTrack.ColorOption.SMRT_CCS_FWD_IPD);
             mappings.put("SMRT CCS fwd-strand aligned PW", AlignmentTrack.ColorOption.SMRT_CCS_FWD_PW);
             mappings.put("SMRT CCS rev-strand aligned IPD", AlignmentTrack.ColorOption.SMRT_CCS_REV_IPD);
-            mappings.put("SMRT CCS rev-strand aligned PW",AlignmentTrack.ColorOption.SMRT_CCS_REV_PW);
+            mappings.put("SMRT CCS rev-strand aligned PW", AlignmentTrack.ColorOption.SMRT_CCS_REV_PW);
             colorMenu.addSeparator();
             for (Map.Entry<String, AlignmentTrack.ColorOption> el : mappings.entrySet()) {
                 JRadioButtonMenuItem mi = getColorMenuItem(el.getKey(), el.getValue());
@@ -649,7 +773,7 @@ class AlignmentTrackMenu extends IGVPopupMenu {
     void addCopyToClipboardItem(final TrackClickEvent te, Alignment alignment) {
 
         final MouseEvent me = te.getMouseEvent();
-        JMenuItem item = new JMenuItem("Copy read details to clipboard");
+        JMenuItem item = new JMenuItem("Copy read details");
         final ReferenceFrame frame = te.getFrame();
         if (frame == null) {
             item.setEnabled(false);
@@ -730,17 +854,6 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         return item;
     }
 
-    void addQuickConsensusModeItem() {
-        // Change track height by attribute
-        final JMenuItem item = new JCheckBoxMenuItem("Quick consensus mode");
-        item.setSelected(renderOptions.isQuickConsensusMode());
-        item.addActionListener(aEvt -> {
-            renderOptions.setQuickConsensusMode(item.isSelected());
-            alignmentTrack.repaint();
-        });
-        add(item);
-    }
-
     JMenuItem addShowMismatchesMenuItem() {
         // Change track height by attribute
         final JMenuItem item = new JCheckBoxMenuItem("Show mismatched bases");
@@ -853,21 +966,20 @@ class AlignmentTrackMenu extends IGVPopupMenu {
 
         /* Add a "Copy left clipped sequence" item if there is  left clipping. */
         int minimumBlatLength = BlatClient.MINIMUM_BLAT_LENGTH;
-        int[] clipping = SAMAlignment.getClipping(alignment.getCigarString());
-        if (clipping[1] > 0) {
-            String lcSeq = getClippedSequence(alignment.getReadSequence(), alignment.getReadStrand(), 0, clipping[1]);
+        ClippingCounts clipping = alignment.getClippingCounts();
+        if (clipping.getLeftSoft() > 0) {
+            String lcSeq = getClippedSequence(alignment.getReadSequence(), 0, clipping.getLeftSoft());
             final JMenuItem lccItem = new JMenuItem("Copy left-clipped sequence");
             add(lccItem);
             lccItem.addActionListener(aEvt -> StringUtils.copyTextToClipboard(lcSeq));
         }
 
         /* Add a "Copy right clipped sequence" item if there is  right clipping. */
-        if (clipping[3] > 0) {
+        if (clipping.getRightSoft() > 0) {
             int seqLength = seq.length();
             String rcSeq = getClippedSequence(
                     alignment.getReadSequence(),
-                    alignment.getReadStrand(),
-                    seqLength - clipping[3],
+                    seqLength - clipping.getRightSoft(),
                     seqLength);
 
             final JMenuItem rccItem = new JMenuItem("Copy right-clipped sequence");
@@ -910,46 +1022,47 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         }
 
         int minimumBlatLength = BlatClient.MINIMUM_BLAT_LENGTH;
-        int[] clipping = SAMAlignment.getClipping(alignment.getCigarString());
+        ClippingCounts clipping = alignment.getClippingCounts();
 
         /* Add a "BLAT left clipped sequence" item if there is significant left clipping. */
-        if (clipping[1] > minimumBlatLength) {
-            String lcSeq = getClippedSequence(alignment.getReadSequence(), alignment.getReadStrand(), 0, clipping[1]);
+        if (clipping.getLeftSoft() > minimumBlatLength) {
+            String lcSeq = getClippedSequence(alignment.getReadSequence(), 0, clipping.getLeftSoft());
+            String blatSeq = alignment.isNegativeStrand() ?
+                    SequenceTrack.getReverseComplement(lcSeq) :
+                    lcSeq;
             final JMenuItem lcbItem = new JMenuItem("BLAT left-clipped sequence");
             add(lcbItem);
             lcbItem.addActionListener(aEvt ->
-                    BlatClient.doBlatQuery(lcSeq, alignment.getReadName() + " - left clip")
+                    BlatClient.doBlatQuery(blatSeq, alignment.getReadName() + " - left clip")
             );
         }
         /* Add a "BLAT right clipped sequence" item if there is significant right clipping. */
-        if (clipping[3] > minimumBlatLength) {
+        if (clipping.getRightSoft() > minimumBlatLength) {
 
             String seq = alignment.getReadSequence();
             int seqLength = seq.length();
             String rcSeq = getClippedSequence(
                     alignment.getReadSequence(),
-                    alignment.getReadStrand(),
-                    seqLength - clipping[3],
+                    seqLength - clipping.getRightSoft(),
                     seqLength);
+            String blatSeq = alignment.isNegativeStrand() ?
+                    SequenceTrack.getReverseComplement(rcSeq) :
+                    rcSeq;
 
             final JMenuItem rcbItem = new JMenuItem("BLAT right-clipped sequence");
             add(rcbItem);
             rcbItem.addActionListener(aEvt ->
-                    BlatClient.doBlatQuery(rcSeq, alignment.getReadName() + " - right clip")
+                    BlatClient.doBlatQuery(blatSeq, alignment.getReadName() + " - right clip")
             );
 
         }
     }
 
-    private String getClippedSequence(String readSequence, Strand strand, int i, int i2) {
+    private String getClippedSequence(String readSequence, int i, int i2) {
         if (readSequence == null || readSequence.equals("*")) {
             return "*";
         }
-        String seq = readSequence.substring(i, i2);
-        if (strand == Strand.NEGATIVE) {
-            seq = SequenceTrack.getReverseComplement(seq);
-        }
-        return seq;
+        return readSequence.substring(i, i2);
     }
 
     void addExtViewItem(final TrackClickEvent te) {
@@ -980,11 +1093,7 @@ class AlignmentTrackMenu extends IGVPopupMenu {
      * and linking by arbitrary tag.
      */
     void addLinkedReadItems() {
-        addSeparator();
-        add(linkedReadViewItem("BX"));
-        add(linkedReadViewItem("MI"));
 
-        addSeparator();
         final JCheckBoxMenuItem supplementalItem = new JCheckBoxMenuItem("Link supplementary alignments");
         supplementalItem.setSelected(alignmentTrack.isLinkedReads() && "READNAME".equals(renderOptions.getLinkByTag()));
         supplementalItem.addActionListener(aEvt -> {
@@ -1035,7 +1144,11 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         item.setSelected(!alignmentTrack.isLinkedReadView() && alignmentTrack.isLinkedReads() && tag.equals(renderOptions.getLinkByTag()));
         item.addActionListener(aEvt -> {
             boolean linkedReads = item.isSelected();
-            setLinkByTag(linkedReads, tag);
+            if ("BX".equals(tag) || "MI".equals(tag)) {
+                alignmentTrack.setLinkedReadView(linkedReads, tag);
+            } else {
+                setLinkByTag(linkedReads, tag);
+            }
         });
         return item;
     }
@@ -1057,46 +1170,15 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         }
     }
 
-    void addThirdGenItems() {
+    void addThirdGenItems(Alignment clickedAlignment, final TrackClickEvent tce) {
 
-        final JMenuItem qcItem = new JCheckBoxMenuItem("Quick consensus mode");
-        qcItem.setSelected(renderOptions.isQuickConsensusMode());
-        qcItem.addActionListener(aEvt -> {
-            renderOptions.setQuickConsensusMode(qcItem.isSelected());
-            alignmentTrack.repaint();
-        });
+        // Linked read items -- mostly for 3rd gen but might also be relevant to 10X and other linked read assays
+        addLinkedReadItems();
 
-        final JMenuItem thresholdItem = new JMenuItem("Small indel threshold...");
-        thresholdItem.addActionListener(evt -> UIUtilities.invokeOnEventThread(() -> {
-            String sith = MessageUtils.showInputDialog("Small indel threshold: ", String.valueOf(renderOptions.getSmallIndelThreshold()));
-            try {
-                renderOptions.setSmallIndelThreshold(Integer.parseInt(sith));
-                alignmentTrack.repaint();
-            } catch (NumberFormatException e) {
-                log.error("Error setting small indel threshold - not an integer", e);
-            }
-        }));
-        thresholdItem.setEnabled(renderOptions.isHideSmallIndels());
+        //Supplementary/chimeric items, only if the read has an SA tag;
+        addShowChimericRegions(alignmentTrack, tce, clickedAlignment);
+        addShowDiagram(tce, clickedAlignment);
 
-        final JMenuItem item = new JCheckBoxMenuItem("Hide small indels");
-        item.setSelected(renderOptions.isHideSmallIndels());
-        item.addActionListener(aEvt -> UIUtilities.invokeOnEventThread(() -> {
-            renderOptions.setHideSmallIndels(item.isSelected());
-            thresholdItem.setEnabled(item.isSelected());
-            alignmentTrack.repaint();
-        }));
-
-        final JMenuItem imItem = new JCheckBoxMenuItem("Show insertion markers");
-        imItem.setSelected(renderOptions.isShowInsertionMarkers());
-        imItem.addActionListener(aEvt -> {
-            renderOptions.setShowInsertionMarkers(imItem.isSelected());
-            alignmentTrack.repaint();
-        });
-
-        add(imItem);
-        add(qcItem);
-        add(item);
-        add(thresholdItem);
     }
 
     /**
@@ -1142,6 +1224,7 @@ class AlignmentTrackMenu extends IGVPopupMenu {
                 int newStart = (int) Math.max(0, (start + (alignment.getEnd() - alignment.getStart()) / 2 - range / 2));
                 int newEnd = newStart + (int) range;
                 frame.jumpTo(chr, newStart, newEnd);
+                AlignmentTrack.sortSelectedReadsToTheTop(alignmentTrack.getSelectedReadNames().keySet());
                 frame.recordHistory();
             } else {
                 MessageUtils.showMessage("Alignment does not have mate, or it is not mapped.");
@@ -1155,73 +1238,11 @@ class AlignmentTrackMenu extends IGVPopupMenu {
      * Need a better name for this method.
      */
     private void splitScreenMate(ReferenceFrame frame, Alignment alignment) {
-
         if (alignment != null) {
             ReadMate mate = alignment.getMate();
             if (mate != null && mate.isMapped()) {
-
                 alignmentTrack.setSelectedAlignment(alignment);
-
-                String mateChr = mate.getChr();
-                int mateStart = mate.start - 1;
-
-                String locus1 = frame.getFormattedLocusString();
-
-                // Generate a locus string for the read mate.  Keep the window width (in base pairs) == to the current range
-                Range range = frame.getCurrentRange();
-                int length = range.getLength();
-                int s2 = Math.max(0, mateStart - length / 2);
-                int e2 = s2 + length;
-                String startStr = String.valueOf(s2);
-                String endStr = String.valueOf(e2);
-                String mateLocus = mateChr + ":" + startStr + "-" + endStr;
-
-                Session currentSession = IGV.getInstance().getSession();
-
-                List<String> loci;
-                if (FrameManager.isGeneListMode()) {
-                    loci = new ArrayList<>(FrameManager.getFrames().size());
-                    for (ReferenceFrame ref : FrameManager.getFrames()) {
-                        //If the frame-name is a locus, we use it unaltered
-                        //Don't want to reprocess, easy to get off-by-one
-                        String name = ref.getName();
-                        if (Locus.fromString(name) != null) {
-                            loci.add(name);
-                        } else {
-                            loci.add(ref.getFormattedLocusString());
-                        }
-
-                    }
-                    loci.add(mateLocus);
-                } else {
-                    loci = Arrays.asList(locus1, mateLocus);
-                }
-
-                StringBuilder listName = new StringBuilder();
-                for (String s : loci) {
-                    listName.append(s + "   ");
-                }
-
-                GeneList geneList = new GeneList(listName.toString(), loci, false);
-                currentSession.setCurrentGeneList(geneList);
-                
-                Comparator<String> geneListComparator = (n0, n1) -> {
-                    ReferenceFrame f0 = FrameManager.getFrame(n0);
-                    ReferenceFrame f1 = FrameManager.getFrame(n1);
-
-                    String chr0 = f0 == null ? "" : f0.getChrName();
-                    String chr1 = f1 == null ? "" : f1.getChrName();
-                    int s0 = f0 == null ? 0 : f0.getCurrentRange().getStart();
-                    int s1 = f1 == null ? 0 : f1.getCurrentRange().getStart();
-
-                    int chrComp = ChromosomeNameComparator.get().compare(chr0, chr1);
-                    if (chrComp != 0) return chrComp;
-                    return s0 - s1;
-                };
-
-                //Need to sort the frames by position
-                currentSession.sortGeneList(geneListComparator);
-                IGV.getInstance().resetFrames();
+                FrameManager.addNewLociToFrames(frame, List.of(mate), alignmentTrack.getSelectedReadNames().keySet());
             } else {
                 MessageUtils.showMessage("Alignment does not have mate, or it is not mapped.");
             }
@@ -1268,7 +1289,7 @@ class AlignmentTrackMenu extends IGVPopupMenu {
         }
         renderOptions.setLinkedReads(linkReads);
         dataManager.packAlignments(renderOptions);
-        repaint();
+        alignmentTrack.repaint();
     }
 
 
@@ -1294,3 +1315,4 @@ class AlignmentTrackMenu extends IGVPopupMenu {
     }
 
 }
+
