@@ -29,9 +29,11 @@
  */
 package org.broad.igv.sam;
 
+import htsjdk.samtools.Cigar;
 import htsjdk.samtools.SAMFileHeader;
 import htsjdk.samtools.SAMReadGroupRecord;
 import htsjdk.samtools.SAMRecord;
+import htsjdk.samtools.SAMTag;
 import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.Strand;
@@ -42,12 +44,14 @@ import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.sam.mods.BaseModificationUtils;
 import org.broad.igv.sam.mods.BaseModificationSet;
 import org.broad.igv.sam.smrt.SMRTKinetics;
-import org.broad.igv.sam.smrt.SMRTKineticsDecoder;
 import org.broad.igv.ui.color.ColorUtilities;
+import org.broad.igv.ultima.FlowUtil;
+import org.broad.igv.ultima.annotate.FlowBlockAnnotator;
 
 import java.awt.*;
 import java.util.*;
 import java.util.List;
+import java.util.function.Supplier;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
@@ -56,8 +60,10 @@ import java.util.regex.Pattern;
  */
 public class SAMAlignment implements Alignment {
 
-    private static Logger log = LogManager.getLogger(SAMAlignment.class);
-
+    public static final int MAX_CIGAR_STRING_LENGTH_TO_DISPLAY = 50;
+    public static final Pattern RIGHT_CIGAR_PATTERN = Pattern.compile("[A-Z](.{1," + MAX_CIGAR_STRING_LENGTH_TO_DISPLAY / 2 + "})$");
+    public static final Pattern LEFT_CIGAR_PATTERN = Pattern.compile("^(.{1," + (MAX_CIGAR_STRING_LENGTH_TO_DISPLAY / 2 - 1) + "}[A-Z])");
+    private static final Logger log = LogManager.getLogger(SAMAlignment.class);
     public static final char DELETE_CHAR = '-';
     public static final char SKIP_CHAR = '=';
     public static final char MATCH = 'M';
@@ -90,6 +96,8 @@ public class SAMAlignment implements Alignment {
 
     private int flags;
 
+    final private static FlowBlockAnnotator flowBlockAnnotator = new FlowBlockAnnotator();
+
     /**
      * Picard object upon which this SAMAlignment is based
      */
@@ -116,6 +124,25 @@ public class SAMAlignment implements Alignment {
     private List<BaseModificationSet> baseModificationSets;
     private SMRTKinetics smrtKinetics;
 
+    private enum CacheKey {CLIPPING_COUNTS, SA_GROUP}
+
+    ;
+
+    /**
+     * Use this to cache an expensive to compute value on this record and retrieve it as necessary.
+     */
+    @SuppressWarnings("unchecked")
+    private <T> T getCachedOrCompute(CacheKey key, Supplier<T> supplier) {
+        final Object value = record.getTransientAttribute(key);
+        if (value == null) {
+            final T newValue = supplier.get();
+            record.setTransientAttribute(key, newValue);
+            return newValue;
+        } else {
+            return (T) value;
+        }
+    }
+
     public SAMAlignment(SAMRecord record) {
 
         this.record = record;
@@ -195,6 +222,10 @@ public class SAMAlignment implements Alignment {
         }
     }
 
+    private Object getAttribute(SAMTag key) {
+        return key == null ? null : record.getAttribute(key);
+    }
+
     public boolean isFirstOfPair() {
         return isPaired() && (flags & FIRST_OF_PAIR_FLAG) != 0;
     }
@@ -255,6 +286,17 @@ public class SAMAlignment implements Alignment {
         return record.getCigarString();
     }
 
+    @Override
+    public Cigar getCigar() {
+        return record.getCigar();
+    }
+
+    @Override
+    public ClippingCounts getClippingCounts() {
+        return getCachedOrCompute(CacheKey.CLIPPING_COUNTS,
+                () -> ClippingCounts.fromCigar(record.getCigar()));
+    }
+
     public String getReadSequence() {
         return record.getReadString();
     }
@@ -330,12 +372,31 @@ public class SAMAlignment implements Alignment {
 
             Object mm = record.hasAttribute("Mm") ? record.getAttribute("Mm") : record.getAttribute("MM");
             byte[] ml = (byte[]) (record.hasAttribute("Ml") ? record.getAttribute("Ml") : record.getAttribute("ML"));
-            byte[] sequence = record.getReadBases();
 
-            if (mm.toString().length() == 0) { // TODO -- more extensive validation?
-                baseModificationSets = Collections.EMPTY_LIST;
-            } else {
-                baseModificationSets = BaseModificationUtils.getBaseModificationSets(mm.toString(), ml, sequence, isNegativeStrand());
+            // Minimal tag validation  -- 10X uses MM and/or ML for other purposes
+            if (mm instanceof String && (mm.toString().length() > 0) && (ml == null || ml instanceof byte[])) {
+
+                byte[] sequence = record.getReadBases();
+
+                // Sequence length validation -- if MN tag is present use it, otherwise do a partial validation
+                // Test sequence length vs mn if avaliable
+                Integer mn = record.getIntegerAttribute("MN");
+                if (mn == null) {
+                    mn = sequence.length;
+                }
+                if (mn != null) {
+                    if (mn != sequence.length) {
+                        return null;
+                    }
+                } else if (!BaseModificationUtils.validateMMTag(this.getReadName(), mm.toString(), record.getReadBases(), isNegativeStrand())) {  //record.getCigarString().indexOf("H") > 0 &&
+                    return null;
+                }
+
+                if (mm.toString().length() == 0) { // TODO -- more extensive validation?
+                    baseModificationSets = Collections.EMPTY_LIST;
+                } else {
+                    baseModificationSets = BaseModificationUtils.getBaseModificationSets((String) mm, ml, sequence, isNegativeStrand());
+                }
             }
         }
         return baseModificationSets;
@@ -471,7 +532,7 @@ public class SAMAlignment implements Alignment {
                 gapTypes = new char[nGaps];
             }
             if (nRealGaps > 0) {
-                gaps = new ArrayList<Gap>();
+                gaps = new ArrayList<>();
             }
 
             // Adjust start to include soft clipped bases a
@@ -575,7 +636,7 @@ public class SAMAlignment implements Alignment {
      */
     private static List<CigarOperator> buildOperators(String cigarString) {
 
-        java.util.List<CigarOperator> operators = new ArrayList();
+        java.util.List<CigarOperator> operators = new ArrayList<>();
         StringBuilder buffer = new StringBuilder(4);
 
         // Create list of cigar operators
@@ -603,8 +664,9 @@ public class SAMAlignment implements Alignment {
     }
 
 
+    @Override
     public String getClipboardString(double location, int mouseX) {
-        return getAlignmentValueString(location, mouseX, (AlignmentTrack.RenderOptions) null);
+        return getAlignmentValueString(location, mouseX, null);
     }
 
     private Integer positionToReadIndex(double position) {
@@ -616,6 +678,7 @@ public class SAMAlignment implements Alignment {
         return null;
     }
 
+    @Override
     public String getAlignmentValueString(double position, int mouseX, AlignmentTrack.RenderOptions renderOptions) {
 
         boolean truncate = renderOptions != null;
@@ -644,6 +707,10 @@ public class SAMAlignment implements Alignment {
                             buf.append("Insertion (" + bases.length + " bases): " + new String(bases.copyOfRange(0, 25)) + "..." +
                                     new String(bases.copyOfRange(len - 25, len)) + "<br>");
                         }
+
+                        // extended annotation?
+                        if (flowBlockAnnotator.handlesBlocks(block))
+                            flowBlockAnnotator.appendBlockQualityAnnotation(this, block, buf);
                     }
                     atInsertion = true;
                 }
@@ -662,7 +729,7 @@ public class SAMAlignment implements Alignment {
                         for (BaseModificationSet bmSet : baseModificationSets) {
                             if (bmSet.containsPosition(p)) {
                                 if (modString.length() > 0) modString += "<br>";
-                                modString += BaseModificationUtils.valueString(bmSet.getModification(), bmSet.getLikelihoods().get(p));
+                                modString += bmSet.valueString(p);
                             }
                         }
                         if (modString.length() > 0) {
@@ -731,11 +798,10 @@ public class SAMAlignment implements Alignment {
         String cigarString = getCigarString();
         // Abbreviate long CIGAR strings.  Retain the start and end of the CIGAR, which show
         // clipping; trim the middle.
-        int maxCigarStringLength = 1000;
-        if (cigarString.length() > maxCigarStringLength) {
+        if (cigarString.length() > MAX_CIGAR_STRING_LENGTH_TO_DISPLAY) {
             // Match only full <length><operator> pairs at the beginning and end of the string.
-            Matcher lMatcher = Pattern.compile("^(.{1," + Integer.toString(maxCigarStringLength / 2 - 1) + "}[A-Z])").matcher(cigarString);
-            Matcher rMatcher = Pattern.compile("[A-Z](.{1," + Integer.toString(maxCigarStringLength / 2) + "})$").matcher(cigarString);
+            Matcher lMatcher = LEFT_CIGAR_PATTERN.matcher(cigarString);
+            Matcher rMatcher = RIGHT_CIGAR_PATTERN.matcher(cigarString);
             cigarString = (lMatcher.find() ? lMatcher.group(1) : "") + "..." + (rMatcher.find() ? rMatcher.group(1) : "");
         }
 
@@ -749,37 +815,27 @@ public class SAMAlignment implements Alignment {
         buf.append("Clipping = ");
 
         // Identify the number of hard and soft clipped bases.
-        Matcher lclipMatcher = Pattern.compile("^(([0-9]+)H)?(([0-9]+)S)?").matcher(cigarString);
-        Matcher rclipMatcher = Pattern.compile("(([0-9]+)S)?(([0-9]+)H)?$").matcher(cigarString);
-        int lclipHard = 0, lclipSoft = 0, rclipHard = 0, rclipSoft = 0;
-        if (lclipMatcher.find()) {
-            lclipHard = lclipMatcher.group(2) == null ? 0 : Integer.parseInt(lclipMatcher.group(2), 10);
-            lclipSoft = lclipMatcher.group(4) == null ? 0 : Integer.parseInt(lclipMatcher.group(4), 10);
-        }
-        if (rclipMatcher.find()) {
-            rclipHard = rclipMatcher.group(4) == null ? 0 : Integer.parseInt(rclipMatcher.group(4), 10);
-            rclipSoft = rclipMatcher.group(2) == null ? 0 : Integer.parseInt(rclipMatcher.group(2), 10);
-        }
+        ClippingCounts clipping = getClippingCounts();
 
-        if (lclipHard + lclipSoft + rclipHard + rclipSoft == 0) {
+        if (!clipping.isClipped()) {
             buf.append("None");
         } else {
-            if (lclipHard + lclipSoft > 0) {
+            if (clipping.isLeftClipped()) {
                 buf.append("Left");
-                if (lclipHard > 0) {
-                    buf.append(" " + Globals.DECIMAL_FORMAT.format(lclipHard) + " hard");
+                if (clipping.getLeftHard() > 0) {
+                    buf.append(" " + Globals.DECIMAL_FORMAT.format(clipping.getLeftHard()) + " hard");
                 }
-                if (lclipSoft > 0) {
-                    buf.append(" " + Globals.DECIMAL_FORMAT.format(lclipSoft) + " soft");
+                if (clipping.getLeftSoft() > 0) {
+                    buf.append(" " + Globals.DECIMAL_FORMAT.format(clipping.getLeftSoft()) + " soft");
                 }
             }
-            if (rclipHard + rclipSoft > 0) {
-                buf.append((lclipHard + lclipSoft > 0 ? "; " : "") + "Right");
-                if (rclipHard > 0) {
-                    buf.append(" " + Globals.DECIMAL_FORMAT.format(rclipHard) + " hard");
+            if (clipping.isRightClipped()) {
+                buf.append((clipping.isLeftClipped() ? "; " : "") + "Right");
+                if (clipping.getRightHard() > 0) {
+                    buf.append(" " + Globals.DECIMAL_FORMAT.format(clipping.getRightHard()) + " hard");
                 }
-                if (rclipSoft > 0) {
-                    buf.append(" " + Globals.DECIMAL_FORMAT.format(rclipSoft) + " soft");
+                if (clipping.getRightSoft() > 0) {
+                    buf.append(" " + Globals.DECIMAL_FORMAT.format(clipping.getRightSoft()) + " soft");
                 }
             }
         }
@@ -809,10 +865,10 @@ public class SAMAlignment implements Alignment {
             }
         }
 
-        Object suppAlignment = this.getAttribute("SA");
+        Object suppAlignment = this.getAttribute(SAMTag.SA);
         if (suppAlignment != null) {
             buf.append("----------------------<br>");
-            buf.append(getSupplAlignmentString(suppAlignment.toString()));
+            buf.append(getSupplAlignmentString());
             buf.append("<br>");
         }
 
@@ -846,8 +902,10 @@ public class SAMAlignment implements Alignment {
 
                 byte quality = block.getQuality(offset);
                 buf.append("Location = " + getChr() + ":" + Globals.DECIMAL_FORMAT.format(1 + (long) position) + "<br>");
-                buf.append("Base = " + (char) base + " @ QV " + Globals.DECIMAL_FORMAT.format(quality) + "<br>");
-
+                buf.append("Base = " + (char) base + " @ QV " + Globals.DECIMAL_FORMAT.format(quality));
+                if (FlowUtil.isFlow(this) && flowBlockAnnotator.handlesBlocks(block))
+                    flowBlockAnnotator.appendBlockAttrAnnotation(this, block, offset, buf);
+                buf.append("<br>");
                 break;
             }
         }
@@ -931,30 +989,44 @@ public class SAMAlignment implements Alignment {
 
     private String getMlTagString(SAMRecord.SAMTagAndValue tag) {
         byte[] bytes = (byte[]) tag.value;
-        String buf = "";
+        StringBuilder buf = new StringBuilder();
         for (int i = 0; i < bytes.length; i++) {
-            if (i > 0) buf += ",";
-            buf += Byte.toUnsignedInt(bytes[i]);
-        }
-        return buf;
-    }
-
-    private String getSupplAlignmentString(String sa) {
-
-        StringBuffer buf = new StringBuffer();
-        buf.append("SupplementaryAlignments");
-        String[] records = Globals.semicolonPattern.split(sa);
-        for (String rec : records) {
-            try {
-                SupplementaryAlignment a = new SupplementaryAlignment(rec);
-                buf.append("<br>" + a.printString());
-            } catch (Exception e) {
-                buf.append("<br>* Invalid SA entry (not listed) *");
-            }
+            if (i > 0) buf.append(",");
+            buf.append(Byte.toUnsignedInt(bytes[i]));
         }
         return buf.toString();
     }
 
+    private String getSupplAlignmentString() {
+        StringBuilder sb = new StringBuilder();
+        sb.append("Supplementary Alignments");
+        final List<SupplementaryAlignment> supplementaryAlignments = getSupplementaryAlignments();
+        final int insertionIndex = SupplementaryAlignment.getInsertionIndex(this, supplementaryAlignments);
+        int i = 0;
+        for (SupplementaryAlignment sa : supplementaryAlignments) {
+            if (i == insertionIndex) { //Add this read into the list
+                sb.append(getThisReadDescriptionForSAList());
+            }
+            i++;
+            try {
+                sb.append("<br>" + sa.printString());
+            } catch (Exception e) {
+                sb.append("<br>* Invalid SA entry (not listed) *");
+            }
+        }
+        if (i == insertionIndex) {
+            sb.append(getThisReadDescriptionForSAList());
+        }
+        return sb.toString();
+    }
+
+    private String getThisReadDescriptionForSAList() {
+        return "<br>"
+                + "<b>"
+                + chr + ":" + Globals.DECIMAL_FORMAT.format(getAlignmentStart() + 1) + "-" + Globals.DECIMAL_FORMAT.format(getAlignmentEnd())
+                + " (" + this.getReadStrand().toShortString() + ") = " + Globals.DECIMAL_FORMAT.format(getLengthOnReference()) + "bp  @MAPQ " + this.getMappingQuality() + " NM" + getAttribute(SAMTag.NM)
+                + "</b>";
+    }
 
     public float getScore() {
         return getMappingQuality();
@@ -989,12 +1061,6 @@ public class SAMAlignment implements Alignment {
         return gaps;
     }
 
-
-    @Override
-    public void finish() {
-
-    }
-
     @Override
     public AlignmentBlock getInsertionAt(int position) {
         for (AlignmentBlock block : insertions) {
@@ -1117,18 +1183,18 @@ public class SAMAlignment implements Alignment {
     public String getSynopsisString() {
 
         char st = isNegativeStrand() ? '-' : '+';
-        Object nm = getAttribute("NM");
+        Object nm = getAttribute(SAMTag.NM);
         String numMismatches = nm == null ? "?" : nm.toString();
         int lenOnRef = getAlignmentEnd() - getAlignmentStart();
 
-        int[] clipping = getClipping(getCigarString());
+        ClippingCounts clipping = getClippingCounts();
         String clippingString = "";
-        if (clipping[0] + clipping[1] + clipping[2] + clipping[3] > 0) {
-            if (clipping[0] > 0) clippingString += clipping[0] + "H";
-            if (clipping[1] > 0) clippingString += clipping[1] + "S";
+        if (clipping.isClipped()) {
+            if (clipping.getLeftHard() > 0) clippingString += clipping.getLeftHard() + "H";
+            if (clipping.getLeftSoft() > 0) clippingString += clipping.getLeftSoft() + "S";
             clippingString += " ... ";
-            if (clipping[3] > 0) clippingString += clipping[3] + "S";
-            if (clipping[2] > 0) clippingString += clipping[2] + "H";
+            if (clipping.getRightSoft() > 0) clippingString += clipping.getRightSoft() + "S";
+            if (clipping.getRightHard() > 0) clippingString += clipping.getRightHard() + "H";
         }
 
         return chr + ":" + Globals.DECIMAL_FORMAT.format(getAlignmentStart()) + "-" +
@@ -1138,22 +1204,6 @@ public class SAMAlignment implements Alignment {
 
     }
 
-    public static int[] getClipping(String cigarString) {
-        // Identify the number of hard and soft clipped bases.
-        Matcher lclipMatcher = Pattern.compile("^(([0-9]+)H)?(([0-9]+)S)?").matcher(cigarString);
-        Matcher rclipMatcher = Pattern.compile("(([0-9]+)S)?(([0-9]+)H)?$").matcher(cigarString);
-        int lclipHard = 0, lclipSoft = 0, rclipHard = 0, rclipSoft = 0;
-        if (lclipMatcher.find()) {
-            lclipHard = lclipMatcher.group(2) == null ? 0 : Integer.parseInt(lclipMatcher.group(2), 10);
-            lclipSoft = lclipMatcher.group(4) == null ? 0 : Integer.parseInt(lclipMatcher.group(4), 10);
-        }
-        if (rclipMatcher.find()) {
-            rclipHard = rclipMatcher.group(4) == null ? 0 : Integer.parseInt(rclipMatcher.group(4), 10);
-            rclipSoft = rclipMatcher.group(2) == null ? 0 : Integer.parseInt(rclipMatcher.group(2), 10);
-        }
-        return new int[]{lclipHard, lclipSoft, rclipHard, rclipSoft};
-    }
-
     private static boolean operatorIsMatch(boolean showSoftClipped, char operator) {
         return operator == MATCH || operator == PERFECT_MATCH || operator == MISMATCH
                 || (showSoftClipped && operator == SOFT_CLIP);
@@ -1186,4 +1236,16 @@ public class SAMAlignment implements Alignment {
         return hapDistance;
     }
 
+    public List<SupplementaryAlignment> getSupplementaryAlignments() {
+        return getCachedOrCompute(CacheKey.SA_GROUP,
+                () -> {
+                    Object rawSAValue = this.getAttribute(SAMTag.SA);
+                    if (rawSAValue == null) {
+                        return null;
+                    } else {
+                        final List<SupplementaryAlignment> sas = Collections.unmodifiableList(SupplementaryAlignment.parseFromSATag(rawSAValue.toString()));
+                        return sas;
+                    }
+                });
+    }
 }
