@@ -45,6 +45,8 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import static org.broad.igv.ui.IGV.DATA_PANEL_NAME;
+
 /**
  * Class to parse a UCSC session file
  *
@@ -84,7 +86,7 @@ public class UCSCSessionReader implements SessionReader {
         igv.resetSession(sessionPath);
         Genome genome = GenomeManager.getInstance().getCurrentGenome();
         if (genome != null) {  // Can this ever be null?
-              GenomeManager.getInstance().restoreGenomeTracks(GenomeManager.getInstance().getCurrentGenome());
+            GenomeManager.getInstance().restoreGenomeTracks(GenomeManager.getInstance().getCurrentGenome());
         }
 
         while ((nextLine = reader.readLine()) != null) {
@@ -111,6 +113,9 @@ public class UCSCSessionReader implements SessionReader {
                     // Alignment tracks must be loaded synchronously
                     if (isAlignmentFile(locator.getPath())) {
                         TrackPanel panel = igv.getPanelFor(locator);
+                        if (panel == null) {
+                            panel = igv.getTrackPanel(DATA_PANEL_NAME);
+                        }
                         panel.addTracks(igv.load(locator));
                     } else {
                         aSync.add(locator);
