@@ -26,7 +26,9 @@
 package org.broad.igv.track;
 
 import htsjdk.tribble.Feature;
+import htsjdk.tribble.NamedFeature;
 import htsjdk.tribble.TribbleException;
+import org.broad.igv.event.IGVEvent;
 import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.event.DataLoadedEvent;
@@ -73,7 +75,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     private static Logger log = LogManager.getLogger(FeatureTrack.class);
 
 
-    public static final int MINIMUM_FEATURE_SPACING = 5;
+    public static final int MINIMUM_FEATURE_SPACING = 0;
     public static final int DEFAULT_MARGIN = 5;
     public static final int NO_FEATURE_ROW_SELECTED = -1;
     protected static final Color SELECTED_FEATURE_ROW_COLOR = new Color(100, 100, 100, 30);
@@ -222,7 +224,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     /**
      * Called after features are finished loading, which can be asynchronous
      */
-    public void receiveEvent(Object e) {
+    public void receiveEvent(IGVEvent e) {
         if (e instanceof DataLoadedEvent) {
 //            DataLoadedEvent event = (DataLoadedEvent) e;
 //            if (IGV.hasInstance()) {
@@ -363,7 +365,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
     public Renderer getRenderer() {
         if (renderer == null) {
-            setRendererClass(IGVFeatureRenderer.class);
+            setRenderer(new IGVFeatureRenderer());
         }
         return renderer;
     }
@@ -378,7 +380,6 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
      */
     public String getValueStringAt(String chr, double position, int mouseX, int mouseY, ReferenceFrame frame) {
 
-
         if (showFeatures) {
 
             List<Feature> allFeatures = getAllFeatureAt(position, mouseY, frame);
@@ -388,7 +389,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
             StringBuffer buf = new StringBuffer();
             boolean firstFeature = true;
-            int maxNumber = 100;
+            int maxNumber = IGV.getInstance().isShowDetailsOnClick() ? 100 : 10;
             int n = 1;
             for (Feature feature : allFeatures) {
                 if (feature != null && feature instanceof IGVFeature) {
@@ -406,17 +407,14 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
                             buf.append("<br/><a href=\"" + url + "\">" + url + "</a>");
                         }
                     }
-
                     firstFeature = false;
-
                     if (n > maxNumber) {
-                        buf.append("...");
+                        buf.append("<hr><br<b>>+ " + (allFeatures.size() - maxNumber) +  " more ...</b>");
                         break;
                     }
                 }
                 n++;
             }
-
             return buf.toString();
         } else {
             int zoom = Math.max(0, frame.getZoom());
@@ -562,9 +560,8 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
         if (possFeatures != null) {
             // give a minum 2 pixel or 1/2 bp window, otherwise very narrow features will be missed.
             double bpPerPixel = frame.getScale();
-            double minWidth = Math.max(0.5, MINIMUM_FEATURE_SPACING * bpPerPixel);
-            int maxFeatureLength = packedFeatures.getMaxFeatureLength();
-            featureList = FeatureUtils.getAllFeaturesAt(position, maxFeatureLength, minWidth, possFeatures);
+            double flanking = 4 * bpPerPixel;
+            featureList = FeatureUtils.getAllFeaturesAt(position, flanking, possFeatures);
         }
         return featureList;
     }
@@ -1060,6 +1057,16 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
     }
 
+    @Override
+    public boolean isSearchable() {
+        return source != null && source.isSearchable();
+    }
+
+    @Override
+    public NamedFeature search(String token) {
+        return source.search(token);
+    }
+
     @Override
     public void unmarshalXML(Element element, Integer version) {
 
