@@ -28,10 +28,8 @@ package org.broad.igv.track;
 import htsjdk.tribble.AsciiFeatureCodec;
 import htsjdk.tribble.Feature;
 import htsjdk.variant.vcf.VCFHeader;
-import org.broad.igv.bbfile.BBFileReader;
 import org.broad.igv.bedpe.BedPEParser;
 import org.broad.igv.bedpe.InteractionTrack;
-import org.broad.igv.bbfile.BBDataSource;
 import org.broad.igv.blast.BlastMapping;
 import org.broad.igv.blast.BlastParser;
 import org.broad.igv.data.*;
@@ -58,8 +56,7 @@ import org.broad.igv.feature.tribble.CodecFactory;
 import org.broad.igv.feature.tribble.FeatureFileHeader;
 import org.broad.igv.feature.tribble.GFFCodec;
 import org.broad.igv.feature.tribble.TribbleIndexNotFoundException;
-import org.broad.igv.util.GoogleUtils;
-import org.broad.igv.gwas.GWASFeature;
+import org.broad.igv.gwas.GWASData;
 import org.broad.igv.gwas.GWASParser;
 import org.broad.igv.gwas.GWASTrack;
 import org.broad.igv.htsget.HtsgetUtils;
@@ -69,15 +66,20 @@ import org.broad.igv.lists.GeneListManager;
 import org.broad.igv.logging.LogManager;
 import org.broad.igv.logging.Logger;
 import org.broad.igv.maf.MultipleAlignmentTrack;
-import org.broad.igv.methyl.MethylTrack;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.renderer.HeatmapRenderer;
 import org.broad.igv.renderer.MutationRenderer;
 import org.broad.igv.renderer.PointsRenderer;
-import org.broad.igv.sam.*;
+import org.broad.igv.sam.AlignmentDataManager;
+import org.broad.igv.sam.AlignmentTrack;
+import org.broad.igv.sam.EWigTrack;
 import org.broad.igv.sam.reader.IndexNotFoundException;
 import org.broad.igv.tdf.TDFDataSource;
 import org.broad.igv.tdf.TDFReader;
+import org.broad.igv.ucsc.Trix;
+import org.broad.igv.ucsc.bb.BBDataSource;
+import org.broad.igv.ucsc.bb.BBFeatureSource;
+import org.broad.igv.ucsc.bb.BBFile;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.util.ConfirmDialog;
 import org.broad.igv.ui.util.ConvertFileDialog;
@@ -90,7 +92,10 @@ import org.broad.igv.variant.util.PedigreeUtils;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.StringReader;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.List;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -123,6 +128,7 @@ public class TrackLoader {
 
         log.info("Loading resource:  " + (locator.isDataURL() ? "<data url>" : path));
         try {
+
             String format = locator.getFormat();
 
             if (format.equals("tbi")) {
@@ -133,8 +139,21 @@ public class TrackLoader {
             //This list will hold all new tracks created for this locator
             List<Track> newTracks = new ArrayList<Track>();
 
+            // Determine track type, if possible, and add new tracks
             if (locator.isHtsget()) {
-                tryHtsget(locator, newTracks, genome);
+                HtsgetUtils.Metadata htsgetMeta = HtsgetUtils.getMetadata(locator.getPath());
+                locator.setFormat(htsgetMeta.getFormat().toLowerCase());
+                if (htsgetMeta.getFormat().equals("VCF")) {
+                    locator.setHtsget(true);
+                    HtsgetVariantSource source = new HtsgetVariantSource(htsgetMeta, genome);
+                    loadVCFWithSource(locator, source, newTracks);
+                } else if (htsgetMeta.getFormat().equals("BAM") || htsgetMeta.getFormat().equals("CRAM")) {
+                    locator.setHtsget(true);
+                    loadAlignmentsTrack(locator, newTracks, genome);
+                } else {
+                    throw new RuntimeException("Format: '" + htsgetMeta.getFormat() + "' is not supported for htsget servers.");
+                }
+
             } else if (format.equals("gmt")) {
                 loadGMT(locator);
             } else if (format.equals("vcf.list")) {
@@ -172,7 +191,7 @@ public class TrackLoader {
             } else if (format.equals("ewig.tdf")) {
                 loadEwigIBFFile(locator, newTracks, genome);
             } else if (format.equals("bw") || format.equals("bb") || format.equals("bigwig") ||
-                    format.equals("bigbed")) {
+                    format.equals("bigbed") || format.equals("biggenepred") || format.equals("bigrepmsk")) {
                 loadBWFile(locator, newTracks, genome);
             } else if (format.equals("ibf") || format.equals("tdf")) {
                 loadTDFFile(locator, newTracks, genome);
@@ -208,32 +227,28 @@ public class TrackLoader {
             } else if (format.equals("maf")) {
                 loadMultipleAlignmentTrack(locator, newTracks, genome);
             } else {
-                //if a url, try htsget
-                boolean isHtsget = tryHtsget(locator, newTracks, genome);
-                if (!isHtsget) {
-
-                    // If the file is too large, give up
-                    // TODO -- ftp test
-                    final int tenMB = 10000000;
-                    long fileLength = ParsingUtils.getContentLength(locator.getPath());
-                    if (fileLength > tenMB) {
-                        MessageUtils.confirm("<html>Cannot determine file type of: " + locator.getPath());
-                    }
-
-                    // Read file contents and try to sort it out
-                    String contents = FileUtils.getContents(locator.getPath());
-                    BufferedReader reader = new BufferedReader(new StringReader(contents));
-
-                    if (CytoBandFileParser.isValid(reader, locator.getPath())) {
-                        Track track = new CytobandTrack(locator, new BufferedReader(new StringReader(contents)), genome);
-                        newTracks.add(track);
-                    } else if (AttributeManager.isSampleInfoFile(reader)) {
-                        // This might be a sample information file.
-                        AttributeManager.getInstance().loadSampleInfo(locator);
-                    } else {
-                        MessageUtils.showMessage("<html>Unknown file type: " + path + "<br>Check file extension");
-                    }
+                // If the file is too large, give up
+                // TODO -- ftp test
+                final int tenMB = 10000000;
+                long fileLength = ParsingUtils.getContentLength(locator.getPath());
+                if (fileLength > tenMB) {
+                    MessageUtils.confirm("<html>Cannot determine file type of: " + locator.getPath());
                 }
+
+                // Read file contents and try to sort it out
+                String contents = FileUtils.getContents(locator.getPath());
+                BufferedReader reader = new BufferedReader(new StringReader(contents));
+
+                if (CytoBandFileParser.isValid(reader, locator.getPath())) {
+                    Track track = new CytobandTrack(locator, new BufferedReader(new StringReader(contents)), genome);
+                    newTracks.add(track);
+                } else if (AttributeManager.isSampleInfoFile(reader)) {
+                    // This might be a sample information file.
+                    AttributeManager.getInstance().loadSampleInfo(locator);
+                } else {
+                    MessageUtils.showMessage("<html>Unknown file type: " + path + "<br>Check file extension");
+                }
+
             }
 
             // Track line
@@ -274,46 +289,6 @@ public class TrackLoader {
 
     }
 
-    /**
-     * Try to load as an htsget resource.  As most (all?) htsget endpoints use an https:// scheme URL, there is
-     * no way to detect other than try.
-     *
-     * @param locator
-     * @param newTracks
-     * @param genome
-     * @return
-     */
-    private boolean tryHtsget(ResourceLocator locator, List<Track> newTracks, Genome genome) {
-        boolean isHtsget = false;
-        if (locator.getPath().startsWith("https://") ||
-                locator.getPath().startsWith("http://") ||
-                locator.getPath().startsWith("htsget://")) {
-            try {
-                HtsgetUtils.Metadata htsgetMeta = HtsgetUtils.getMetadata(locator.getPath());
-                if (htsgetMeta != null) {
-                    isHtsget = true;
-                    locator.setFormat(htsgetMeta.getFormat().toLowerCase());
-                    if (htsgetMeta.getFormat().equals("VCF")) {
-                        locator.setHtsget(true);
-                        HtsgetVariantSource source = new HtsgetVariantSource(htsgetMeta, genome);
-                        loadVCFWithSource(locator, source, newTracks);
-                    } else if (htsgetMeta.getFormat().equals("BAM") || htsgetMeta.getFormat().equals("CRAM")) {
-                        locator.setHtsget(true);
-                        loadAlignmentsTrack(locator, newTracks, genome);
-                    } else {
-                        throw new RuntimeException("Format: '" + htsgetMeta.getFormat() + "' is not supported for htsget servers.");
-                    }
-                }
-            } catch (IOException e) {
-                // Not neccessarily an error, might just indicate its not an htsget server.  Not sure
-                // if this should be logged or not, it will be a common and expected occurence when loading
-                // sample information, which is checked after htsget
-                return false;
-            }
-        }
-        return isHtsget;
-    }
-
     public static boolean isAlignmentTrack(String typeString) {
         if (typeString == null) {
             return false;
@@ -329,7 +304,7 @@ public class TrackLoader {
         List<Feature> features = SMAPParser.parseFeatures(locator, genome);
         FeatureCollectionSource src = new FeatureCollectionSource(features, genome);
         FeatureTrack track = new FeatureTrack(locator, locator.getName(), src);
-        track.setRendererClass(SMAPRenderer.class);
+        track.setRenderer(new SMAPRenderer());
         track.setDisplayMode(Track.DisplayMode.EXPANDED);
         newTracks.add(track);
     }
@@ -526,7 +501,7 @@ public class TrackLoader {
             t.setTrackLine(tp.getTrackLine());
         }
 
-        t.setRendererClass(DSIRenderer.class);
+        t.setRenderer(new DSIRenderer());
 
         newTracks.add(t);
     }
@@ -544,9 +519,8 @@ public class TrackLoader {
     private void loadGWASFile(ResourceLocator locator, List<Track> newTracks, Genome genome) throws IOException {
 
         GWASParser gwasParser = new GWASParser(locator, genome);
-        Map<String, List<GWASFeature>> gwasData = gwasParser.parse();
-
-        GWASTrack gwasTrack = new GWASTrack(locator, locator.getPath(), locator.getFileName(), gwasData, gwasParser.getColumnHeaders(), genome);
+        GWASData gwasData = gwasParser.parse();
+        GWASTrack gwasTrack = new GWASTrack(locator, locator.getPath(), locator.getFileName(), gwasData, gwasParser.getColumnHeaders(), gwasParser.delimiter, genome);
         newTracks.add(gwasTrack);
 
     }
@@ -566,9 +540,7 @@ public class TrackLoader {
         ds = parser.createDataset();
         if (ds.isEmpty()) {
             String message = "The probes in the file <br>&nbsp;&nbsp;&nbsp;" + locator.getPath() + "<br>" +
-                    "could not be mapped to genomic positions.  This can be corrected by specify a probe mapping<br>" +
-                    "file from the Preferences window (Probes tab), or by specifing the genomic positions in the<br>" +
-                    "expression data file.  Please see the user guide for more details.";
+                    "could not be mapped to genomic positions.";
             MessageUtils.showMessage(message);
 
         } else {
@@ -590,8 +562,8 @@ public class TrackLoader {
             for (String trackName : ds.getTrackNames()) {
                 DatasetDataSource dataSource = new DatasetDataSource(trackName, ds, genome);
                 String trackId = path + "_" + trackName;
-                Track track = new DataSourceTrack(locator, trackId, trackName, dataSource);
-                track.setRendererClass(HeatmapRenderer.class);
+                DataSourceTrack track = new DataSourceTrack(locator, trackId, trackName, dataSource);
+                track.setRenderer(new HeatmapRenderer());
                 track.setProperties(trackProperties);
                 newTracks.add(track);
             }
@@ -644,7 +616,7 @@ public class TrackLoader {
             track.setProperties(trackProperties);
 
             if (type == TrackType.ALLELE_FREQUENCY) {
-                track.setRendererClass(PointsRenderer.class);
+                track.setRenderer(new PointsRenderer());
                 track.setHeight(40);
             }
             newTracks.add(track);
@@ -819,21 +791,16 @@ public class TrackLoader {
         String trackId = locator.getPath();
 
         String path = locator.getPath();
-        BBFileReader reader = new BBFileReader(path);
-        BBDataSource bigwigSource = new BBDataSource(reader, genome);
-        Track track = null;
-
+        String trixURL = locator.getTrixURL();
+        BBFile reader = trixURL == null ? new BBFile(path, genome) : new BBFile(path, genome, trixURL);
+        Track track;
         if (reader.isBigWigFile()) {
+            BBDataSource bigwigSource = new BBDataSource(reader, genome);
             track = new DataSourceTrack(locator, trackId, trackName, bigwigSource);
         } else if (reader.isBigBedFile()) {
+            BBFeatureSource featureSource = new BBFeatureSource(reader, genome);
+            track = new FeatureTrack(locator, trackId, trackName, featureSource);
 
-            if (locator.getPath().contains("RRBS_cpgMethylation") ||
-                    locator.getPath().contains("BiSeq_cpgMethylation") ||
-                    (reader.getAutoSql() != null && reader.getAutoSql().startsWith("table BisulfiteSeq"))) {
-                loadMethylTrack(locator, reader, newTracks, genome);
-            } else {
-                track = new FeatureTrack(locator, trackId, trackName, bigwigSource);
-            }
         } else {
             throw new RuntimeException("Unknown BIGWIG type: " + locator.getPath());
         }
@@ -844,12 +811,6 @@ public class TrackLoader {
 
     }
 
-    private void loadMethylTrack(ResourceLocator locator, BBFileReader reader, List<Track> newTracks, Genome genome) throws IOException {
-
-        MethylTrack track = new MethylTrack(locator, reader, genome);
-        newTracks.add(track);
-    }
-
     private void loadEwigIBFFile(ResourceLocator locator, List<Track> newTracks, Genome genome) {
 
         TDFReader reader = TDFReader.getReader(locator.getPath());
@@ -913,7 +874,14 @@ public class TrackLoader {
             // not represented in the genome, buf if there are no matches warn the user.
             List<String> seqNames = dataManager.getSequenceNames();
             if (seqNames != null && seqNames.size() > 0) {
-                if (!dataManager.hasMatchingSequences()) {
+                boolean seqMatch = false;
+                for (String seq : seqNames) {
+                    if (genome.getAliasRecord(seq) != null) {
+                        seqMatch = true;
+                        break;
+                    }
+                }
+                if (!seqMatch) {
                     showMismatchSequenceNameMessage(locator.getPath(), genome, seqNames);
                 }
             }
@@ -990,7 +958,7 @@ public class TrackLoader {
                 break;
             }
         }
-        if (genome != null && genome.getAllChromosomeNames() != null) {
+        if (genome != null && genome.getAllChromosomeNames() != null && genome.getAllChromosomeNames().size() > 0) {
             message.append("<br>Genome: ");
             n = 0;
             for (String cn : genome.getAllChromosomeNames()) {
@@ -1019,7 +987,7 @@ public class TrackLoader {
         List<FeatureTrack> mutationTracks = loader.loadMutationTracks(locator, genome);
         for (FeatureTrack track : mutationTracks) {
             track.setTrackType(TrackType.MUTATION);
-            track.setRendererClass(MutationRenderer.class);
+            track.setRenderer(new MutationRenderer());
             newTracks.add(track);
         }
     }
@@ -1077,7 +1045,7 @@ public class TrackLoader {
             String trackId = path + "_" + trackName;
             SegmentedDataSource dataSource = new SegmentedDataSource(trackName, ds);
             DataSourceTrack track = new DataSourceTrack(locator, trackId, trackName, dataSource);
-            track.setRendererClass(HeatmapRenderer.class);
+            track.setRenderer(new HeatmapRenderer());
             track.setTrackType(ds.getType());
 
             if (props != null) {
