@@ -43,7 +43,6 @@ import org.broad.igv.batch.BatchRunner;
 import org.broad.igv.batch.CommandListener;
 import org.broad.igv.event.*;
 import org.broad.igv.exceptions.DataLoadException;
-import org.broad.igv.feature.MaximumContigGenomeException;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.feature.Strand;
@@ -51,6 +50,7 @@ import org.broad.igv.feature.genome.*;
 import org.broad.igv.lists.GeneList;
 import org.broad.igv.logging.LogManager;
 import org.broad.igv.logging.Logger;
+import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesEditor;
 import org.broad.igv.prefs.PreferencesManager;
@@ -58,6 +58,8 @@ import org.broad.igv.sam.AlignmentTrack;
 import org.broad.igv.sam.InsertionSelectionEvent;
 import org.broad.igv.sam.SortOption;
 import org.broad.igv.session.*;
+import org.broad.igv.session.autosave.AutosaveTimerTask;
+import org.broad.igv.session.autosave.SessionAutosaveManager;
 import org.broad.igv.track.*;
 import org.broad.igv.ui.WaitCursorManager.CursorToken;
 import org.broad.igv.ui.commandbar.GenomeListManager;
@@ -81,6 +83,7 @@ import java.io.InputStream;
 import java.net.URL;
 import java.util.List;
 import java.util.*;
+import java.util.Timer;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -126,6 +129,11 @@ public class IGV implements IGVEventObserver {
      */
     private Session session;
 
+    /**
+     * Timer for triggering periodic autosave of current session
+     */
+    private Timer sessionAutosaveTimer = new Timer();
+
     // Misc state
     private Map<String, List<Track>> overlayTracksMap = new HashMap();
     private Set<Track> overlaidTracks = new HashSet();
@@ -249,6 +257,12 @@ public class IGV implements IGVEventObserver {
         mainFrame.setBounds(applicationBounds);
 
         subscribeToEvents();
+
+        // Start running periodic autosaves (unless the user has specified not to retain timed autosaves)
+        if (PreferencesManager.getPreferences().getAsInt(Constants.AUTOSAVES_TO_KEEP) > 0) {
+            int timerDelay = PreferencesManager.getPreferences().getAsInt(AUTOSAVE_FREQUENCY) * 60000; // Convert timer delay to ms
+            sessionAutosaveTimer.scheduleAtFixedRate(new AutosaveTimerTask(this), timerDelay, timerDelay);
+        }
     }
 
     public JRootPane getRootPane() {
@@ -318,71 +332,6 @@ public class IGV implements IGVEventObserver {
         contentPane.getCommandBar().selectGenome(genomeId);
     }
 
-
-    public void defineGenome(javax.swing.ProgressMonitor monitor) {
-
-        ProgressBar.ProgressDialog progressDialog = null;
-        File archiveFile = null;
-
-        try {
-            GenomeBuilderDialog genomeBuilderDialog = new GenomeBuilderDialog(mainFrame, this);
-            genomeBuilderDialog.setVisible(true);
-
-            File genomeZipFile = genomeBuilderDialog.getArchiveFile();
-            if (genomeBuilderDialog.isCanceled() || genomeZipFile == null) {
-                return;
-            }
-
-
-            String cytobandFileName = genomeBuilderDialog.getCytobandFileName();
-            String geneAnnotFileName = genomeBuilderDialog.getGeneAnnotFileName();
-            String fastaFileName = genomeBuilderDialog.getFastaFileName();
-            String chrAliasFile = genomeBuilderDialog.getChrAliasFileName();
-            String genomeDisplayName = genomeBuilderDialog.getGenomeDisplayName();
-            String genomeId = genomeBuilderDialog.getGenomeId();
-
-            GenomeListItem genomeListItem = GenomeManager.getInstance().defineGenome(
-                    genomeZipFile, cytobandFileName, geneAnnotFileName,
-                    fastaFileName, chrAliasFile, genomeDisplayName,
-                    genomeId, monitor);
-
-            if (genomeListItem != null) {
-                contentPane.getCommandBar().refreshGenomeListComboBox();
-                contentPane.getCommandBar().selectGenome(genomeListItem.getId());
-            }
-            if (monitor != null) {
-                monitor.setProgress(100);
-            }
-
-        } catch (MaximumContigGenomeException e) {
-
-            String genomePath = "";
-            if (archiveFile != null) {
-                genomePath = archiveFile.getAbsolutePath();
-            }
-
-            log.error("Failed to define genome: " + genomePath, e);
-
-            JOptionPane.showMessageDialog(mainFrame, "Failed to define genome " +
-                    genomePath + "\n" + e.getMessage());
-        } catch (GenomeException e) {
-            log.error("Failed to define genome.", e);
-            MessageUtils.showMessage(e.getMessage());
-        } catch (Exception e) {
-            String genomePath = "";
-            if (archiveFile != null) {
-                genomePath = archiveFile.getAbsolutePath();
-            }
-
-            log.error("Failed to define genome: " + genomePath, e);
-            MessageUtils.showMessage("Unexpected error while importing a genome: " + e.getMessage());
-        } finally {
-            if (progressDialog != null) {
-                progressDialog.setVisible(false);
-            }
-        }
-    }
-
     public void enableExtrasMenu() {
 
         menuBar.enableExtrasMenu();
@@ -534,7 +483,7 @@ public class IGV implements IGVEventObserver {
         try {
             PreferencesEditor.open(this.mainFrame);
         } catch (Exception e) {
-            log.error("Error openining preference dialog", e);
+            log.error("Error opening preference dialog", e);
         }
     }
 
@@ -561,6 +510,20 @@ public class IGV implements IGVEventObserver {
             PreferencesManager.getPreferences().remove(RECENT_SESSIONS);
             PreferencesManager.getPreferences().setRecentSessions(recentSessions);
         }
+
+        // Stop the timer that is triggering the timed autosave
+        this.stopTimedAutosave();
+
+        // Autosave current session if configured to do so
+        if (PreferencesManager.getPreferences().getAsBoolean(AUTOSAVE_ON_EXIT)) {
+            try {
+                SessionAutosaveManager.saveExitSessionAutosaveFile(session);
+            } catch (Exception e) {
+                log.error("Error autosaving session", e);
+            }
+        }
+
+
     }
 
     final public void doShowAttributeDisplay(boolean enableAttributeView) {
@@ -909,13 +872,33 @@ public class IGV implements IGVEventObserver {
     }
 
 
+    /**
+     * Scroll panel(s) containing track with specified name to the top.  Supports batch command "scrolltotrack" *
+     *
+     * @param trackName
+     * @return
+     */
     public boolean scrollToTrack(String trackName) {
+        boolean found = false;
         for (TrackPanel tp : getTrackPanels()) {
             if (tp.getScrollPane().getNamePanel().scrollTo(trackName)) {
-                return true;
+                found = true;
             }
         }
-        return false;
+        return found;
+    }
+
+
+    /**
+     * Scroll all panels to the top (position 0).  Supports batch command "scrolltotop"
+     *
+     * @return
+     */
+    public void scrollToTop() {
+        for (TrackPanel tp : getTrackPanels()) {
+            tp.getScrollPane().getNamePanel().scrollToPosition(0);
+
+        }
     }
 
 
@@ -1000,7 +983,7 @@ public class IGV implements IGVEventObserver {
             String message = "Error loading session session: " + e.getMessage();
             MessageUtils.showMessage(message);
             recentSessionList.remove(sessionPath);
-            log.error(message);
+            log.error(e);
             return false;
         } finally {
             if (inputStream != null) {
@@ -1111,10 +1094,6 @@ public class IGV implements IGVEventObserver {
         contentPane.getMainPanel().removeDataPanel(name);
     }
 
-    public void layoutMainPanel() {
-        contentPane.getMainPanel().doLayout();
-    }
-
     public MainPanel getMainPanel() {
         return contentPane.getMainPanel();
     }
@@ -1123,10 +1102,6 @@ public class IGV implements IGVEventObserver {
         return recentSessionList;
     }
 
-    public void setRecentSessionList(LinkedList<String> recentSessionList) {
-        this.recentSessionList = recentSessionList;
-    }
-
     public IGVContentPane getContentPane() {
         return contentPane;
     }
@@ -1177,7 +1152,7 @@ public class IGV implements IGVEventObserver {
 
             try {
                 List<Track> tracks = load(locator);
-                addTracks(tracks, locator);
+                addTracks(tracks);
             } catch (Exception e) {
                 log.error("Error loading track", e);
                 messages.append("Error loading " + locator + ": " + e.getMessage());
@@ -1211,12 +1186,11 @@ public class IGV implements IGVEventObserver {
      * is chosen based on characteristics of the {@code locator}.
      *
      * @param tracks
-     * @param locator
      */
-    public void addTracks(List<Track> tracks, ResourceLocator locator) {
+    public void addTracks(List<Track> tracks) {
 
         if (tracks.size() > 0) {
-            String path = locator.getPath();
+            String path = tracks.get(0).getResourceLocator().getPath();//  locator.getPath();
             Track representativeTrack = tracks.get(0);
 
             // Get an appropriate panel.  If its a VCF file create a new panel if the number of genotypes
@@ -1244,26 +1218,41 @@ public class IGV implements IGVEventObserver {
      */
     public List<Track> load(ResourceLocator locator) throws DataLoadException {
 
-        TrackLoader loader = new TrackLoader();
-        Genome genome = GenomeManager.getInstance().getCurrentGenome();
-        List<Track> newTracks = loader.load(locator, genome);
-        if (newTracks.size() > 0) {
-            for (Track track : newTracks) {
-                String fn = locator.getPath();
-                int lastSlashIdx = fn.lastIndexOf("/");
-                if (lastSlashIdx < 0) {
-                    lastSlashIdx = fn.lastIndexOf("\\");
+        try {
+            if (IGV.hasInstance()) {
+                IGV.getInstance().setStatusBarMessage3("Loading " + locator.getPath());
+            }
+
+            TrackLoader loader = new TrackLoader();
+            Genome genome = GenomeManager.getInstance().getCurrentGenome();
+            List<Track> newTracks = loader.load(locator, genome);
+            if (newTracks.size() > 0) {
+                for (Track track : newTracks) {
+                    String fn = locator.getPath();
+                    int lastSlashIdx = fn.lastIndexOf("/");
+                    if (lastSlashIdx < 0) {
+                        lastSlashIdx = fn.lastIndexOf("\\");
+                    }
+                    if (lastSlashIdx > 0) {
+                        fn = fn.substring(lastSlashIdx + 1);
+                    }
+                    track.setAttributeValue(Globals.TRACK_NAME_ATTRIBUTE, track.getName());
+                    track.setAttributeValue(Globals.TRACK_DATA_FILE_ATTRIBUTE, fn);
+                    track.setAttributeValue(Globals.TRACK_DATA_TYPE_ATTRIBUTE, track.getTrackType().toString());
+
+                    TrackProperties properties = locator.getTrackProperties();
+                    if (properties != null) {
+                        track.setProperties(properties);
+                    }
                 }
-                if (lastSlashIdx > 0) {
-                    fn = fn.substring(lastSlashIdx + 1);
-                }
-                track.setAttributeValue(Globals.TRACK_NAME_ATTRIBUTE, track.getName());
-                track.setAttributeValue(Globals.TRACK_DATA_FILE_ATTRIBUTE, fn);
-                track.setAttributeValue(Globals.TRACK_DATA_TYPE_ATTRIBUTE, track.getTrackType().toString());
+            }
+            //revalidateTrackPanels();
+            return newTracks;
+        } finally {
+            if (IGV.hasInstance()) {
+                IGV.getInstance().setStatusBarMessage3("");
             }
         }
-        //revalidateTrackPanels();
-        return newTracks;
     }
 
 
@@ -1306,7 +1295,7 @@ public class IGV implements IGVEventObserver {
             }
         }
 
-        if(track.getClass() == FeatureTrack.class && !PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY))
+        if (track.getClass() == FeatureTrack.class && !PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY))
             return getTrackPanel(FEATURE_PANEL_NAME);
         else {
             return getTrackPanel(DATA_PANEL_NAME);
@@ -1323,7 +1312,9 @@ public class IGV implements IGVEventObserver {
     public TrackPanel getPanelFor(ResourceLocator locator) {
 
         final String format = locator.getFormat();
-        if ("alist".equals(format)) {
+        if (locator.getPanelName() != null) {
+            return getTrackPanel(locator.getPanelName());
+        } else if ("alist".equals(format)) {
             return getVcfBamPanel();
         } else if (PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY)) {
             return getTrackPanel(DATA_PANEL_NAME);
@@ -1378,14 +1369,14 @@ public class IGV implements IGVEventObserver {
 
 
     public void sortAlignmentTracks(SortOption option, String tag, final boolean invertSort) {
-        sortAlignmentTracks(option, null, tag, invertSort);
+        sortAlignmentTracks(option, null, tag, invertSort, null);
     }
 
-    public void sortAlignmentTracks(SortOption option, Double location, String tag, boolean invertSort) {
+    public void sortAlignmentTracks(SortOption option, Double location, String tag, boolean invertSort, Set<String> priorityRecords) {
         List<AlignmentTrack> alignmentTracks = getAllTracks().stream()
                 .filter(track -> track instanceof AlignmentTrack)
-                .map(track -> (AlignmentTrack)track)
-                .peek(track -> track.sortRows(option, location, tag, invertSort))
+                .map(track -> (AlignmentTrack) track)
+                .peek(track -> track.sortRows(option, location, tag, invertSort, priorityRecords))
                 .collect(Collectors.toList());
         this.repaint(alignmentTracks);
     }
@@ -1659,26 +1650,21 @@ public class IGV implements IGVEventObserver {
     /**
      * Add gene and sequence tracks.  This is called upon switching genomes.
      *
-     * @param newGeneTrack
      * @param
      */
-    public void setGenomeTracks(Track newGeneTrack) {
+    public void setSequenceTrack() {
 
         TrackPanel panel = PreferencesManager.getPreferences().getAsBoolean(SHOW_SINGLE_TRACK_PANE_KEY) ?
                 getTrackPanel(DATA_PANEL_NAME) : getTrackPanel(FEATURE_PANEL_NAME);
         SequenceTrack newSeqTrack = new SequenceTrack("Reference sequence");
         panel.addTrack(newSeqTrack);
 
-        if (newGeneTrack != null) {
-            newGeneTrack.setAttributeValue(Globals.TRACK_NAME_ATTRIBUTE, newGeneTrack.getName());
-            newGeneTrack.setAttributeValue(Globals.TRACK_DATA_FILE_ATTRIBUTE, "");
-            newGeneTrack.setAttributeValue(Globals.TRACK_DATA_TYPE_ATTRIBUTE, newGeneTrack.getTrackType().toString());
-            panel.addTrack(newGeneTrack);
-        }
-    }
-
-    public boolean hasSequenceTrack() {
-        return getSequenceTrack() != null;
+//        if (newGeneTrack != null) {
+//            newGeneTrack.setAttributeValue(Globals.TRACK_NAME_ATTRIBUTE, newGeneTrack.getName());
+//            newGeneTrack.setAttributeValue(Globals.TRACK_DATA_FILE_ATTRIBUTE, "");
+//            newGeneTrack.setAttributeValue(Globals.TRACK_DATA_TYPE_ATTRIBUTE, newGeneTrack.getTrackType().toString());
+//            panel.addTrack(newGeneTrack);
+//        }
     }
 
     /**
@@ -1820,7 +1806,7 @@ public class IGV implements IGVEventObserver {
         session.setGroupByAttribute(attributeName);
         resetGroups();
         // Some tracks need to respond to changes in grouping, fire notification event
-        IGVEventBus.getInstance().post(new TrackGroupEvent(this));
+        IGVEventBus.getInstance().post(new TrackGroupEvent());
     }
 
 
@@ -1920,8 +1906,16 @@ public class IGV implements IGVEventObserver {
                 }
 
             } else {
-                boolean genomeLoaded = false;
+                // Check whether autosave is set to load and exists
+                boolean autosavePresent = false;
+                try {
+                    autosavePresent = SessionAutosaveManager.getMostRecentAutosaveFile().isPresent();
+                } catch (Exception e) {
+                    log.error("Failure trying to get most recent autosave file", e);
+                }
+                boolean loadAutosave = autosavePresent && PreferencesManager.getPreferences().getAsBoolean(AUTOLOAD_LAST_AUTOSAVE);
 
+                boolean genomeLoaded = false;
                 if (igvArgs.getGenomeId() != null) {
                     String genomeId = igvArgs.getGenomeId();
                     try {
@@ -1932,8 +1926,8 @@ public class IGV implements IGVEventObserver {
                         log.error("Error loading genome: " + genomeId, e);
                     }
                 }
-
-                if (genomeLoaded == false && igvArgs.getSessionFile() == null) {
+                // If we're not loading a session file, attempt to load a default genome file
+                if (igvArgs.getSessionFile() == null && !loadAutosave && !genomeLoaded) {
                     String genomeId = preferences.getDefaultGenome();
                     try {
                         GenomeManager.getInstance().loadGenomeById(genomeId);
@@ -1942,49 +1936,49 @@ public class IGV implements IGVEventObserver {
                         MessageUtils.showErrorMessage("Error loading genome: " + genomeId, e);
                         log.error("Error loading genome: " + genomeId, e);
                     }
-                }
 
-                if (genomeLoaded == false && igvArgs.getSessionFile() == null) {
-                    String genomeId = GenomeListManager.DEFAULT_GENOME.getId();
-                    try {
-                        GenomeManager.getInstance().loadGenomeById(genomeId);
-                    } catch (IOException e) {
-                        MessageUtils.showErrorMessage("Error loading genome: " + genomeId, e);
-                        log.error("Error loading genome: " + genomeId, e);
+                    if (!genomeLoaded) {
+                        genomeId = GenomeListManager.DEFAULT_GENOME.getId();
+                        try {
+                            GenomeManager.getInstance().loadGenomeById(genomeId);
+                        } catch (IOException e) {
+                            MessageUtils.showErrorMessage("Error loading genome: " + genomeId, e);
+                            log.error("Error loading genome: " + genomeId, e);
+                        }
                     }
                 }
 
-                if (igvArgs.getSessionFile() != null || igvArgs.getDataFileStrings() != null) {
+                if (igvArgs.getSessionFile() != null || igvArgs.getDataFileStrings() != null || loadAutosave) {
 
                     if (log.isDebugEnabled()) {
                         log.debug("Loading session data");
                     }
 
-                    final IndefiniteProgressMonitor indefMonitor = new IndefiniteProgressMonitor();
-                    final ProgressBar.ProgressDialog progressDialog2 = ProgressBar.showProgressDialog(mainFrame, "Loading session data", indefMonitor, false);
-                    indefMonitor.start();
-
-
                     if (log.isDebugEnabled()) {
                         log.debug("Calling restore session");
                     }
 
 
                     if (igvArgs.getSessionFile() != null) {
-                        boolean success = false;
-                        if (HttpUtils.isRemoteURL(igvArgs.getSessionFile())) {
-                            boolean merge = false;
-                            success = loadSession(igvArgs.getSessionFile(), igvArgs.getLocusString());
-                        } else {
-                            File sf = new File(igvArgs.getSessionFile());
-                            if (sf.exists()) {
-                                success = loadSession(sf.getAbsolutePath(), igvArgs.getLocusString());
+                        IGV.getInstance().setStatusBarMessage3("Loading " + igvArgs.getSessionFile());
+                        boolean success;
+                        try {
+                            success = false;
+                            if (HttpUtils.isRemoteURL(igvArgs.getSessionFile())) {
+                                boolean merge = false;
+                                success = loadSession(igvArgs.getSessionFile(), igvArgs.getLocusString());
+                            } else {
+                                File sf = new File(igvArgs.getSessionFile());
+                                if (sf.exists()) {
+                                    success = loadSession(sf.getAbsolutePath(), igvArgs.getLocusString());
+                                }
                             }
+                        } finally {
+                            IGV.getInstance().setStatusBarMessage3("");
                         }
                         if (!success) {
                             String genomeId = preferences.getDefaultGenome();
                             contentPane.getCommandBar().selectGenome(genomeId);
-
                         }
                     } else if (igvArgs.getDataFileStrings() != null) {
 
@@ -2051,12 +2045,22 @@ public class IGV implements IGVEventObserver {
                             locators.add(rl);
                         }
                         loadTracks(locators);
+                    } else if (loadAutosave) {
+                        boolean success = false;
+                        try {
+                            // Get the last autosave and attempt to load
+                            File sessionAutosave = SessionAutosaveManager.getMostRecentAutosaveFile().get();
+                            success = loadSession(sessionAutosave.getAbsolutePath(), null);
+                        } catch (Exception e) {
+                            log.error("Failure trying to load most recent autosave file", e);
+                        }
+
+                        // Load the default genome if unsuccessful
+                        if (!success) {
+                            String genomeId = preferences.getDefaultGenome();
+                            contentPane.getCommandBar().selectGenome(genomeId);
+                        }
                     }
-
-
-                    indefMonitor.stop();
-                    closeWindow(progressDialog2);
-
                 }
 
                 UIUtilities.invokeAndWaitOnEventThread(() -> {
@@ -2146,12 +2150,12 @@ public class IGV implements IGVEventObserver {
      *
      * @param event
      */
-    public void postEvent(Object event) {
+    public void postEvent(IGVEvent event) {
         IGVEventBus.getInstance().post(event);
     }
 
 
-    public void receiveEvent(Object event) {
+    public void receiveEvent(IGVEvent event) {
         if (event instanceof ViewChange || event instanceof InsertionSelectionEvent) {
             repaint();
         } else if (event instanceof GenomeChangeEvent) {
@@ -2234,7 +2238,7 @@ public class IGV implements IGVEventObserver {
                 try {
                     for (ReferenceFrame frame : FrameManager.getFrames()) {
                         for (Track track : trackList) {
-                            if (track.isReadyToPaint(frame) == false) {
+                            if (!track.isReadyToPaint(frame)) {
                                 track.load(frame);
                             }
                         }
@@ -2259,11 +2263,11 @@ public class IGV implements IGVEventObserver {
                 return;
             }
 
-            List<CompletableFuture> futures = new ArrayList();
+            List<CompletableFuture<Void>> futures = new ArrayList<>();
 
             for (ReferenceFrame frame : FrameManager.getFrames()) {
                 for (Track track : trackList) {
-                    if (track.isReadyToPaint(frame) == false) {
+                    if (!track.isReadyToPaint(frame)) {
                         futures.add(CompletableFuture.runAsync(() -> track.load(frame), threadExecutor));
                     }
                 }
@@ -2276,9 +2280,9 @@ public class IGV implements IGVEventObserver {
                     component.repaint();
                 });
             } else {
-                // One ore more tracks require loading before repaint.   Load all needed tracks, autscale if needed, then
-                // repaint.  The autoscale step is key, since tracks can be grouped for autoscaling it is neccessary that
-                // all data is loaded before any track is repainted.  Otherwise tracks be loaded an painted independently.
+                // One or more tracks require loading before repaint.   Load all needed tracks, autoscale if needed, then
+                // repaint.  The autoscale step is key, since tracks can be grouped for autoscaling it is necessary that
+                // all data is loaded before any track is repainted.  Otherwise tracks be loaded and painted independently.
 
                 final CompletableFuture[] futureArray = futures.toArray(new CompletableFuture[futures.size()]);
                 WaitCursorManager.CursorToken token = WaitCursorManager.showWaitCursor();
@@ -2299,6 +2303,7 @@ public class IGV implements IGVEventObserver {
                     });
                     return null;
                 }).exceptionally(ex -> {
+                    WaitCursorManager.removeWaitCursor(token);
                     log.error("Error loading track data", ex);
                     isLoading = false;
                     pending = null;
@@ -2324,6 +2329,13 @@ public class IGV implements IGVEventObserver {
                 collect(Collectors.toList());
     }
 
+    /**
+     * Stops the scheduled autosave task
+     */
+    public void stopTimedAutosave() {
+        sessionAutosaveTimer.cancel();
+    }
+
     // Thread pool for loading data
     private static final ExecutorService threadExecutor = Executors.newFixedThreadPool(5);
 }
