@@ -32,10 +32,14 @@ import org.broad.igv.aws.S3LoadDialog;
 import org.broad.igv.batch.CommandExecutor;
 import org.broad.igv.charts.ScatterPlotUtils;
 import org.broad.igv.event.GenomeChangeEvent;
+import org.broad.igv.event.IGVEvent;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
+import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.feature.genome.GenomeUtils;
+import org.broad.igv.track.AttributeManager;
+import org.broad.igv.track.Track;
 import org.broad.igv.util.GoogleUtils;
 import org.broad.igv.oauth.OAuthProvider;
 import org.broad.igv.oauth.OAuthUtils;
@@ -69,10 +73,10 @@ import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyEvent;
-import java.io.File;
-import java.io.IOException;
+import java.io.*;
 import java.net.URI;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.List;
 
 import static org.broad.igv.prefs.Constants.*;
@@ -97,6 +101,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
     private JMenu toolsMenu;
     private JMenu googleMenu;
     private JMenu AWSMenu;
+    private AutosaveMenu autosaveMenu;
     private FilterTracksMenuAction filterTracksAction;
     private JMenu viewMenu;
     private IGV igv;
@@ -104,7 +109,9 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
      * We store this as a field because we alter it if
      * we can't access genome server list
      */
-    private JMenuItem loadFromServerMenuItem;
+    private JMenuItem loadGenomeFromServerMenuItem;
+    private JMenuItem loadTracksFromServerMenuItem;
+    private JMenuItem selectGenomeAnnotationsItem;
     private JMenuItem encodeMenuItem;
     private JMenuItem reloadSessionItem;
 
@@ -135,7 +142,6 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
         IGVEventBus.getInstance().subscribe(GenomeChangeEvent.class, this);
 
-
         //This is for Macs, so showing the about dialog
         //from the command bar does what we want.
         if (Globals.IS_MAC) {
@@ -146,11 +152,11 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
 
     public void notifyGenomeServerReachable(boolean reachable) {
-        if (loadFromServerMenuItem != null) {
+        if (loadGenomeFromServerMenuItem != null) {
             UIUtilities.invokeOnEventThread(() -> {
-                loadFromServerMenuItem.setEnabled(reachable);
+                loadGenomeFromServerMenuItem.setEnabled(reachable);
                 String tooltip = reachable ? LOAD_GENOME_SERVER_TOOLTIP : CANNOT_LOAD_GENOME_SERVER_TOOLTIP;
-                loadFromServerMenuItem.setToolTipText(tooltip);
+                loadGenomeFromServerMenuItem.setToolTipText(tooltip);
             });
         }
     }
@@ -177,12 +183,13 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         //extrasMenu.setVisible(false);
         menus.add(extrasMenu);
 
+        // Create a placehold Google menu.  If not explicitly enabled it will remain invisible until triggered
+        // by loading a protected Google resource
         try {
             googleMenu = createGoogleMenu();
-            if (googleMenu != null) {
-                googleMenu.setVisible(PreferencesManager.getPreferences().getAsBoolean(ENABLE_GOOGLE_MENU));
-                menus.add(googleMenu);
-            }
+            boolean enabled = PreferencesManager.getPreferences().getAsBoolean(ENABLE_GOOGLE_MENU);
+            enableGoogleMenu(enabled);
+            menus.add(googleMenu);
         } catch (IOException e) {
             log.error("Error creating google menu: " + e.getMessage());
         }
@@ -191,7 +198,6 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
             AWSMenu = createAWSMenu();
             AWSMenu.setVisible(AmazonUtils.isAwsProviderPresent());
             menus.add(AWSMenu);
-
         } catch (IOException e) {
             log.error("Error creating the Amazon AWS menu: " + e.getMessage());
             AWSMenu.setVisible(false);
@@ -266,6 +272,9 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
     JMenu createFileMenu() {
 
+        Genome genome = GenomeManager.getInstance().getCurrentGenome();
+        String genomeId = genome == null ?  null : genome.getId();
+
         List<JComponent> menuItems = new ArrayList<JComponent>();
         MenuAction menuAction = null;
 
@@ -280,9 +289,25 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuAction.setToolTipText(UIConstants.LOAD_TRACKS_TOOLTIP);
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
-        menuAction = new LoadFromServerAction("Load from Server...", KeyEvent.VK_S, igv);
+        menuAction = new LoadFromServerAction("Load From Server...", KeyEvent.VK_S, igv);
         menuAction.setToolTipText(UIConstants.LOAD_SERVER_DATA_TOOLTIP);
-        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+        loadTracksFromServerMenuItem = MenuAndToolbarUtils.createMenuItem(menuAction);
+        loadTracksFromServerMenuItem.setVisible(genomeId != null && LoadFromServerAction.getNodeURLs(genomeId) != null);
+        menuItems.add(loadTracksFromServerMenuItem);
+
+        // Track hubs -- moved to Genomes menu
+//        menuAction = new LoadFromURLMenuAction(LoadFromURLMenuAction.LOAD_TRACKHUB, KeyEvent.VK_S, igv);
+//        menuAction.setToolTipText(UIConstants.LOAD_TRACKHUB_TOOLTIP);
+//        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+//
+//        menuAction = new SelectGenomeAnnotationTracksAction("Select Hub Tracks...", igv);
+//        selectGenomeAnnotationsItem = MenuAndToolbarUtils.createMenuItem(menuAction);
+//        selectGenomeAnnotationsItem.setVisible(genome != null && genome.getHub() != null);
+//        menuItems.add(selectGenomeAnnotationsItem);
+//
+//        menuAction = new LoadFromURLMenuAction(LoadFromURLMenuAction.LOAD_FROM_HTSGET, 0, igv);
+//        menuAction.setToolTipText(UIConstants.LOAD_HTSGET_TOOLTOP);
+//        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
         if (PreferencesManager.getPreferences().getAsBoolean(DB_ENABLED)) {
             menuAction = new LoadFromDatabaseAction("Load from Database...", 0, igv);
@@ -290,10 +315,8 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         }
 
         encodeMenuItem = MenuAndToolbarUtils.createMenuItem(new BrowseEncodeAction("Load from ENCODE (2012)...", KeyEvent.VK_E, igv));
-        menuItems.add(encodeMenuItem);
-        String genomeId = GenomeManager.getInstance().getGenomeId();
         encodeMenuItem.setVisible(EncodeFileBrowser.genomeSupported(genomeId));
-
+        menuItems.add(encodeMenuItem);
 
         menuItems.add(new JSeparator());
         menuAction = new ReloadTracksMenuAction("Reload Tracks", -1, igv);
@@ -322,6 +345,9 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         reloadSessionItem.setEnabled(false);
         menuItems.add(reloadSessionItem);
 
+        autosaveMenu = new AutosaveMenu();
+        menuItems.add(autosaveMenu);
+
         menuItems.add(new JSeparator());
 
         // ***** Snapshots
@@ -375,7 +401,8 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
             String[] sessions = recentSessions.split(";");
             for (String sessionPath : sessions) {
                 if (!sessionPath.equals("null") &&
-                        !igv.getRecentSessionList().contains(sessionPath)) {
+                        !igv.getRecentSessionList().contains(sessionPath) &&
+                        (new File(sessionPath)).exists()) {
                     igv.getRecentSessionList().add(sessionPath);
                 }
 
@@ -399,52 +426,57 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
     }
 
     private JMenu createGenomesMenu() {
-        List<JComponent> menuItems = new ArrayList<JComponent>();
-        MenuAction menuAction = null;
 
-        // Load genome
-        menuAction =
-                new MenuAction("Load Genome from File...", null, KeyEvent.VK_I) {
-                    @Override
-                    public void actionPerformed(ActionEvent event) {
-                        try {
-                            File importDirectory = PreferencesManager.getPreferences().getLastGenomeImportDirectory();
-                            if (importDirectory == null) {
-                                PreferencesManager.getPreferences().setLastGenomeImportDirectory(DirectoryManager.getUserDirectory());
-                            }
-                            // Display the dialog
-                            File file = FileDialogUtils.chooseFile("Load Genome", importDirectory, FileDialog.LOAD);
+        JMenu menu = new JMenu("Genomes");
 
-                            // If a file selection was made
-                            if (file != null) {
-                                GenomeManager.getInstance().loadGenome(file.getAbsolutePath(), null);
-                            }
-                        } catch (Exception e) {
-                            MessageUtils.showErrorMessage(e.getMessage(), e);
-                        }
-                    }
-                };
+        loadGenomeFromServerMenuItem = new JMenuItem("Select Hosted Genome...");
+        loadGenomeFromServerMenuItem.addActionListener(e -> GenomeComboBox.loadGenomeFromServer());
+        loadGenomeFromServerMenuItem.setToolTipText(LOAD_GENOME_SERVER_TOOLTIP);
+        menu.add(loadGenomeFromServerMenuItem);
 
-        menuAction.setToolTipText("Load a FASTA, .json, or .genome file...");
-        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
+        // Load genome json file
+        JMenuItem fileItem = new JMenuItem("Load Genome from File...", KeyEvent.VK_I);
+        fileItem.addActionListener(e -> {
+            try {
+                File importDirectory = PreferencesManager.getPreferences().getLastGenomeImportDirectory();
+                if (importDirectory == null) {
+                    PreferencesManager.getPreferences().setLastGenomeImportDirectory(DirectoryManager.getUserDirectory());
+                }
+                // Display the dialog
+                File file = FileDialogUtils.chooseFile("Load Genome", importDirectory, FileDialog.LOAD);
+
+                // If a file selection was made
+                if (file != null) {
+                    GenomeManager.getInstance().loadGenome(file.getAbsolutePath());
+                }
+            } catch (Exception ex) {
+                MessageUtils.showErrorMessage(ex.getMessage(), ex);
+            }
+        });
+
+        fileItem.setToolTipText("Load a FASTA, .json, or .genome file...");
+        menu.add(fileItem);
 
         // Load genome from URL
-        menuAction = new LoadFromURLMenuAction(LoadFromURLMenuAction.LOAD_GENOME_FROM_URL, 0, igv);
-        menuAction.setToolTipText("Load a FASTA, .json, or .genome file...");
-        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+        MenuAction urlMenuAction = new LoadFromURLMenuAction(LoadFromURLMenuAction.LOAD_GENOME_FROM_URL, 0, igv);
+        urlMenuAction.setToolTipText("Load a FASTA, .json, or .genome file...");
+        menu.add(MenuAndToolbarUtils.createMenuItem(urlMenuAction));
 
-        // Download genome from server 
-        menuAction = new MenuAction("Select Hosted Genome...", null) {
-            @Override
-            public void actionPerformed(ActionEvent event) {
-                GenomeComboBox.loadGenomeFromServer();
-            }
-        };
-        menuAction.setToolTipText(LOAD_GENOME_SERVER_TOOLTIP);
-        loadFromServerMenuItem = MenuAndToolbarUtils.createMenuItem(menuAction);
-        menuItems.add(loadFromServerMenuItem);
 
-        menuItems.add(new JSeparator());
+        // Track hubs
+        menu.add(new JSeparator());
+        MenuAction genArkAction = new UCSCGenArkAction("Load Genome from UCSC GenArk...", 0, igv);
+        menu.add(MenuAndToolbarUtils.createMenuItem(genArkAction));
+
+        MenuAction menuAction = new SelectGenomeAnnotationTracksAction("Select GenArk Tracks...", igv);
+        selectGenomeAnnotationsItem = MenuAndToolbarUtils.createMenuItem(menuAction);
+        Genome genome = GenomeManager.getInstance().getCurrentGenome();
+        selectGenomeAnnotationsItem.setEnabled(genome != null && genome.getHub() != null);
+        menu.add(selectGenomeAnnotationsItem);
+
+
+        menu.add(new JSeparator());
 
         // Add genome to combo box from server
         menuAction = new MenuAction("Remove Genomes...", null) {
@@ -455,26 +487,28 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
             }
         };
         menuAction.setToolTipText("Remove genomes which appear in the dropdown list");
-        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
-//
-//        menuItems.add(new JSeparator());
-//
-//        menuAction =
-//                new MenuAction("Create .genome File...", null, KeyEvent.VK_D) {
-//                    @Override
-//                    public void actionPerformed(ActionEvent event) {
-//                        javax.swing.ProgressMonitor monitor = new javax.swing.ProgressMonitor(igv.getMainPanel(),
-//                                "Creating genome", null, 0, 100);
-//                        igv.defineGenome(monitor);
-//                    }
-//                };
-//
-//        menuAction.setToolTipText(UIConstants.IMPORT_GENOME_TOOLTIP);
-//        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+        menu.addMenuListener(new MenuListener() {
+            @Override
+            public void menuSelected(MenuEvent e) {
+                Genome genome = GenomeManager.getInstance().getCurrentGenome();
+                selectGenomeAnnotationsItem.setEnabled(genome != null && genome.getHub() != null);
+            }
+
+            @Override
+            public void menuDeselected(MenuEvent e) {
+
+            }
+
+            @Override
+            public void menuCanceled(MenuEvent e) {
+
+            }
+        });
+
+        return menu;
 
-        MenuAction genomeMenuAction = new MenuAction("Genomes", null);
-        return MenuAndToolbarUtils.createMenu(menuItems, genomeMenuAction);
     }
 
 
@@ -507,6 +541,16 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuAction.setToolTipText(UIConstants.OVERLAY_TRACKS_TOOLTIP);
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
+        // Export track names and attributes -- if > 1 i sselected export those, otherwise export all
+        JMenuItem exportNames = new JMenuItem("Export Track Names and Attributes...");
+        exportNames.addActionListener(e12 -> {
+            Collection<Track> exportTracks = IGV.getInstance().getSelectedTracks();
+            if (exportTracks.size() <= 1) {
+                exportTracks = IGV.getInstance().getAllTracks();
+            }
+            exportTrackNames(exportTracks);
+        });
+        menuItems.add(exportNames);
 
         menuItems.add(new JSeparator());
 
@@ -590,11 +634,11 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                 if (newValue != null) {
                     try {
                         Integer w = Integer.parseInt(newValue);
-                        if (w <= 0 || w == 1000) throw new NumberFormatException();
+                        if (w <= 0) throw new NumberFormatException();
                         PreferencesManager.getPreferences().put(NAME_PANEL_WIDTH, newValue);
                         mainPanel.setNamePanelWidth(w);
                     } catch (NumberFormatException ex) {
-                        MessageUtils.showErrorMessage("Error: value must be a positive integer < 1000.", ex);
+                        MessageUtils.showErrorMessage("Error: value must be a positive integer.", ex);
                     }
                 }
             }
@@ -1003,72 +1047,72 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         return menu;
     }
 
-    private JMenu createGoogleMenu() throws IOException {
+    private JMenu createGoogleMenu() {
 
-        // Dynamically name menu - dwm08
-        final OAuthProvider oauth = OAuthUtils.getInstance().getDefaultProvider();
+        googleMenu = new JMenu("Google");
 
-        if (oauth != null) {
+        final JMenuItem login = new JMenuItem("Login ... ");
+        login.addActionListener(e -> {
+            try {
+                OAuthUtils.getInstance().getGoogleProvider().openAuthorizationPage();
+            } catch (Exception ex) {
+                MessageUtils.showErrorMessage("Error fetching oAuth tokens.  See log for details", ex);
+                log.error("Error fetching oAuth tokens", ex);
+            }
 
-            JMenu menu = new JMenu("Google");
+        });
+        googleMenu.add(login);
 
-            boolean isLoggedIn = oauth.isLoggedIn();
+        final JMenuItem logout = new JMenuItem("Logout ");
+        logout.addActionListener(e -> {
+            OAuthUtils.getInstance().getGoogleProvider().logout();
+            GoogleUtils.setProjectID(null);
+        });
+        googleMenu.add(logout);
 
-            final JMenuItem login = new JMenuItem("Login ... ");
-            login.addActionListener(e -> {
-                try {
-                    oauth.openAuthorizationPage();
-                } catch (Exception ex) {
-                    MessageUtils.showErrorMessage("Error fetching oAuth tokens.  See log for details", ex);
-                    log.error("Error fetching oAuth tokens", ex);
+        final JMenuItem projectID = new JMenuItem("Enter Project ID ...");
+        projectID.addActionListener(e -> GoogleUtils.enterGoogleProjectID());
+        googleMenu.add(projectID);
+
+        googleMenu.addMenuListener(new MenuListener() {
+            @Override
+            public void menuSelected(MenuEvent e) {
+                OAuthProvider oauth = OAuthUtils.getInstance().getGoogleProvider();
+                boolean loggedIn = oauth.isLoggedIn();
+                if (loggedIn && oauth.getCurrentUserName() != null) {
+                    login.setText(oauth.getCurrentUserName());
+                } else {
+                    login.setText("Login ...");
                 }
+                login.setEnabled(!loggedIn);
+                logout.setEnabled(loggedIn);
+            }
 
-            });
-            login.setEnabled(!isLoggedIn);
-            menu.add(login);
+            @Override
+            public void menuDeselected(MenuEvent e) {
+
+            }
+
+            @Override
+            public void menuCanceled(MenuEvent e) {
+
+            }
+
+        });
+
+        return googleMenu;
+    }
 
 
-            final JMenuItem logout = new JMenuItem("Logout ");
-            logout.addActionListener(e -> {
-                oauth.logout();
-                GoogleUtils.setProjectID(null);
-            });
-            logout.setEnabled(isLoggedIn);
-            menu.add(logout);
-
-            final JMenuItem projectID = new JMenuItem("Enter Project ID ...");
-            projectID.addActionListener(e -> GoogleUtils.enterGoogleProjectID());
-            menu.add(projectID);
-
-            menu.addMenuListener(new MenuListener() {
-                @Override
-                public void menuSelected(MenuEvent e) {
-                    OAuthProvider oAuthProvider = OAuthUtils.getInstance().getDefaultProvider();
-                    boolean loggedIn = oAuthProvider.isLoggedIn();
-                    if (loggedIn && oAuthProvider.getCurrentUserName() != null) {
-                        login.setText(oAuthProvider.getCurrentUserName());
-                    } else {
-                        login.setText("Login ...");
-                    }
-                    login.setEnabled(!loggedIn);
-                    logout.setEnabled(loggedIn);
-                }
-
-                @Override
-                public void menuDeselected(MenuEvent e) {
-
-                }
-
-                @Override
-                public void menuCanceled(MenuEvent e) {
-
-                }
-
-            });
-            return menu;
-        } else {
-            return null;
-        }
+    /**
+     * The Google menu is enabled dynamically to defer loading of oAuth properties until needed.
+     * *
+     *
+     * @return
+     * @throws IOException
+     */
+    public void enableGoogleMenu(boolean enable) throws IOException {
+        googleMenu.setVisible(enable);
     }
 
 //    public void enableRemoveGenomes() {
@@ -1139,17 +1183,17 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         instance = null;
     }
 
-    public void enableGoogleMenu(boolean aBoolean) {
-        if (googleMenu != null) {
-            googleMenu.setVisible(aBoolean);
-        }
-    }
 
     @Override
-    public void receiveEvent(final Object event) {
+    public void receiveEvent(final IGVEvent event) {
 
         if (event instanceof GenomeChangeEvent) {
-            UIUtilities.invokeOnEventThread(() -> encodeMenuItem.setVisible(EncodeFileBrowser.genomeSupported(((GenomeChangeEvent) event).genome.getId())));
+            UIUtilities.invokeOnEventThread(() -> {
+                final Genome genome = ((GenomeChangeEvent) event).genome();
+                encodeMenuItem.setVisible(EncodeFileBrowser.genomeSupported(genome.getId()));
+                loadTracksFromServerMenuItem.setVisible(LoadFromServerAction.getNodeURLs(genome.getId()) != null);
+                //selectGenomeAnnotationsItem .setEnabled(genome != null && genome.getHub() != null);
+            });
         }
     }
 
@@ -1177,4 +1221,53 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
         return menuItem;
     }
+
+    private void exportTrackNames(final Collection<Track> selectedTracks) {
+
+        if (selectedTracks.isEmpty()) {
+            return;
+        }
+
+        File file = FileDialogUtils.chooseFile("Export track names",
+                PreferencesManager.getPreferences().getLastTrackDirectory(),
+                new File("trackNames.txt"),
+                FileDialogUtils.SAVE);
+
+        if (file == null) {
+            return;
+        }
+
+        PrintWriter pw = null;
+        try {
+            pw = new PrintWriter(new BufferedWriter(new FileWriter(file)));
+
+            List<String> attributes = AttributeManager.getInstance().getVisibleAttributes();
+
+            pw.print("Name");
+            for (String att : attributes) {
+                pw.print("\t" + att);
+            }
+            pw.println();
+
+            for (Track track : selectedTracks) {
+                //We preserve the alpha value. This is motivated by MergedTracks
+                pw.print(track.getName());
+
+                for (String att : attributes) {
+                    String val = track.getAttributeValue(att);
+                    pw.print("\t" + (val == null ? "" : val));
+                }
+                pw.println();
+            }
+
+
+        } catch (IOException e) {
+            MessageUtils.showErrorMessage("Error writing to file", e);
+            log.error(e);
+        } finally {
+            if (pw != null) pw.close();
+        }
+
+    }
+
 }
