@@ -29,7 +29,6 @@
  */
 package org.broad.igv.ui.panel;
 
-import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.ViewChange;
@@ -38,10 +37,11 @@ import org.broad.igv.feature.Locus;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
-import org.broad.igv.sam.InsertionManager;
 import org.broad.igv.sam.InsertionMarker;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.util.MessageUtils;
@@ -54,7 +54,7 @@ import java.awt.event.MouseEvent;
  */
 public class ReferenceFrame {
 
-    private static Logger log = LogManager.getLogger(ReferenceFrame.class);
+    private static final Logger log = LogManager.getLogger(ReferenceFrame.class);
 
     IGVEventBus eventBus;
 
@@ -122,6 +122,7 @@ public class ReferenceFrame {
     protected volatile double scale;
 
     protected Locus initialLocus = null;
+    private InsertionMarker expandedInsertion;
 
 
     public ReferenceFrame(String name) {
@@ -166,7 +167,7 @@ public class ReferenceFrame {
 
     public void dragStopped() {
         setOrigin(Math.round(origin));   // Snap to gride
-        eventBus.post(ViewChange.Result());
+        eventBus.post(ViewChange.LocusChangeResult(chrName, origin, getEnd(), false));
     }
 
     public void changeGenome(Genome genome) {
@@ -176,8 +177,7 @@ public class ReferenceFrame {
     public void changeChromosome(String chrName, boolean recordHistory) {
         boolean changed = setChromosomeName(chrName, false);
         // if (changed) {
-        ViewChange resultEvent = ViewChange.ChromosomeChangeResult(chrName);
-        resultEvent.setRecordHistory(recordHistory);
+        ViewChange resultEvent = ViewChange.ChromosomeChangeResult(chrName, recordHistory);
         eventBus.post(resultEvent);
         changeZoom(0);
         // }
@@ -185,8 +185,7 @@ public class ReferenceFrame {
 
     public void changeZoom(int newZoom) {
         doSetZoom(newZoom);
-        ViewChange result = ViewChange.Result();
-        result.setRecordHistory(false);
+        ViewChange result = ViewChange.LocusChangeResult(chrName, origin, getEnd(), false);
         eventBus.post(result);
     }
 
@@ -380,8 +379,8 @@ public class ReferenceFrame {
     public void shiftOriginPixels(int delta) {
 
         double shiftBP = delta * getScale();
-        setOrigin(origin + shiftBP);
-        eventBus.post(ViewChange.Result());
+        setOrigin(shiftBP + origin);
+        eventBus.post(ViewChange.LocusChangeResult(chrName, origin, getEnd(), false));
     }
 
     public void centerOnLocation(String chr, double chrLocation) {
@@ -394,7 +393,7 @@ public class ReferenceFrame {
     public void centerOnLocation(double chrLocation) {
         double windowWidth = (widthInPixels * getScale()) / 2;
         setOrigin(Math.round(chrLocation - windowWidth));
-        eventBus.post(ViewChange.LocusChangeResult(chrName, origin, chrLocation + windowWidth));
+        eventBus.post(ViewChange.LocusChangeResult(chrName, origin, chrLocation + windowWidth, false));
     }
 
     public boolean windowAtEnd() {
@@ -447,19 +446,32 @@ public class ReferenceFrame {
             log.debug("Scale = " + scale);
         }
 
-        eventBus.post(ViewChange.LocusChangeResult(chrName, start, end));
+        eventBus.post(ViewChange.LocusChangeResult(chrName, start, end, false));
     }
 
     public double getOrigin() {
         return origin;
     }
 
+    /**
+     * Return the origin in reference genome coordinates.  When an insertion is expanded "origin" is in expansion modified
+     * coordinates, we need to adjust for the expansion to convert to reference genome coordinates* *
+     *
+     * @return
+     */
+    public double getExpansionAdjustedOrigin() {
+        InsertionMarker im = getExpandedInsertion();
+        return (im == null || origin < im.position) ?
+                origin :
+                origin - im.size;
+    }
+
     public double getCenter() {
         return origin + getScale() * widthInPixels / 2;
     }
 
     public double getEnd() {
-        return origin + getScale() * widthInPixels;
+        return widthInPixels > 0 ? getChromosomePosition(widthInPixels) : initialLocus != null ? initialLocus.getEnd() : 0;
     }
 
     public int getZoom() {
@@ -517,59 +529,53 @@ public class ReferenceFrame {
     }
 
     /**
-     * Return the chromosome position corresponding to the pixel index.  The
-     * pixel index is the pixel "position" translated by -origin.
+     * Return the chromosome position corresponding to the screen (pixel) position.
      *
      * @param screenPosition
      * @return
      */
     public double getChromosomePosition(int screenPosition) {
-
-        InsertionMarker i = InsertionManager.getInstance().getSelectedInsertion(getChrName());
-
-        if (i != null && i.position > origin) {
-            // if (IGV.getInstance().getSession().expandInsertions && insertionMarkers != null && insertionMarkers.size() > 0) {
-            double start = getOrigin();
-            double scale = getScale();
-            double iEnd = 0,
-                    iStart = 0;
-
-
-            iStart = iEnd + (i.position - start) / scale; // Screen position of insertionMarker start
-            if (screenPosition < iStart) {
-                return start + scale * (screenPosition - iEnd);
+        InsertionMarker i = expandedInsertion;
+        if (i != null) {
+            int insertionPixelPosition = (int) ((i.position - origin) / scale);
+            int insertionPixelWidth = (int) Math.ceil(i.size / scale);
+            if (insertionPixelPosition < 0) {
+                // Only a portion of the insertion is occupying screen
+                insertionPixelWidth += insertionPixelPosition;
+                insertionPixelPosition = 0;
             }
-
-            iEnd = iStart + i.size / scale;  // Screen position of insertionMarker end
-            if (screenPosition < iEnd) {
-                return i.position;   // In the gap
+            if (insertionPixelWidth > 0) {
+                if (screenPosition < insertionPixelPosition) {
+                    // Nothing to do -- expanded insertion is to the right of screen position
+                } else if (screenPosition < insertionPixelPosition + insertionPixelWidth) {
+                    // Over expanded insertion, chromosome position is position of insertion
+                    screenPosition = insertionPixelPosition;
+                } else {
+                    // To the right of the expanded insertion
+                    screenPosition -= insertionPixelWidth;
+                }
             }
-
-            start = i.position + 1;
-            //    }
-            return start + scale * (screenPosition - iEnd);
-
-        } else {
-            return origin + getScale() * screenPosition;
         }
-
+        return origin + getScale() * screenPosition;
     }
 
 
     /**
-     * Return the screen position corresponding to the chromosomal position.
+     * Return the screen position corresponding to the chromosomal position, accounting for the possibility of
+     * an expanded insertion in view.  For the special case when chromosomePosition (i.e. genomic coordinate)
+     * is at the expanded insertion there is no unique screen position.  By convention we return the screen position
+     * at the left boundary of the expansion
      *
-     * @param chromosomePosition
-     * @return
+     * @param chromosomePosition The genomic position in base pairs
+     * @return screen position (pixels)
      */
     public int getScreenPosition(double chromosomePosition) {
 
-        InsertionMarker i = InsertionManager.getInstance().getSelectedInsertion(chrName);
-
-        if (i == null || i.position < origin || i.position > chromosomePosition) {
-            return (int) ((chromosomePosition - origin) / getScale());
+        // If there is an expanded insertion in view we need to account for the empty screen space
+        if (expandedInsertion != null && chromosomePosition > expandedInsertion.position) {
+            return (int) ((chromosomePosition + expandedInsertion.size - origin) / getScale());
         } else {
-            return (int) ((chromosomePosition + i.size - origin) / getScale());
+            return (int) ((chromosomePosition - origin) / getScale());
         }
     }
 
@@ -579,7 +585,6 @@ public class ReferenceFrame {
         if (genome == null) {
             return null;
         }
-
         return genome.getChromosome(chrName);
     }
 
@@ -631,17 +636,17 @@ public class ReferenceFrame {
      */
     public String getFormattedLocusString() {
 
-        if (zoom == 0) {
-            return getChrName();
-        } else {
-
-            Range range = getCurrentRange();
-            return Locus.getFormattedLocusString(range.getChr(), range.getStart(), range.getEnd());
-        }
+//        if (zoom == 0) {
+//            return getGenome().getChromosomeDisplayName(getChrName());
+//        } else {
+        Range range = getCurrentRange();
+        String c = getGenome().getChromosomeDisplayName(range.getChr());
+        return Locus.getFormattedLocusString(c, range.getStart(), range.getEnd());
+        // }
     }
 
     public Range getCurrentRange() {
-        int endLoc = (int) Math.round(getChromosomePosition(widthInPixels));
+        int endLoc = (int) Math.round(getEnd());
         Range range = new Range(getChrName(), (int) origin, endLoc);
         return range;
     }
@@ -707,7 +712,7 @@ public class ReferenceFrame {
     }
 
     /**
-     * Calculate the zoom level given start/end in bp.
+     * Calculate the minimum zoom level which can completely contain the given start/end in bp.
      * Doesn't change anything
      *
      * @param start
@@ -715,8 +720,15 @@ public class ReferenceFrame {
      * @return
      */
     public int calculateZoom(double start, double end) {
-        final double windowLength = Math.min(end - start, getChromosomeLength());
-        return (int) Math.round(Globals.log2((getChromosomeLength() / windowLength) * (((double) widthInPixels) / binsPerTile)));
+        final double windowLength = Math.ceil(end) - start;
+        final int chrLength = getChromosomeLength();
+        if (windowLength >= getChromosomeLength()) {
+            return 0;
+        } else {
+            final double exactZoom = Globals.log2((getChromosomeLength() / windowLength) * (((double) widthInPixels) / binsPerTile));
+            //round up so that you get a zoom level which contains the given window
+            return (int) Math.ceil(exactZoom);
+        }
     }
 
 
@@ -729,7 +741,7 @@ public class ReferenceFrame {
 
         if (chrName.equals("All")) {
             // Genome coordinates are in kb => divde by 1000
-            return (int) (genome.getNominalLength() / 1000);
+            return (int) (genome.getWGLength() / 1000);
         } else {
             Chromosome chromosome = genome.getChromosome(chrName);
             if (chromosome == null) {
@@ -744,6 +756,17 @@ public class ReferenceFrame {
         }
     }
 
+    public void setExpandedInsertion(InsertionMarker im) {
+        InsertionMarker previousInsertion = this.expandedInsertion;
+        this.expandedInsertion = im;
+        if (im == null && previousInsertion != null) {
+            this.centerOnLocation(previousInsertion.position);
+        }
+    }
+
+    public InsertionMarker getExpandedInsertion() {
+        return expandedInsertion;
+    }
 
     public int stateHash() {
         int result;
