@@ -27,7 +27,7 @@ package org.broad.igv.util;
 
 import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.io.output.StringBuilderWriter;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.util.ftp.FTPUtils;
 
 import java.io.*;
@@ -41,7 +41,7 @@ import java.util.regex.Pattern;
  */
 public class FileUtils {
 
-    private static final Logger log = Logger.getLogger(FileUtils.class);
+    private static final Logger log = LogManager.getLogger(FileUtils.class);
 
     final public static String LINE_SEPARATOR = System.getProperty("line.separator");
     final public static String FILE_SEP = System.getProperty("file.separator");
@@ -55,6 +55,9 @@ public class FileUtils {
         if (isRemote(path)) {
             return HttpUtils.getInstance().resourceAvailable(path);
         } else {
+            if (path.startsWith("file://")) {
+                path = path.substring(7);
+            }
             return (new File(path)).exists();
         }
 
@@ -66,7 +69,8 @@ public class FileUtils {
             return false;
         }
         return path.startsWith("http://") || path.startsWith("https://") ||
-                path.startsWith("ftp://") || path.startsWith("gs://");
+                path.startsWith("ftp://") || path.startsWith("gs://") ||
+                path.startsWith("s3://") || path.startsWith("htsget://");
     }
 
 
@@ -215,33 +219,25 @@ public class FileUtils {
 
 
     /**
-     * Test to see if the first comment line (first line not starting with #) is tab-delimited with the
+     * Test to see if the first non-comment line (first line not starting with #) is tab-delimited with the
      * given number of minimum columns.  Limit the test to the first 1,000 lines.
      */
-    public static boolean isTabDelimited(ResourceLocator loc, int minColumnCount) throws IOException {
+    public static boolean isTabDelimited(BufferedReader reader, int minColumnCount) throws IOException {
 
-        BufferedReader reader = null;
-
-        try {
-            reader = ParsingUtils.openBufferedReader(loc.getPath());
-            int nLinesTested = 0;
-            String nextLine;
-            while ((nextLine = reader.readLine()) != null && nLinesTested < 1000) {
-                if (nextLine.startsWith("#")) {
-                    continue;
-                }
-                nLinesTested++;
-                String[] tokens = nextLine.split("\t");
-                if (tokens.length >= minColumnCount) {
-                    return true;
-                }
+        int nLinesTested = 0;
+        String nextLine;
+        while ((nextLine = reader.readLine()) != null && nLinesTested < 1000) {
+            if (nextLine.startsWith("#")) {
+                continue;
             }
-            return false;
-        } finally {
-            if (reader != null) {
-                reader.close();
+            nLinesTested++;
+            String[] tokens = nextLine.split("\t");
+            if (tokens.length >= minColumnCount) {
+                return true;
             }
         }
+        return false;
+
     }
 
     /**
@@ -254,7 +250,7 @@ public class FileUtils {
      */
     public static String getFirstLine(String path, String commentChar) throws IOException {
 
-        if(path == null) return null;
+        if (path == null) return null;
 
         BufferedReader reader = null;
         try {
@@ -390,7 +386,7 @@ public class FileUtils {
      * @return
      */
     public static String getAbsolutePath(String inputPath, String referencePath) {
-        if (isRemote(inputPath)) {
+        if (isRemote(inputPath) || referencePath == null) {
             return inputPath;
         }
         File inFile = new File(inputPath);
@@ -497,15 +493,20 @@ public class FileUtils {
      */
     public static String getContents(String path) throws IOException {
 
-        BufferedReader reader = ParsingUtils.openBufferedReader(path);
+        BufferedReader reader = null;
 
-        StringBuilder contents = new StringBuilder();
-        PrintWriter pw = new PrintWriter(new StringBuilderWriter(contents));
-        String nextLine;
-        while ((nextLine = reader.readLine()) != null) {
-            pw.println(nextLine);
+        try {
+            reader = ParsingUtils.openBufferedReader(path);
+            StringBuilder contents = new StringBuilder();
+            PrintWriter pw = new PrintWriter(new StringBuilderWriter(contents));
+            String nextLine;
+            while ((nextLine = reader.readLine()) != null) {
+                pw.println(nextLine);
+            }
+            return contents.toString();
+        } finally {
+            reader.close();
         }
-        return contents.toString();
 
     }
 
