@@ -57,7 +57,6 @@ import java.util.List;
 import java.util.*;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import java.util.regex.PatternSyntaxException;
 
 import static org.broad.igv.prefs.Constants.*;
 import static org.broad.igv.util.stream.SeekableServiceStream.WEBSERVICE_URL;
@@ -88,6 +87,8 @@ public class HttpUtils {
     private static boolean BYTE_RANGE_DISABLED = false;
     private Map<URL, Boolean> headURLCache = new HashMap<URL, Boolean>();
 
+    private Map<String, Long> contentLengthCache = new HashMap<>();
+
     private class CachedRedirect {
         private URL url = null;
         private ZonedDateTime expires = null;
@@ -98,7 +99,7 @@ public class HttpUtils {
     private Map<URL, CachedRedirect> redirectCache = new HashMap<URL, CachedRedirect>();
 
     // oauth tokens set from command line script
-    private Map<Pattern, String> accessTokens = new HashMap<>();
+    Deque<Pair<Pattern, String>> accessTokens = new ArrayDeque<>();
 
     /**
      * @return the single instance
@@ -136,29 +137,30 @@ public class HttpUtils {
         } else {
             host = host.replace("*", ".*");
         }
-        this.accessTokens.put(Pattern.compile(host, Pattern.CASE_INSENSITIVE), token);
+
+        Pattern newPattern = Pattern.compile(host, Pattern.CASE_INSENSITIVE);
+        this.accessTokens.add(new Pair<>(newPattern, token));
     }
 
 
     /**
-     * Return an access token, if any, from the access token cache.
+     * Return an access token, if any, from the access token cache.  The queue is iterated
+     * in reverse order so the latest match is returned.  
      *
      * @param url
      * @return
      */
-    String getAccessTokenFor(URL url) {
+    String getCachedTokenFor(URL url) {
 
-        for (Map.Entry<Pattern, String> entry : this.accessTokens.entrySet()) {
-            final Pattern pattern = entry.getKey();
-            Matcher matcher = pattern.matcher(url.getHost());
+        Iterator<Pair<Pattern, String>> iter = accessTokens.descendingIterator();
+        while(iter.hasNext()) {
+            Pair<Pattern, String> next = iter.next();
+            Matcher matcher = next.getFirst().matcher(url.getHost());
             if (matcher.find()) {
-                return entry.getValue();
+                return next.getSecond();
             }
         }
         return null;
-//        if (token == null && oauthProvider != null && oauthProvider.appliesToUrl(url)) {
-//            token = oauthProvider.getAccessToken();
-//        }
     }
 
     public void clearAccessTokens() {
@@ -198,7 +200,7 @@ public class HttpUtils {
     public static String mapURL(String urlString) throws MalformedURLException {
 
         // Check explicit mappings first
-        if(urlMappings.containsKey(urlString)) {
+        if (urlMappings.containsKey(urlString)) {
             return urlMappings.get(urlString);
         }
 
@@ -226,6 +228,8 @@ public class HttpUtils {
             urlString = urlString.replace("//www.dropbox.com", "//dl.dropboxusercontent.com");
         } else if (host.equals("drive.google.com")) {
             urlString = GoogleUtils.driveDownloadURL(urlString);
+        } else if (host.equals("igv.genepattern.org")) {
+            urlString = urlString.replace("//igv.genepattern.org", "//igv-genepattern-org.s3.amazonaws.com");
         }
 
         // data.broadinstitute.org requires https
@@ -319,7 +323,10 @@ public class HttpUtils {
         conn.getOutputStream().write(postDataBytes);
 
         StringBuilder response = new StringBuilder();
+        System.out.println(conn.getResponseMessage());
         Reader in = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));
+
+
         for (int c; (c = in.read()) >= 0; ) {
             response.append((char) c);
         }
@@ -459,17 +466,21 @@ public class HttpUtils {
 
     public long getContentLength(URL url) throws IOException {
 
+        if (contentLengthCache.containsKey(url.toExternalForm())) {
+            return contentLengthCache.get(url.toExternalForm());
+        }
+
+        long contentLength = -1;
         try {
             String contentLengthString = getHeaderField(url, "Content-Length");
-            if (contentLengthString == null) {
-                return -1;
-            } else {
-                return Long.parseLong(contentLengthString);
+            if (contentLengthString != null) {
+                contentLength = Long.parseLong(contentLengthString);
             }
         } catch (Exception e) {
             log.error("Error fetching content length", e);
-            return -1;
         }
+        contentLengthCache.put(url.toExternalForm(), contentLength);
+        return contentLength;
     }
 
     public void updateProxySettings() {
@@ -642,6 +653,7 @@ public class HttpUtils {
         return openConnection(url, Collections.<String, String>emptyMap(), "DELETE");
     }
 
+    // Called by IGVSeekableHTTPStream.openInputStreamForRange
     public HttpURLConnection openConnection(URL url, Map<String, String> requestProperties) throws IOException {
         return openConnection(url, requestProperties, "GET");
     }
@@ -678,7 +690,7 @@ public class HttpUtils {
 
         // If we have an explicitly set oauth token for this URL use it.  This is used by port and batch commands
         // and will ovveride oAuth authentication check
-        String token = this.getAccessTokenFor(url);
+        String token = this.getCachedTokenFor(url);
 
         if (token == null) {
 
@@ -687,7 +699,7 @@ public class HttpUtils {
 
             if (oauthProvider != null) {
                 //Google is skipped here as we don't yet know if the url is protected or not.  Login is invoked after 401 error
-                if(!oauthProvider.isGoogle()) {
+                if (!oauthProvider.isGoogle()) {
                     oauthProvider.checkLogin();
                 }
                 token = oauthProvider.getAccessToken();
@@ -851,6 +863,9 @@ public class HttpUtils {
                     throw new FileNotFoundException(message);
                 } else if (code == 401) {
                     OAuthProvider provider = OAuthUtils.getInstance().getProviderForURL(url);
+                    if (provider == null && GoogleUtils.isGoogleURL(url.toExternalForm())) {
+                        provider = OAuthUtils.getInstance().getGoogleProvider();
+                    }
                     if (provider != null && retries == 0) {
                         if (!provider.isLoggedIn()) {
                             provider.checkLogin();
@@ -1153,6 +1168,19 @@ public class HttpUtils {
         }
     }
 
+    /**
+     * Return true if URL has a known "Signed" signature.  There is no expecation this is comprehensive, but it
+     * does match Amazon and Google patterns and possibly others*
+     *
+     * @param url
+     * @return
+     */
+    public static boolean isSignedURL(String url) {
+        Pattern pattern = Pattern.compile("X-.*-Signature");
+        Matcher matcher = pattern.matcher(url);
+        return matcher.find();
+    }
+
     public class UnsatisfiableRangeException extends RuntimeException {
 
         String message;
@@ -1196,6 +1224,7 @@ public class HttpUtils {
     }
 
     private static Map<String, String> urlMappings;
+
     static {
         // mutable map
         urlMappings = new HashMap<>();
