@@ -27,8 +27,12 @@ package org.broad.igv.util;
 
 import com.google.gson.JsonObject;
 import htsjdk.tribble.Tribble;
+import org.broad.igv.feature.genome.load.TrackConfig;
 import org.broad.igv.logging.LogManager;
 import org.broad.igv.logging.Logger;
+import org.broad.igv.track.Track;
+import org.broad.igv.track.TrackProperties;
+import org.broad.igv.ui.color.ColorUtilities;
 
 import java.awt.*;
 import java.io.File;
@@ -38,6 +42,7 @@ import java.util.List;
 import java.util.*;
 
 import static org.broad.igv.feature.tribble.CodecFactory.ucscSNP;
+import static org.broad.igv.util.StringUtils.stripQuotes;
 
 //import java.awt.*;
 
@@ -104,12 +109,13 @@ public class ResourceLocator {
      */
     public String format;
 
-
     /**
      * A UCSC style track line.  Overrides value in file, if any.
      */
     String trackLine;  //
 
+    TrackProperties trackProperties;
+
     /**
      * Color for features or data.  Somewhat redundant with trackLine.
      */
@@ -127,6 +133,8 @@ public class ResourceLocator {
     private boolean htsget;
 
     private Integer visibilityWindow;
+    private String trixURL;
+    private String panelName;
 
     public static List<ResourceLocator> getLocators(Collection<File> files) {
 
@@ -399,6 +407,13 @@ public class ResourceLocator {
         }
     }
 
+    public TrackProperties getTrackProperties() {
+        return trackProperties;
+    }
+
+    public void setTrackProperties(TrackProperties trackProperties) {
+        this.trackProperties = trackProperties;
+    }
 
     public void setName(String name) {
         this.name = name;
@@ -599,6 +614,94 @@ public class ResourceLocator {
         }
     }
 
+    public void setTrixURL(String trixURL) {
+        this.trixURL = trixURL;
+    }
+
+    public String getTrixURL() {
+        return trixURL;
+    }
+
+
+    public static ResourceLocator fromTrackConfig(TrackConfig trackConfig) {
+        String trackPath = trackConfig.url;
+        ResourceLocator res = new ResourceLocator(trackPath);
+        res.setName(trackConfig.name);
+        res.setIndexPath(trackConfig.indexURL);
+        res.setFormat(trackConfig.format);
+        Integer vw = trackConfig.visibilityWindow;
+        if (vw != null) {
+            res.setVisibilityWindow(vw);
+        }
+        if(trackConfig.panelName != null) {
+            res.setPanelName(trackConfig.panelName);
+        }
+        if (trackConfig.searchTrix != null) {
+            res.setTrixURL(trackConfig.searchTrix);
+        }
+
+        res.setFeatureInfoURL(trackConfig.infoURL);
+        Boolean indexed = trackConfig.indexed;
+        if (indexed != null) {
+            res.setIndexed(indexed);
+        }
+
+        // Track properties
+        TrackProperties properties = new TrackProperties();
+        String color = trackConfig.color;
+        if (color != null) {
+            try {
+                properties.setColor(ColorUtilities.stringToColor(color.toString()));
+            } catch (Exception e) {
+                log.error("Error parsing color string: " + color, e);
+            }
+        }
+        String altColor = trackConfig.altColor;
+        if (altColor != null) {
+            try {
+                properties.setAltColor(ColorUtilities.stringToColor(altColor.toString()));
+            } catch (Exception e) {
+                log.error("Error parsing color string: " + altColor, e);
+            }
+        }
+        String displayMode = trackConfig.displayMode;
+        if (displayMode != null) {
+            try {
+                Track.DisplayMode dp = Track.DisplayMode.valueOf(stripQuotes(displayMode.toString()));
+                properties.setDisplayMode(dp);
+            } catch (Exception e) {
+                log.error("Error parsing displayMode " + displayMode, e);
+            }
+        }
+        Integer vizwindow = trackConfig.visibilityWindow;
+        if (vizwindow != null) {
+            properties.setFeatureVisibilityWindow(vizwindow);
+        } else {
+            // If not explicitly set, assume whole chromosome viz window for annotations
+            properties.setFeatureVisibilityWindow(-1);
+        }
+
+        if (trackConfig.min != null) {
+            properties.setMinValue(trackConfig.min);
+        }
+        if (trackConfig.max != null) {
+            properties.setMaxValue(trackConfig.max);
+        }
+        res.setTrackProperties(properties);
+
+        return res;
+
+    }
+
+    public String getPanelName() {
+        return panelName;
+    }
+
+    public void setPanelName(String panelName) {
+        this.panelName = panelName;
+    }
+
+
     /**
      * FOR LOAD FROM SERVER
      */
@@ -640,5 +743,5 @@ public class ResourceLocator {
     }
 
     static Set<String> knownFormats = new HashSet<>(Arrays.asList("gff", "bed", "gtf", "gff3",
-            "seg", "bb", "bigbed", "bigwig", "bam", "cram", "vcf"));
+            "seg", "bb", "bigbed", "bigwig", "bam", "cram", "vcf", "bedmethyl"));
 }
