@@ -86,11 +86,6 @@ public class BisulfiteBaseInfoTest {
             return 0;
         }
 
-        @Override
-        public ReadMate getMate() {
-            return null;
-        }
-
         @Override
         public Strand getReadStrand() {
             return null;
@@ -101,11 +96,6 @@ public class BisulfiteBaseInfoTest {
             return 0;
         }
 
-        @Override
-        public String getCigarString() {
-            return null;  //To change body of implemented methods use File | Settings | File Templates.
-        }
-
         @Override
         public List<Gap> getGaps() {
             return null;
@@ -160,11 +150,6 @@ public class BisulfiteBaseInfoTest {
             return null;  //To change body of implemented methods use File | Settings | File Templates.
         }
 
-        @Override
-        public Object getAttribute(String key) {
-            return null;
-        }
-
         @Override
         public void setMateSequence(String sequence) {
 
@@ -334,10 +319,5 @@ public class BisulfiteBaseInfoTest {
         public String getClipboardString(double location, int mouseX) {
             return null;
         }
-
-        @Override
-        public void finish() {
-
-        }
     }
 }
