@@ -29,7 +29,9 @@ import com.google.common.base.Function;
 import com.google.common.base.Supplier;
 import org.broad.igv.DirectoryManager;
 import org.broad.igv.Globals;
-import org.broad.igv.feature.NamedFeature;
+import org.broad.igv.feature.Chromosome;
+import org.broad.igv.feature.genome.ChromAliasDefaults;
+import org.broad.igv.feature.IGVNamedFeature;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.ui.commandbar.GenomeListManager;
 import org.broad.igv.prefs.PreferencesManager;
@@ -49,6 +51,7 @@ import java.lang.management.ThreadMXBean;
 import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.util.*;
+import java.util.List;
 
 import static junit.framework.Assert.assertTrue;
 import static org.junit.Assert.assertEquals;
@@ -64,15 +67,15 @@ public class TestUtils {
     // but highly unlikely to be repeated.
     public static final String TEST_UNIQUE_RUN_BASE = System.currentTimeMillis() + "_" + Math.random();
     private static final String TESTPREFS_PROPERTIES = "testprefs_" + TEST_UNIQUE_RUN_BASE + ".properties";
-    public static final String DATA_DIR = "test/data/";    
+    public static final String DATA_DIR = "test/data/";
     public static final String TMP_OUTPUT_DIR = DATA_DIR + "out/" + TEST_UNIQUE_RUN_BASE + "/";
-    
+
     static {
         File tmpOutputDir = new File(TMP_OUTPUT_DIR);
         tmpOutputDir.mkdirs();
         tmpOutputDir.deleteOnExit();
     }
-    
+
     public static final String defaultGenome = DATA_DIR + "genomes/hg18.unittest.genome";
 
     //This is so ant can set the large data directory
@@ -83,6 +86,68 @@ public class TestUtils {
         LARGE_DATA_DIR = System.getProperty(LARGE_DATA_DIR_KEY, LARGE_DATA_DIR);
     }
 
+    static Map<Integer, Integer> chromSizes;
+
+    static {
+        chromSizes = new HashMap<>();
+        chromSizes.put(1, 248956422);
+        chromSizes.put(2, 242193529);
+        chromSizes.put(3, 198295559);
+        chromSizes.put(4, 190214555);
+        chromSizes.put(5, 181538259);
+        chromSizes.put(6, 170805979);
+        chromSizes.put(7, 159345973);
+        chromSizes.put(8, 145138636);
+        chromSizes.put(9, 138394717);
+        chromSizes.put(10, 133797422);
+        chromSizes.put(11, 135086622);
+        chromSizes.put(12, 133275309);
+        chromSizes.put(13, 114364328);
+        chromSizes.put(14, 107043718);
+        chromSizes.put(15, 101991189);
+        chromSizes.put(16, 90338345);
+        chromSizes.put(17, 83257441);
+        chromSizes.put(18, 80373285);
+        chromSizes.put(19, 58617616);
+        chromSizes.put(20, 64444167);
+        chromSizes.put(21, 46709983);
+        chromSizes.put(22, 50818468);
+        chromSizes.put(23, 156040895);
+        chromSizes.put(24, 57227415);
+        chromSizes.put(25, 16569);
+    }
+
+    public static Genome mockUCSCGenome() {
+
+        List<Chromosome> chromosomeList = new ArrayList<>();
+        for (int i = 1; i < 23; i++) {
+            chromosomeList.add(new Chromosome(i, "chr" + i, chromSizes.get(i)));
+        }
+        chromosomeList.add(new Chromosome(23, "chrX", chromSizes.get(23)));
+        chromosomeList.add(new Chromosome(24, "chrY", chromSizes.get(24)));
+        chromosomeList.add(new Chromosome(25, "chrM", chromSizes.get(25)));
+
+        Genome mockGenome = new Genome("hg38", chromosomeList);
+        mockGenome.setChromAliasSource(new ChromAliasDefaults(mockGenome.getId(), mockGenome.getAllChromosomeNames()));
+
+        return mockGenome;
+    }
+
+    public static Genome mockNCBIGenome() {
+
+        List<Chromosome> chromosomeList = new ArrayList<>();
+        for (int i = 1; i < 25; i++) {
+            chromosomeList.add(new Chromosome(i, "chr" + i, chromSizes.get(i)));
+        }
+        chromosomeList.add(new Chromosome(25, "MT", chromSizes.get(25)));
+
+        Genome mockGenome = new Genome("GRCh38", chromosomeList);
+        mockGenome.setChromAliasSource(new ChromAliasDefaults(mockGenome.getId(), mockGenome.getAllChromosomeNames()));
+
+        return mockGenome;
+    }
+
+
     public static void setUpTestEnvironment() throws IOException {
         Globals.setTesting(true);
         //Globals.setBatch(true);
@@ -99,7 +164,7 @@ public class TestUtils {
         clearOutputDir();
     }
 
-    public static void resetPrefsFile(){
+    public static void resetPrefsFile() {
         File prefsFile = new File(TESTPREFS_PROPERTIES);
         prefsFile.delete();
         prefsFile.deleteOnExit();
@@ -161,9 +226,9 @@ public class TestUtils {
             for (File fi : listFiles) {
                 //Keep hidden files and directories
                 if (!fi.isHidden()) {
-                    if(fi.isFile()){
+                    if (fi.isFile()) {
                         fi.delete();
-                    }else if(fi.isDirectory()){
+                    } else if (fi.isDirectory()) {
                         FileUtils.deleteDir(fi);
                     }
                 }
@@ -178,7 +243,7 @@ public class TestUtils {
 //        tempDir.deleteOnExit();
 //        return tempDir;
 //    }
-    
+
     /**
      * Returns either 1 or 2, representing the number of
      * bytes used to end a line. Reads only from first line of a file
@@ -226,18 +291,19 @@ public class TestUtils {
 
     /**
      * Name matching is case-insensitive
+     *
      * @param exp
      * @param act
      */
-    public static void assertNamedFeaturesEqual(NamedFeature exp, NamedFeature act) {
+    public static void assertNamedFeaturesEqual(IGVNamedFeature exp, IGVNamedFeature act) {
         assertFeaturesEqual(exp, act);
         assertEquals(exp.getName().toUpperCase(), act.getName().toUpperCase());
     }
 
-    public static void assertTrackLoaded(IGV igv, String trackName){
+    public static void assertTrackLoaded(IGV igv, String trackName) {
         boolean found = false;
-        for(Track t: igv.getAllTracks()){
-            if(t.getName().equals(trackName)){
+        for (Track t : igv.getAllTracks()) {
+            if (t.getName().equals(trackName)) {
                 found = true;
                 break;
             }
@@ -246,12 +312,11 @@ public class TestUtils {
     }
 
     /**
-     *
      * @param featureIterator
      * @return Number of features in the iterator
      * @throws Exception
      */
-    public static int assertFeatureIteratorSorted(Iterator<? extends Feature> featureIterator){
+    public static int assertFeatureIteratorSorted(Iterator<? extends Feature> featureIterator) {
         int lastStart = -1;
         int count = 0;
         while (featureIterator.hasNext()) {
@@ -348,7 +413,7 @@ public class TestUtils {
      * @param nTrials
      * @param <T>
      * @return The runtime, in nanoseconds, of each call of predicate with input of supplier.
-     *         Array is sorted ascending
+     * Array is sorted ascending
      */
     public static <T> long[] timeMethod(Supplier<T> supplier, Function<T, Void> predicate, int nTrials) {
         long total = 0;
@@ -423,19 +488,18 @@ public class TestUtils {
 
             long endTime = System.nanoTime();
             benchmarkTime = endTime - startTime;
-            System.out.println("Benchmark Time (s): " + ((double) benchmarkTime)/1e9d);
+            System.out.println("Benchmark Time (s): " + ((double) benchmarkTime) / 1e9d);
         }
         return benchmarkTime;
     }
 
 
-
-    private static Object getField(Object object, Class clazz, String fieldName) throws Exception{
-        if(clazz == null) throw new NoSuchFieldException(fieldName + " not found all the way up");
+    private static Object getField(Object object, Class clazz, String fieldName) throws Exception {
+        if (clazz == null) throw new NoSuchFieldException(fieldName + " not found all the way up");
         Field field;
-        try{
+        try {
             field = object.getClass().getDeclaredField(fieldName);
-        }catch (NoSuchFieldException e){
+        } catch (NoSuchFieldException e) {
             return getField(object, clazz.getSuperclass(), fieldName);
         }
         field.setAccessible(true);
@@ -444,43 +508,47 @@ public class TestUtils {
 
     /**
      * Get the specified field, ignoring access restrictions
+     *
      * @param object
      * @param fieldName
      * @return
      */
-    public static Object getField(Object object, String fieldName) throws Exception{
+    public static Object getField(Object object, String fieldName) throws Exception {
         return getField(object, object.getClass(), fieldName);
     }
 
-    private static Object runMethod(Object object, Class clazz, String methodName, Object... args) throws Exception{
-        if(clazz == null) throw new NoSuchFieldException(methodName + " not found all the way up");
+    private static Object runMethod(Object object, Class clazz, String methodName, Object... args) throws Exception {
+        if (clazz == null) throw new NoSuchFieldException(methodName + " not found all the way up");
         Method method;
-        try{
+        try {
             method = object.getClass().getDeclaredMethod(methodName);
-        }catch (NoSuchMethodException e){
+        } catch (NoSuchMethodException e) {
             return runMethod(object, clazz.getSuperclass(), methodName, args);
         }
         method.setAccessible(true);
         return method.invoke(object, args);
     }
 
-    public static Object runMethod(Object object, String methodName, Object... args) throws Exception{
+    public static Object runMethod(Object object, String methodName, Object... args) throws Exception {
         return runMethod(object, object.getClass(), methodName, args);
     }
 
 
     private static Map<String, String> replaceMap = new HashMap<String, String>(2);
-    static{
+
+    static {
         replaceMap.put("${DATA_DIR}", TestUtils.DATA_DIR);
         replaceMap.put("${LARGE_DATA_DIR}", TestUtils.LARGE_DATA_DIR);
     }
+
     /**
      * Mainly for session files, we want use some test paths stored in otherwise hardcoded
      * test data files. This re-writes the input text file
+     *
      * @param inputPath
      * @return File of the output location
      */
-    public static File replaceTestPaths(File inputPath) throws Exception{
+    public static File replaceTestPaths(File inputPath) throws Exception {
 
         BufferedReader reader = new BufferedReader(new FileReader(inputPath));
 
@@ -490,8 +558,8 @@ public class TestUtils {
         PrintWriter writer = new PrintWriter(new FileWriter(outputFile));
 
         String line;
-        while((line = reader.readLine()) != null){
-            for(Map.Entry<String, String> entry: replaceMap.entrySet()){
+        while ((line = reader.readLine()) != null) {
+            for (Map.Entry<String, String> entry : replaceMap.entrySet()) {
                 line = line.replace(entry.getKey(), entry.getValue());
             }
             writer.println(line);
@@ -501,7 +569,7 @@ public class TestUtils {
         return outputFile;
     }
 
-    public static void resetTestUserDefinedGenomes() throws IOException{
+    public static void resetTestUserDefinedGenomes() throws IOException {
         File userDefinedGenomeListFile = new File(DirectoryManager.getGenomeCacheDirectory(), GenomeListManager.TEST_USER_DEFINED_GENOME_LIST_FILE);
         userDefinedGenomeListFile.delete();
         userDefinedGenomeListFile.deleteOnExit();
